Package: bpgmm
Type: Package
Title: Bayesian Model Selection Approach for Parsimonious Gaussian
        Mixture Models
Version: 1.3.1
Date: 2026-05-26
Depends: R(>= 3.1.0)
Imports: methods (>= 3.5.1), mcmcse (>= 1.3-2), pgmm (>= 1.2.3),
        mvtnorm (>= 1.0-10), MASS (>= 7.3-51.1), parallel, Rcpp (>=
        1.0.1), gtools (>= 3.8.1), label.switching (>= 1.8), fabMix (>=
        5.0), mclust (>= 5.4.3)
Authors@R: c(
    person(given = "Yaoxiang", family = "Li",  role = c("aut","cre"),email = "liyaoxiang@outlook.com"),
    person(given = "Xiang",  family = "Lu",    role = "aut"),
    person(given = "Tanzy",  family = "Love",  role = "aut"))
Author: Yaoxiang Li [aut, cre],
  Xiang Lu [aut],
  Tanzy Love [aut]
Maintainer: Yaoxiang Li <liyaoxiang@outlook.com>
Description: Model-based clustering using Bayesian parsimonious Gaussian mixture models.
  MCMC (Markov chain Monte Carlo) are used for parameter estimation. The RJMCMC (Reversible-jump Markov chain Monte Carlo) is used for model selection. 
  GREEN et al. (1995) <doi:10.1093/biomet/82.4.711>.
License: GPL-3
URL: https://github.com/YaoxiangLi/bpgmm,
        https://yaoxiangli.github.io/bpgmm/,
        https://doi.org/10.1007/s00357-021-09391-8
BugReports: https://github.com/YaoxiangLi/bpgmm/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-28 03:48:04 UTC; Li
Repository: CRAN
Date/Publication: 2026-05-28 07:10:17 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-28 11:39:40 UTC; unix
Archs: bpgmm.so.dSYM

# Generated by roxygen2: do not edit by hand

export(constraint_to_model)
export(model_to_constraint)
export(pgmm_rjmcmc)
export(pgmm_rjmcmc_chains)
export(summarize_pgmm_rjmcmc)
import(MASS)
import(fabMix)
import(label.switching)
import(mcmcse)
import(methods)
import(pgmm)
import(stats)
importFrom(Rcpp,sourceCpp)
importFrom(gtools,rdirichlet)
importFrom(mclust,adjustedRandIndex)
importFrom(mvtnorm,rmvnorm)
useDynLib(bpgmm, .registration = TRUE)

# bpgmm: Bayesian inference for parsimonious Gaussian mixture models

[![CRAN status](https://www.r-pkg.org/badges/version/bpgmm)](https://cran.r-project.org/package=bpgmm)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/bpgmm)](https://cran.r-project.org/package=bpgmm)
[![R-CMD-check](https://github.com/YaoxiangLi/bpgmm/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/YaoxiangLi/bpgmm/actions/workflows/R-CMD-check.yaml)
[![GitHub release](https://img.shields.io/github/v/release/YaoxiangLi/bpgmm?display_name=tag&sort=semver)](https://github.com/YaoxiangLi/bpgmm/releases)
[![pkgdown](https://github.com/YaoxiangLi/bpgmm/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/YaoxiangLi/bpgmm/actions/workflows/pkgdown.yaml)
[![License: GPL-3](https://img.shields.io/badge/license-GPL--3-blue.svg)](https://cran.r-project.org/web/licenses/GPL-3)
[![DOI](https://img.shields.io/badge/DOI-10.1007%2Fs00357--021--09391--8-blue)](https://doi.org/10.1007/s00357-021-09391-8)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)

`bpgmm` implements Bayesian inference for parsimonious Gaussian mixture
models. It is used for model-based clustering when the number of clusters, the
object partition, and the cluster covariance structure are all inferential
targets.

The package uses Markov chain Monte Carlo for posterior estimation and
reversible-jump MCMC (RJMCMC) for model selection across constrained mixtures of
factor analyzers.

## Installation

Install the released version from CRAN:

```r
install.packages("bpgmm")
```

Load the package:

```r
library(bpgmm)
```

## Inferential Targets

The package supports analyses in which the inferential targets include:

- estimate the number of clusters from the data;
- infer posterior cluster membership probabilities;
- compare parsimonious covariance structures;
- fit mixture-of-factor-analyzers models for high-dimensional clustering;
- use RJMCMC to move across models with different parameter dimensions.

## Minimal Runnable Example

The example below creates two small clusters, fits a short RJMCMC chain, and
summarizes the posterior samples. Applied analyses should use a longer burn-in
and more posterior samples.

```r
set.seed(2026)

X <- cbind(
  matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
  matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
)
known_labels <- rep(1:2, each = 4)

fit <- pgmm_rjmcmc(
  X = X,
  m_init = 2,
  m_range = c(1, 3),
  q_new = 1,
  burn = 1,
  niter = 3,
  constraint = model_to_constraint("UUU"),
  m_step = 0,
  v_step = 0,
  verbose = FALSE
)

fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)
as.integer(fit_summary$n_clusters["2"])
#> [1] 3

as.integer(fit_summary$n_constraints["UUU"])
#> [1] 3

fit_summary$ari
#> [1] 1
```

In this call, `X` is a numeric matrix with variables in rows and observations
in columns. Set `m_step = 1` to allow RJMCMC updates for the number of clusters
and `v_step = 1` to allow updates for the variance structure.

The main user-facing function is `pgmm_rjmcmc()`. Important settings include:

- `m_init`: initial number of clusters.
- `m_range`: allowed cluster range, such as `c(1, 6)`.
- `q_new`: number of latent factors for a newly proposed cluster.
- `burn` and `niter`: burn-in and posterior sampling iterations.
- `constraint`: initial covariance model, usually set with
  `model_to_constraint()`.
- `m_step`, `v_step`, and `split_combine`: switches for cluster-number,
  covariance-model, and split/combine RJMCMC moves.
- `verbose`: set to `FALSE` to suppress per-iteration progress output.

## Multi-Core Runs

Individual RJMCMC iterations are sequential because each state depends on the
previous state. To use multiple cores safely, run independent chains in
parallel:

```r
fits <- pgmm_rjmcmc_chains(
  X = X,
  m_init = 2,
  m_range = c(1, 3),
  q_new = 1,
  burn = 100,
  niter = 1000,
  chains = 4,
  cores = 4,
  seed = 2026,
  verbose = FALSE
)

length(fits)
#> [1] 4
```

This uses separate worker processes where available and stores deterministic
per-chain seeds in `attr(fits, "chain_seeds")`.

Starting with version 1.2.0, the public API uses snake_case names throughout.
Older camelCase function names and argument names are no longer exported.

The eight covariance structures in the paper are represented by model labels
such as `"CCC"`, `"CUU"`, and `"UUU"`. The package also accepts the legacy
three-number constraint encoding:

```r
model_to_constraint("UUU")
constraint_to_model(c(1, 0, 0))
```

## Paper

The methodology behind this package is described in:

> Lu, X., Li, Y., & Love, T. (2021). On Bayesian Analysis of Parsimonious
> Gaussian Mixture Models. *Journal of Classification*, 38, 576-593.
> https://doi.org/10.1007/s00357-021-09391-8

The paper develops an RJMCMC inferential procedure for constrained
mixture-of-factor-analyzers models. The inferential goals are the partition of
observations, the number of clusters, and the covariance structure of the
clusters, each represented through posterior distributions.

## Citation

If you use `bpgmm` in published work, please cite both the package and the
methodology paper. In R, run:

```r
citation("bpgmm")
```

BibTeX for the paper:

```bibtex
@article{lu2021bayesian,
  author = {Lu, Xiang and Li, Yaoxiang and Love, Tanzy},
  title = {On Bayesian Analysis of Parsimonious Gaussian Mixture Models},
  journal = {Journal of Classification},
  year = {2021},
  volume = {38},
  pages = {576--593},
  doi = {10.1007/s00357-021-09391-8}
}
```

## License

`bpgmm` is released under the GPL-3 license.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----fig.width = 5.5, fig.height = 4.5, fig.alt = "Two covariance ellipses illustrating cluster-specific covariance matrices."----
ellipse_points <- function(center, sigma, level = 0.90, n = 120) {
  eig <- eigen(sigma, symmetric = TRUE)
  angles <- seq(0, 2 * pi, length.out = n)
  circle <- rbind(cos(angles), sin(angles))
  radius <- sqrt(qchisq(level, df = 2))
  points <- t(center + radius * eig$vectors %*% diag(sqrt(eig$values), 2) %*% circle)
  colnames(points) <- c("x", "y")
  points
}

lambda_1 <- matrix(c(1.1, 0.4), ncol = 1)
lambda_2 <- matrix(c(0.2, 1.0), ncol = 1)
psi_1 <- diag(c(0.20, 0.08))
psi_2 <- diag(c(0.10, 0.25))
sigma_1 <- lambda_1 %*% t(lambda_1) + psi_1
sigma_2 <- lambda_2 %*% t(lambda_2) + psi_2

ell_1 <- ellipse_points(c(-1.2, -0.5), sigma_1)
ell_2 <- ellipse_points(c(1.1, 0.6), sigma_2)

plot(
  ell_1,
  type = "l",
  lwd = 2,
  col = "#0072B2",
  xlim = c(-3.5, 3.5),
  ylim = c(-2.5, 3),
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Mixture-of-factor-analyzers covariance"
)
lines(ell_2, lwd = 2, col = "#D55E00")
points(rbind(c(-1.2, -0.5), c(1.1, 0.6)), pch = 19, col = c("#0072B2", "#D55E00"))
arrows(-1.2, -0.5, -1.2 + lambda_1[1], -0.5 + lambda_1[2], col = "#0072B2", lwd = 2, length = 0.08)
arrows(1.1, 0.6, 1.1 + lambda_2[1], 0.6 + lambda_2[2], col = "#D55E00", lwd = 2, length = 0.08)
legend(
  "topleft",
  legend = c("Cluster 1 covariance", "Cluster 2 covariance", "Loading direction"),
  col = c("#0072B2", "#D55E00", "gray30"),
  lwd = c(2, 2, 2),
  bty = "n"
)

## -----------------------------------------------------------------------------
library(bpgmm)

models <- c("CCC", "CCU", "CUC", "CUU", "UCC", "UCU", "UUC", "UUU")
data.frame(
  model = models,
  constraint = vapply(
    models,
    function(x) paste(model_to_constraint(x), collapse = ","),
    character(1)
  )
)

## -----------------------------------------------------------------------------
model_to_constraint("UUU")
constraint_to_model(c(1, 1, 0))

## ----eval = FALSE-------------------------------------------------------------
# fit <- pgmm_rjmcmc(
#   X = your_data_matrix,
#   m_init = 5,
#   m_range = c(1, 10),
#   q_new = 4,
#   burn = 5000,
#   niter = 15000,
#   constraint = model_to_constraint("UUU"),
#   m_step = 1,
#   v_step = 1,
#   split_combine = 1,
#   verbose = FALSE
# )

## -----------------------------------------------------------------------------
citation("bpgmm")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(bpgmm)

set.seed(2026)

X <- cbind(
  matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
  matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
)
known_labels <- rep(1:2, each = 4)

## ----fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of the small two-cluster example."----
plot(
  X[1, ], X[2, ],
  col = c("#0072B2", "#D55E00")[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Small two-cluster example",
  asp = 1
)
legend(
  "topleft",
  legend = paste("Reference", sort(unique(known_labels))),
  col = c("#0072B2", "#D55E00"),
  pch = 19,
  bty = "n"
)

## -----------------------------------------------------------------------------
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 1,
    niter = 3,
    constraint = model_to_constraint("UUU"),
    m_step = 0,
    v_step = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)

## -----------------------------------------------------------------------------
summary <- summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)

summary$allocation
summary$n_clusters
summary$n_constraints
summary$ari

## ----fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot colored by posterior modal allocation."----
plot(
  X[1, ], X[2, ],
  col = c("#009E73", "#CC79A7", "#E69F00")[summary$allocation],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Posterior modal allocation",
  asp = 1
)
text(X[1, ], X[2, ], labels = seq_along(summary$allocation), pos = 3, cex = 0.75)
legend(
  "topleft",
  legend = paste("Cluster", sort(unique(summary$allocation))),
  col = c("#009E73", "#CC79A7", "#E69F00")[sort(unique(summary$allocation))],
  pch = 19,
  bty = "n"
)

## -----------------------------------------------------------------------------
citation("bpgmm")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(bpgmm)

simulate_mfa_data <- function(n_per_cluster = 20, p = 6, q = 2) {
  means <- rbind(
    c(-3.0, -2.0,  0.0, 0, 0, 0),
    c( 2.5, -1.0,  2.0, 0, 0, 0),
    c( 0.0,  2.5, -2.0, 0, 0, 0)
  )

  lambdas <- list(
    matrix(c(1.2,  0.5, 0.2, 0, 0, 0,
             0.0,  0.2, 0.8, 0.3, 0, 0), nrow = p),
    matrix(c(0.2,  1.1, 0.6, 0, 0, 0,
             0.8,  0.1, 0.2, 0.6, 0, 0), nrow = p),
    matrix(c(0.7, -0.4, 1.0, 0, 0, 0,
            -0.2,  0.8, 0.4, 0.3, 0, 0), nrow = p)
  )

  psi <- list(
    diag(c(0.25, 0.35, 0.30, 0.80, 0.90, 1.00)),
    diag(c(0.30, 0.25, 0.40, 0.80, 0.90, 1.00)),
    diag(c(0.35, 0.30, 0.25, 0.80, 0.90, 1.00))
  )

  n <- n_per_cluster * 3
  X <- matrix(NA_real_, nrow = p, ncol = n)
  true_cluster <- rep(seq_len(3), each = n_per_cluster)

  column <- 1
  for (k in seq_len(3)) {
    for (i in seq_len(n_per_cluster)) {
      latent_score <- rnorm(q)
      noise <- MASS::mvrnorm(1, mu = rep(0, p), Sigma = psi[[k]])
      X[, column] <- means[k, ] + lambdas[[k]] %*% latent_score + noise
      column <- column + 1
    }
  }

  rownames(X) <- paste0("variable_", seq_len(p))
  list(X = X, true_cluster = true_cluster)
}

set.seed(2027)
sim <- simulate_mfa_data()
X <- sim$X
true_cluster <- sim$true_cluster

dim(X)
table(true_cluster)

## ----fig.width = 5.5, fig.height = 4.2, fig.alt = "Scatter plot of the first two variables colored by true cluster."----
cluster_cols <- c("#0072B2", "#D55E00", "#009E73", "#CC79A7", "#E69F00")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[true_cluster],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "Simulated MFA data",
  asp = 1
)
legend(
  "topleft",
  legend = paste("True cluster", sort(unique(true_cluster))),
  col = cluster_cols[sort(unique(true_cluster))],
  pch = 19,
  bty = "n"
)

## -----------------------------------------------------------------------------
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 3,
    m_range = c(1, 5),
    q_new = 2,
    burn = 2,
    niter = 6,
    constraint = "UUU",
    m_step = 1,
    v_step = 1,
    split_combine = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)

## -----------------------------------------------------------------------------
fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = true_cluster)

fit_summary$n_clusters
fit_summary$n_constraints
fit_summary$ari

## ----fig.width = 7, fig.height = 4, fig.alt = "Bar plots of posterior counts for cluster number and covariance model."----
old_par <- par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))
barplot(
  fit_summary$n_clusters,
  col = "#56B4E9",
  border = NA,
  xlab = "Number of clusters",
  ylab = "Posterior sample count",
  main = "Cluster-number selection"
)
barplot(
  fit_summary$n_constraints,
  col = "#E69F00",
  border = NA,
  xlab = "PGMM model",
  ylab = "Posterior sample count",
  main = "Covariance-model selection"
)
par(old_par)

## ----fig.width = 7, fig.height = 4, fig.alt = "Two-panel scatter plot comparing true cluster labels and posterior modal allocation."----
old_par <- par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
plot(
  X[1, ], X[2, ],
  col = cluster_cols[true_cluster],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "True clusters",
  asp = 1
)
plot(
  X[1, ], X[2, ],
  col = cluster_cols[fit_summary$allocation],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "Posterior modal allocation",
  asp = 1
)
par(old_par)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(bpgmm)

simulate_screening_data <- function(n_per_cluster = 18, p = 6) {
  means <- rbind(
    c(-2.5, -1.5,  0.0, 0, 0, 0),
    c( 2.0, -0.5,  1.8, 0, 0, 0),
    c( 0.0,  2.2, -1.8, 0, 0, 0)
  )

  covariances <- list(
    diag(c(0.35, 0.35, 0.35, 1.40, 0.35, 1.20)),
    diag(c(0.35, 0.35, 0.35, 0.35, 1.40, 1.20)),
    matrix(c(
      0.35, 0,    0,    0,    0,    0,
      0,    0.35, 0,    0,    0,    0,
      0,    0,    0.35, 0,    0,    0,
      0,    0,    0,    1.00, 0.45, 0,
      0,    0,    0,    0.45, 1.00, 0,
      0,    0,    0,    0,    0,    1.20
    ), nrow = p)
  )

  n <- n_per_cluster * 3
  X <- matrix(NA_real_, nrow = p, ncol = n)
  true_cluster <- rep(seq_len(3), each = n_per_cluster)

  column <- 1
  for (k in seq_len(3)) {
    for (i in seq_len(n_per_cluster)) {
      X[, column] <- MASS::mvrnorm(1, mu = means[k, ], Sigma = covariances[[k]])
      column <- column + 1
    }
  }

  rownames(X) <- paste0("variable_", seq_len(p))
  list(X = X, true_cluster = true_cluster)
}

set.seed(2028)
sim <- simulate_screening_data()
X <- sim$X
true_cluster <- sim$true_cluster

## -----------------------------------------------------------------------------
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 3,
    m_range = c(1, 5),
    q_new = 2,
    burn = 2,
    niter = 6,
    constraint = "UUU",
    m_step = 1,
    v_step = 1,
    verbose = FALSE
  )
})
tail(fit_log, 1)

fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = true_cluster)
fit_summary$ari

## -----------------------------------------------------------------------------
cluster_separation <- function(X, allocation) {
  vapply(seq_len(nrow(X)), function(j) {
    x <- X[j, ]
    overall <- mean(x)
    total <- sum((x - overall)^2)
    between <- sum(vapply(split(x, allocation), function(group) {
      length(group) * (mean(group) - overall)^2
    }, numeric(1)))
    if (total == 0) 0 else between / total
  }, numeric(1))
}

separation <- cluster_separation(X, fit_summary$allocation)
separation_table <- data.frame(
  variable = rownames(X),
  separation = round(separation, 3)
)
separation_table[order(separation_table$separation, decreasing = TRUE), ]

## ----fig.width = 6.5, fig.height = 4, fig.alt = "Bar plot of exploratory variable separation scores."----
ordered <- order(separation, decreasing = TRUE)
barplot(
  separation[ordered],
  names.arg = rownames(X)[ordered],
  las = 2,
  col = "#56B4E9",
  border = NA,
  ylab = "Between-cluster variation / total variation",
  main = "Exploratory variable separation"
)

## -----------------------------------------------------------------------------
loading_importance <- function(fit) {
  totals <- NULL
  count <- 0

  for (sample_id in seq_along(fit$lambda_samples)) {
    lambdas <- fit$lambda_samples[[sample_id]]
    active <- which(fit$active_cluster_samples[[sample_id]] == 1)

    for (k in active) {
      lambda <- lambdas[[k]]
      if (!is.matrix(lambda)) {
        next
      }
      score <- rowMeans(abs(lambda))
      if (is.null(totals)) {
        totals <- numeric(length(score))
      }
      totals <- totals + score
      count <- count + 1
    }
  }

  if (count == 0) {
    return(rep(NA_real_, nrow(X)))
  }
  totals / count
}

loading_score <- loading_importance(fit)
loading_table <- data.frame(
  variable = rownames(X),
  mean_abs_loading = round(loading_score, 3)
)
loading_table[order(loading_table$mean_abs_loading, decreasing = TRUE), ]

## ----fig.width = 6.5, fig.height = 4, fig.alt = "Bar plot of average posterior absolute loading magnitudes by variable."----
ordered_loading <- order(loading_score, decreasing = TRUE)
barplot(
  loading_score[ordered_loading],
  names.arg = rownames(X)[ordered_loading],
  las = 2,
  col = "#E69F00",
  border = NA,
  ylab = "Mean absolute loading",
  main = "Posterior loading magnitude"
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(bpgmm)

set.seed(2026)

X <- cbind(
  matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
  matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
)
known_labels <- rep(1:2, each = 4)

dim(X)
known_labels

## ----fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of simulated observations colored by known cluster."----
cluster_cols <- c("#0072B2", "#D55E00", "#009E73")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Simulated data",
  asp = 1
)
legend(
  "topleft",
  legend = paste("Known", sort(unique(known_labels))),
  col = cluster_cols[sort(unique(known_labels))],
  pch = 19,
  bty = "n"
)

## -----------------------------------------------------------------------------
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 1,
    niter = 3,
    constraint = "UUU",
    m_step = 0,
    v_step = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)

## -----------------------------------------------------------------------------
names(fit)
length(fit$allocation_samples)

## -----------------------------------------------------------------------------
fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)

fit_summary$allocation
fit_summary$n_clusters
fit_summary$n_constraints
fit_summary$ari

## ----fig.width = 7, fig.height = 4, fig.alt = "Two-panel plot comparing known labels and posterior allocation."----
old_par <- par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Known labels",
  asp = 1
)
plot(
  X[1, ], X[2, ],
  col = cluster_cols[fit_summary$allocation],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Posterior allocation",
  asp = 1
)
par(old_par)

## -----------------------------------------------------------------------------
last <- length(fit$tau_samples)

fit$tau_samples[[last]]
fit$mean_samples[[last]]
lapply(fit$lambda_samples[[last]], dim)
lapply(fit$psi_samples[[last]], dim)

## ----eval = FALSE-------------------------------------------------------------
# fit <- pgmm_rjmcmc(
#   X = your_data_matrix,
#   m_init = 2,
#   m_range = c(1, 10),
#   q_new = 4,
#   burn = 5000,
#   niter = 15000,
#   constraint = model_to_constraint("UUU"),
#   m_step = 1,
#   v_step = 1,
#   split_combine = 1,
#   verbose = FALSE
# )
# 
# summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(bpgmm)

set.seed(2029)
X <- cbind(
  matrix(rnorm(10, mean = -2.0, sd = 0.25), nrow = 2),
  matrix(rnorm(10, mean =  0.0, sd = 0.25), nrow = 2),
  matrix(rnorm(10, mean =  2.0, sd = 0.25), nrow = 2)
)
known_labels <- rep(1:3, each = 5)

## ----fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of a compact three-cluster diagnostic data set."----
cluster_cols <- c("#0072B2", "#D55E00", "#009E73", "#CC79A7")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Diagnostic example",
  asp = 1
)

## -----------------------------------------------------------------------------
fits <- pgmm_rjmcmc_chains(
  X = X,
  m_init = 3,
  m_range = c(1, 4),
  q_new = 1,
  burn = 1,
  niter = 4,
  constraint = "UUU",
  m_step = 1,
  v_step = 1,
  chains = 2,
  cores = 1,
  seed = 2029,
  verbose = FALSE
)

length(fits)
attr(fits, "chain_seeds")

## -----------------------------------------------------------------------------
chain_summaries <- lapply(fits, summarize_pgmm_rjmcmc, true_cluster = known_labels)

data.frame(
  chain = names(chain_summaries),
  ari = vapply(chain_summaries, function(x) x$ari, numeric(1)),
  modal_clusters = vapply(chain_summaries, function(x) {
    as.integer(names(which.max(x$n_clusters)))
  }, integer(1))
)

## -----------------------------------------------------------------------------
cluster_count_trace <- function(fit) {
  vapply(fit$active_cluster_samples, sum, numeric(1))
}

constraint_trace <- function(fit) {
  vapply(fit$constraint_samples, constraint_to_model, character(1))
}

cluster_traces <- lapply(fits, cluster_count_trace)
constraint_traces <- lapply(fits, constraint_trace)

cluster_traces
constraint_traces

## ----fig.width = 7, fig.height = 4, fig.alt = "Trace plot of sampled cluster counts across two short chains."----
old_par <- par(mar = c(4, 4, 3, 1))
plot(
  cluster_traces[[1]],
  type = "b",
  pch = 19,
  ylim = range(unlist(cluster_traces)),
  col = "#0072B2",
  xlab = "Saved iteration",
  ylab = "Active clusters",
  main = "Cluster-count trace"
)
lines(cluster_traces[[2]], type = "b", pch = 19, col = "#D55E00")
legend("topright", legend = names(fits), col = c("#0072B2", "#D55E00"), lty = 1, pch = 19, bty = "n")
par(old_par)

## -----------------------------------------------------------------------------
co_clustering_matrix <- function(fit) {
  n <- length(fit$allocation_samples[[1]])
  out <- matrix(0, n, n)

  for (allocation in fit$allocation_samples) {
    out <- out + outer(allocation, allocation, "==")
  }

  out / length(fit$allocation_samples)
}

co_mat <- co_clustering_matrix(fits[[1]])
round(co_mat[1:6, 1:6], 2)

## ----fig.width = 5.5, fig.height = 5, fig.alt = "Heatmap of posterior co-clustering probabilities."----
image(
  seq_len(nrow(co_mat)),
  seq_len(ncol(co_mat)),
  co_mat[nrow(co_mat):1, ],
  col = hcl.colors(20, "YlGnBu", rev = TRUE),
  xlab = "Observation",
  ylab = "Observation",
  main = "Posterior co-clustering"
)

# set.seed(111)
#
# source("./R/sourceR.R")
#
#
# n <- 500 # sample size
# p <- 10 # number of variables
# q <- 4 # number of factors
# K <- 10 # true number of clusters
# # sINV_diag =  # diagonal of inverse variance of errors
#
# syntheticDataset <- simData(
#   sameLambda = TRUE, sameSigma = TRUE, K.true = K, n = n, q = q, p = p,
#   sINV_values = 1 / ((1:p))
# )
#
# ##
# nsim <- 10
# burn <- 20
#
# X <- t(syntheticDataset$data)
#
# # delta = 2; ggamma = 2
# # d_vec = c(1,1,1)
# # s_vec = c(1,1,1)
#
# q_new <- 4
#
# m_step <- 1
# v_step <- 1
# constraint <- c(0, 0, 0)
#
# m_init <- 20
# m_range <- c(1, 20)
#
# res <- pgmm_rjmcmc(X, m_init, m_range, q_new,
#   niter = nsim, burn = burn, constraint = constraint,
#   m_step = m_step, v_step = v_step
# )
#
# summarize_pgmm_rjmcmc(res, true_cluster = syntheticDataset$class)

# # R pkg
#
# library("tidyverse")
# library("fabMix")
# library("mclust")
# library("pgmm")
# library("mvtnorm")
# library("mcmcse")
# library("MASS")
# library("gtools")
# library("RcppArmadillo")
# library("Rcpp")
#
# # R source
#
# source("R/summerizeRes.R")
# source("R/choosem/split.R")
# source("R/choosem/combine.R")
# source("R/pgmmStay.R")
# source("R/pgmmMstep.R")
# source("R/pgmmVstep.R")
# source("R/PGMMmain.R")
# source("R/prior.R")
# source("R/update_post_theta_y.R")
# source("R/utils.R")
# source("R/posteriorZ.R")
# source("R/proposalLambda.R")
# source("R/proposalPsi.R")
# source("R/hyperParamClass.R")
# source("R/paramClass.R")
# source("R/choosem/proposalFunctions.R")
#
# # C++ source
#
# sourceCpp("src/utils.cpp")
# sourceCpp("src/update_post_z.cpp")
# sourceCpp("src/calculate_cxy.cpp")
# sourceCpp("src/calculate_post_lambda_psi.cpp")
# sourceCpp("src/update_latent_scores.cpp")
# sourceCpp("src/update_hyperparameter.cpp")
#
#
#
#
#

#' (internal)
#' @noRd
.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    paste0(
      "bpgmm ",
      utils::packageVersion("bpgmm"),
      " loaded. If you use bpgmm in published work, please cite it with citation(\"bpgmm\")."
    )
  )
}

#' generate a new extra cluster Theta and Y
#'
#' @param m m
#' @param n n
#' @param p p
#' @param hparam hparam
#' @param muBar muBar
#' @param qnew qnew
#' @param thetaYList thetaYList
#' @param constraint constraint
#' @param clusInd clusInd
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' qnew <- 1
#' clusInd <- rep(1, m)
#' generate_new_cluster_parameters(m, n, p, hparam, muBar, qnew, thetaYList, constraint, clusInd)
#' @noRd
generate_new_cluster_parameters <- function(m, n, p, hparam, muBar, qnew, thetaYList, constraint, clusInd) {
  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2


  lambdaPsyList <- get_lambda_psi_for_new_cluster(m, p, qnew, thetaYList, hparam, constraint, clusInd)

  lambda <- lambdaPsyList$lambda
  psy <- lambdaPsyList$psy

  ## proposal tao
  tao <- rbeta(1, shape1 = 1, shape2 = m)
  # tao = rbeta(1, shape1 = 1, shape2 = 100)
  # taoDval = log(dbeta(tao, shape1 = 1, shape2 = m))

  ## proposal M
  Mvar <- 1 / (alpha1) * psy ## prior
  M <- t(c(mvtnorm::rmvnorm(1, muBar, Mvar)))
  # MDval = mvtnorm::dmvnorm(M[[1]], muBar, Mvar, log = T)


  ## proposal Y
  # YvecDval = c()
  Y <- matrix(NA, qnew, n)
  for (i in 1:n) {
    Y[, i] <- mvtnorm::rmvnorm(1, mean = rep(0, qnew), sigma = diag(qnew))
    # YvecDval[i] = mvtnorm::dmvnorm(Y[[1]][,i], mean = rep(0,qNew), sigma = diag(qNew), log = T)
  }
  # YDval = sum(YvecDval)
  # ThetaYGenDval = sum(taoDval, psyDval, MDval, lambdaDval, YDval)
  # ThetaYGenDval = sum(psyDval, MDval, lambdaDval, YDval)


  new("ThetaYList",
    tao = tao,
    psy = list(psy),
    M = list(M),
    lambda = list(lambda),
    Y = list(Y)
  )
}


#' get_lambda_psi_for_new_cluster
#'
#' @param m m
#' @param p p
#' @param qnew qnew
#' @param thetaYList thetaYList
#' @param hparam hparam
#' @param constraint constraint
#' @param clusInd clusInd
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' qnew <- 1
#' clusInd <- rep(1, m)
#' get_lambda_psi_for_new_cluster(
#'   m,
#'   p,
#'   qnew,
#'   thetaYList,
#'   hparam,
#'   constraint,
#'   clusInd
#' )
#' @noRd
get_lambda_psi_for_new_cluster <- function(m,
                                   p,
                                   qnew,
                                   thetaYList,
                                   hparam,
                                   constraint,
                                   clusInd) {
  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  ## psy
  psy <- c()
  if (constraint[2] == T) {
    psy <- thetaYList@psy[[which(clusInd == 1)[1]]]
  } else if (constraint[2] == F & constraint[3] == T) {
    psyValue <- 1 / rgamma(1, shape = hparam@delta, rate = hparam@bbeta)
    psy <- diag(rep(psyValue, p))
  } else if (constraint[2] == F & constraint[3] == F) {
    psy <- diag(1 / rgamma(p, shape = hparam@delta, rate = hparam@bbeta))
  }


  ## lambda
  lambda <- c()
  if (constraint[1] == T) {
    lambda <- thetaYList@lambda[[which(clusInd == 1)[1]]]
  } else if (constraint[1] == F) {
    lambda <- matrix(0, p, qnew)
    for (j in 1:qnew) {
      lambda[, j] <- mvtnorm::rmvnorm(1, rep(0, p), 1 / alpha2 * psy)
    }
  }
  return(list(psy = psy, lambda = lambda))
}


## evaluate a new extra cluster Theta and Y
#'
#' @param m m
#' @param p p
#' @param hparam hparam
#' @param muBar muBar
#' @param newClusterThetaY newClusterThetaY
#' @param constraint constraint
#'
#' @noRd
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' qnew <- 1
#' clusInd <- rep(1, m)
#' \donttest{
#' evaluate_new_cluster_parameters(m, p, hparam, muBar, thetaYList, constraint)
#' }
#'
#' @noRd
evaluate_new_cluster_parameters <- function(m, p, hparam, muBar, qnew, thetaYList, newClusterThetaY, constraint) {
  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  ## eval lambda psy
  lambdaPsyEval <- evaluate_lambda_psi_for_new_cluster(m, p, qnew, thetaYList, newClusterThetaY, hparam, constraint)

  ## eval tao
  tao <- newClusterThetaY@tao
  taoDval <- log(dbeta(tao, shape1 = 1, shape2 = m))

  # taoDval = log(dbeta(tao, shape1 = 1, shape2 = 100))

  # hist(rbeta(10000, shape1 = 1, shape2 = 100))
  ## test
  # taoDval = 0
  ##

  ## eval M
  M <- newClusterThetaY@M[[1]]
  MDval <- mvtnorm::dmvnorm(M, muBar, 1 / alpha1 * newClusterThetaY@psy[[1]], log = T)

  return(sum(taoDval, MDval, lambdaPsyEval$lambdaEval, lambdaPsyEval$psyEval))
}




#' evaluate_lambda_psi_for_new_cluster
#'
#' @param m m
#' @param p p
#' @param qnew qnew
#' @param thetaYList thetaYList
#' @param newClusterThetaY newClusterThetaY
#' @param hparam hparam
#' @param constraint constraint
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' newClusterThetaY <- thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' qnew <- 1
#' clusInd <- rep(1, m)
#' evaluate_lambda_psi_for_new_cluster(
#'   m,
#'   p,
#'   qnew,
#'   thetaYList,
#'   newClusterThetaY,
#'   hparam,
#'   constraint
#' )
#' @noRd
evaluate_lambda_psi_for_new_cluster <- function(m,
                                    p,
                                    qnew,
                                    thetaYList,
                                    newClusterThetaY,
                                    hparam,
                                    constraint) {
  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  ## psy
  psy <- newClusterThetaY@psy[[1]]
  psyEval <- 0
  if (constraint[2] == T) {
    ## psyVal = 0 in this case
  } else if (constraint[2] == F & constraint[3] == T) {
    psyEval <- dgamma((diag(1 / psy)[1]), shape = hparam@delta, rate = hparam@bbeta, log = T)
  } else if (constraint[2] == F & constraint[3] == F) {
    psyEval <- sum(dgamma(1 / diag(psy), shape = hparam@delta, rate = hparam@bbeta, log = T))
  }


  ## lambda
  lambda <- newClusterThetaY@lambda[[1]]
  lambdaEval <- 0
  if (constraint[1] == T) {
    ## lambdaEval = 0
  } else if (constraint[1] == F) {
    for (j in 1:qnew) {
      lambdaEval[j] <- mvtnorm::dmvnorm(lambda[, j], rep(0, p), 1 / alpha2 * psy, log = T)
    }
    lambdaEval <- sum(lambdaEval)
  }
  return(list(psyEval = psyEval, lambdaEval = lambdaEval))
}

#' propose_combined_clusters
#'
#' @param X X
#' @param thetaYList thetaYList
#' @param hparam hparam
#' @param combineClusInd combineClusInd
#' @param combinedClusInd combinedClusInd
#' @param qVec qVec
#' @param constraint constraint
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' combineClusInd <- c(1, 1)
#' combinedClusInd <- 1
#' propose_combined_clusters(X, thetaYList, hparam, combineClusInd, combinedClusInd, qVec, constraint)
#' @noRd
propose_combined_clusters <- function(X, thetaYList, hparam, combineClusInd, combinedClusInd, qVec, constraint) {
  p <- nrow(X)
  n <- ncol(X)

  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2


  resThetaYList <- thetaYList

  ## propose w
  w1 <- thetaYList@tao[combineClusInd[1]]
  w2 <- thetaYList@tao[combineClusInd[2]]
  w <- w1 + w2

  if (w > 1) {
    w <- 1
  }

  resThetaYList@tao[combinedClusInd] <- w


  ## propose mu

  mu1 <- thetaYList@M[[combineClusInd[1]]]
  mu2 <- thetaYList@M[[combineClusInd[2]]]

  mu <- w1 / w * mu1 + w2 / w * mu2

  # mu =   (mu1 + mu2)/2

  resThetaYList@M[[combinedClusInd]] <- mu

  ## propose psy

  ## propose lambda and psy
  if (constraint[2] == 1) {
    psy <- thetaYList@psy[combineClusInd[1]]
  } else {
    psy <- generate_prior_psi(p, 1, delta, bbeta, constraint)
  }

  resThetaYList@psy[[combinedClusInd]] <- psy[[1]]

  if (constraint[1] == 1) {
    lambda <- thetaYList@lambda[combineClusInd[1]]
  } else {
    lambda <- generate_prior_lambda(p, 1, alpha2, qVec, psy, constraint)
  }

  resThetaYList@lambda[combinedClusInd] <- lambda


  ## proposed Y
  for (k in combinedClusInd) {
    resThetaYList@Y[[k]] <- matrix(NA, qVec[k], n)
    for (i in 1:n) {
      resThetaYList@Y[[k]][, i] <- mvtnorm::rmvnorm(1, mean = rep(0, qVec[k]), sigma = diag(qVec[k]))
    }
  }



  return(resThetaYList)
}

#' evaluate_combined_clusters
#'
#' @param X X
#' @param combinedThetaYList combinedThetaYList
#' @param hparam hparam
#' @param combinedClusInd combinedClusInd
#' @param qVec qVec
#' @param constraint constraint
#'
#' @noRd
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' combinedThetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' combinedClusInd <- 1
#' evaluate_combined_clusters(X, combinedThetaYList, hparam, combinedClusInd, qVec, constraint)
#' @noRd
evaluate_combined_clusters <- function(X, combinedThetaYList, hparam, combinedClusInd, qVec, constraint) {
  p <- nrow(X)
  n <- ncol(X)

  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2


  ## eval  psy
  psy <- combinedThetaYList@psy[combinedClusInd]
  evalpsy <- 0
  if (constraint[2] == 0) {
    evalpsy <- evaluate_prior_psi(psy, p, 1, delta, bbeta, constraint, 1)
  }

  ## eval lambda
  lambda <- combinedThetaYList@lambda[combinedClusInd]
  evallambda <- 0
  if (constraint[1] == 0) {
    evallambda <- evaluate_prior_lambda(p, 1, alpha2, qVec, psy, lambda, constraint, 1)
  }


  return(sum(evalpsy, evallambda))
}




#' combine_allocations
#'
#' @param ZOneDim ZOneDim
#' @param combineClusInd combineClusInd
#' @param combinedClusInd combinedClusInd
#'
#' @examples
#' ZOneDim <- c(1, 1, 1, 2, 3, 4)
#' combineClusInd <- c(1, 2)
#' combinedClusInd <- 1
#' combine_allocations(ZOneDim, combineClusInd, combinedClusInd)
#' @noRd
combine_allocations <- function(ZOneDim, combineClusInd, combinedClusInd) {
  ZOneDimNew <- ZOneDim
  ZOneDimNew[ZOneDimNew == combineClusInd[1]] <- combinedClusInd
  ZOneDimNew[ZOneDimNew == combineClusInd[2]] <- combinedClusInd
  return(ZOneDimNew)
}

#' stay_mcmc_update
#'
#' @param X X
#' @param thetaYList thetaYList
#' @param ZOneDim ZOneDim
#' @param hparam hparam
#' @param qVec qVec
#' @param qnew qnew
#' @param dVec dVec
#' @param sVec sVec
#' @param constraint constraint
#' @param clusInd clusInd
#'
#' @noRd
stay_mcmc_update <- function(X,
                           thetaYList,
                           ZOneDim,
                           hparam,
                           qVec,
                           qnew,
                           dVec,
                           sVec,
                           constraint,
                           clusInd) {
  ggamma <- hparam@ggamma
  m <- length(which(clusInd == 1))
  p <- nrow(X)
  n <- ncol(X)
  ## transfor to non empty obj
  sampleVec <- which(clusInd == 1)
  NEthetaYList <- subset_theta_y(thetaYList, sampleVec)

  NEZOneDim <- ZOneDim
  for (i in 1:length(sampleVec)) {
    NEZOneDim[NEZOneDim == sampleVec[i]] <- i
  }
  ## update

  NEthetaYList <- update_post_theta_y(m, n, p, hparam, NEthetaYList, NEZOneDim, qVec[qVec != 0], constraint, X, ggamma)
  NEZOneDim <- update_post_z_r(X, m, n, NEthetaYList)
  hparam <- update_hyperparameter(m, p, qnew, hparam, NEthetaYList, dVec, sVec)


  ZOneDim <- sampleVec[NEZOneDim]

  thetaYList <- theta_y_with_empty_components(NEthetaYList, clusInd)
  return(list(thetaYList = thetaYList, ZOneDim = ZOneDim, hparam = hparam))
}



#' (internal)
#' @noRd
#' @noRd
theta_y_with_empty_components <- function(NEthetaYList, clusInd) {
  thetaYList <- new("ThetaYList")
  j <- 1
  for (i in 1:length(clusInd)) {
    if (clusInd[i] == 1) {
      thetaYList@tao[i] <- NEthetaYList@tao[j]
      thetaYList@psy[[i]] <- NEthetaYList@psy[[j]]
      thetaYList@M[[i]] <- NEthetaYList@M[[j]]
      thetaYList@lambda[[i]] <- NEthetaYList@lambda[[j]]
      thetaYList@Y[[i]] <- NEthetaYList@Y[[j]]
      j <- j + 1
    } else {
      thetaYList@tao[i] <- 0
      thetaYList@psy[[i]] <- NA
      thetaYList@M[[i]] <- NA
      thetaYList@lambda[[i]] <- NA
      thetaYList@Y[[i]] <- NA
    }
  }
  thetaYList
}

#' (internal)
#' @noRd
#' @noRd
to_non_empty_theta_y_list <- function(thetaYList, ZOneDim, qVec, clusInd) {
  ## transfor to non empty obj
  sampleVec <- which(clusInd == 1)
  NEthetaYList <- subset_theta_y(thetaYList, sampleVec)

  NEZOneDim <- ZOneDim
  for (i in 1:length(sampleVec)) {
    NEZOneDim[NEZOneDim == sampleVec[i]] <- i
  }
  qnew <- max(qVec)
  qVec <- qVec[qVec == qnew]
  return(list(thetaYList = NEthetaYList, ZOneDim = NEZOneDim, qVec = qVec))
}

#' Title
#'
#' @param NEthetaYList NEthetaYList
#' @param NEZOneDim NEZOneDim
#' @param qnew qnew
#' @param clusInd clusInd
#'
#' @noRd
to_empty_theta_y_list <- function(NEthetaYList, NEZOneDim, qnew, clusInd) {
  ## transfor back
  sampleVec <- which(clusInd == 1)
  ZOneDim <- NEZOneDim
  for (i in length(sampleVec):1) {
    ZOneDim[ZOneDim == i] <- sampleVec[i]
  }
  qVec <- clusInd
  qVec[clusInd == 1] <- qnew
  thetaYList <- theta_y_with_empty_components(NEthetaYList, clusInd)
  return(list(thetaYList = thetaYList, ZOneDim = ZOneDim, qVec = qVec))
}

available_cores <- function() {
  cores <- parallel::detectCores(logical = FALSE)
  if (is.na(cores) || cores < 1L) {
    return(1L)
  }
  as.integer(cores)
}

validate_positive_integer <- function(x, name) {
  if (!isTRUE(length(x) == 1L) || !is.finite(x) || x < 1) {
    stop(name, " must be a positive scalar", call. = FALSE)
  }
  as.integer(x)
}

restore_random_seed <- function(seed) {
  if (is.null(seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      rm(".Random.seed", envir = .GlobalEnv)
    }
    return(invisible(NULL))
  }
  assign(".Random.seed", seed, envir = .GlobalEnv)
  invisible(NULL)
}

#' calculate_proposal_psi
#'
#' @param hparam hparam
#' @param thetaYList thetaYList
#' @param CxyList CxyList
#' @param constraint constraint
#' @param m the number of clusters
#' @param qVec the vector of the number of factors in each clusters
#' @param p the number of features
#'
#' @noRd
calculate_proposal_psi <- function(hparam, thetaYList, CxyList, constraint, m, p, qVec) {
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2
  bbeta <- hparam@bbeta
  delta <- hparam@delta
  M <- thetaYList@M
  psy <- thetaYList@psy
  lambda <- thetaYList@lambda


  Cxxk <- CxyList$Cxxk
  Cxyk <- CxyList$Cxyk
  Cyyk <- CxyList$Cyyk
  Cytytk <- CxyList$Cytytk
  Cxtytk <- CxyList$Cxtytk
  CxL1k <- CxyList$CxL1k
  Cxmyk <- CxyList$Cxmyk
  sumCxmyk <- CxyList$sumCxmyk
  sumCyyk <- CxyList$sumCyyk
  A <- CxyList$A
  nVec <- CxyList$nVec

  # post tilda lambda_k = {mu_k, lambda_k}, first column is mu_k
  tildaLambda <- list()
  # print(M)
  # print(lambda)
  for (k in 1:m) {
    tildaLambda[[k]] <- cbind(t(M[[k]]), lambda[[k]])
  }

  if (constraint[1] == T & constraint[2] == T & constraint[3] == T) {
    ## model 1

    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + p / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / (m * p) + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + (2 * bbeta) / (m * p) * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1
    ratePara <- sum(ratePara)

    invpsy <- rgamma(1, shape = shapePara, rate = ratePara)
    for (k in 1:m) {
      psy[[k]] <- diag(rep(1 / invpsy, p))
    }
    ## model 1 end
  } else if (constraint[1] == T & constraint[2] == T & constraint[3] == F) {
    ## model 2

    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + 1 / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / m + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + 2 * bbeta / m * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1

    invpsy <- c()
    for (j in 1:p) {
      invpsy[j] <- rgamma(1, shape = shapePara, rate = ratePara[j])
    }
    for (k in 1:m) {
      psy[[k]] <- diag(1 / invpsy)
    }
    ## end model 2
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == T) {
    ## model 3

    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- p / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / p + 1) + 1
      ratePara <- 1 / 2 * sum(diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / p * diag(rep(1, p))))

      invpsy <- rgamma(1, shape = shapePara, rate = ratePara)
      psy[[k]] <- diag(rep(1 / invpsy, p))
    }


    ## end model 3
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == F) {
    ## model 4
    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- 1 / 2 * (nVec[k] + qVec[k] / m + 2 * delta - 1) + 1
      ratePara <- 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + 2 * bbeta * diag(rep(1, p)))


      invpsy <- c()
      for (j in 1:p) {
        invpsy[j] <- rgamma(1, shape = shapePara, rate = ratePara[j])
      }
      # invpsy = rgamma(p, shape = shapePara, rate = ratePara)
      psy[[k]] <- diag(1 / invpsy)
    }

    ## end model 4
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == T) {
    ## model 5
    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + p / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / (m * p) + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / (m * p) * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1
    ratePara <- sum(ratePara)

    invpsy <- rgamma(1, shape = shapePara, rate = ratePara)

    for (k in 1:m) {
      psy[[k]] <- diag(rep(1 / invpsy), p)
    }
    ## end model 5
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == F) {
    ## model 6
    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + 1 / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / m + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / m * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1

    invpsy <- c()
    for (j in 1:p) {
      invpsy[j] <- rgamma(1, shape = shapePara, rate = ratePara[j])
    }
    for (k in 1:m) {
      psy[[k]] <- diag(1 / invpsy)
    }
    ## end model 6
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == T) {
    ## model 7

    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- p / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / p + 1) + 1
      ratePara <- 1 / 2 * sum(diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / p * diag(rep(1, p))))

      invpsy <- rgamma(1, shape = shapePara, rate = ratePara)
      psy[[k]] <- diag(rep(1 / invpsy, p))
    }
    ## end model 7
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == F) {
    ## model 8
    ## post psy
    psy <- list()

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- 1 / 2 * (nVec[k] + qVec[k] + 2 * delta - 1) + 1
      ratePara <- 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta * diag(rep(1, p)))

      invpsy <- rgamma(p, shape = shapePara, rate = ratePara)
      psy[[k]] <- diag(1 / invpsy)
    }

    ## end model 8
  }
  return(psy)
}


#' (internal)
#' @noRd
#' @noRd
evaluate_proposal_psi <- function(hparam, thetaYList, CxyList, constraint, newpsy, m, p, qVec, delta) {
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2
  bbeta <- hparam@bbeta
  delta <- hparam@delta
  M <- thetaYList@M
  lambda <- thetaYList@lambda

  ##
  Cxxk <- CxyList$Cxxk
  Cxyk <- CxyList$Cxyk
  Cyyk <- CxyList$Cyyk
  Cytytk <- CxyList$Cytytk
  Cxtytk <- CxyList$Cxtytk
  CxL1k <- CxyList$CxL1k
  Cxmyk <- CxyList$Cxmyk
  sumCxmyk <- CxyList$sumCxmyk
  sumCyyk <- CxyList$sumCyyk
  A <- CxyList$A
  nVec <- CxyList$nVec

  ##
  psyEval <- c()
  psy <- newpsy
  tildaLambda <- list()
  for (k in 1:m) {
    tildaLambda[[k]] <- cbind(t(M[[k]]), lambda[[k]])
  }
  ##
  if (constraint[1] == T & constraint[2] == T & constraint[3] == T) {
    ## model 1

    ## post psy
    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + p / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / (m * p) + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + (2 * bbeta) / (m * p) * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1
    ratePara <- sum(ratePara)

    psyEval <- dgamma(1 / psy[[1]][1, 1], shape = shapePara, rate = ratePara, log = T)
    ## model 1 end
  } else if (constraint[1] == T & constraint[2] == T & constraint[3] == F) {
    ## model 2

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + 1 / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / m + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + 2 * bbeta / m * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1

    invpsy <- 1 / diag(psy[[1]])
    for (j in 1:p) {
      psyEval[j] <- dgamma(invpsy[j], shape = shapePara, rate = ratePara[j], log = T)
    }
    ## end model 2
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == T) {
    ## post psy

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- p / 2 * (nVec[k] + qVec[k] / m + (2 * delta - 2) / p + 1) + 1
      ratePara <- 1 / 2 * sum(diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / p * diag(rep(1, p))))

      psyEval[k] <- dgamma(1 / psy[[k]][1, 1], shape = shapePara, rate = ratePara, log = T)
    }


    ## end model 3
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == F) {
    ## model 4
    ## post psy

    shapePara <- 0
    ratePara <- 0
    psyEval <- matrix(NA, nrow = m, ncol = p)
    for (k in 1:m) {
      shapePara <- 1 / 2 * (nVec[k] + qVec[k] / m + 2 * delta - 1) + 1
      ratePara <- 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]] / m) %*% t(tildaLambda[[k]])
        + 2 * bbeta * diag(rep(1, p)))


      invpsy <- 1 / diag(psy[[k]])
      for (j in 1:p) {
        psyEval[k, j] <- dgamma(invpsy[j], shape = shapePara, rate = ratePara[j], log = T)
      }
    }

    ## end model 4
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == T) {
    ## model 5

    ## post psy
    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + p / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / (m * p) + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / (m * p) * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1
    ratePara <- sum(ratePara)

    psyEval <- dgamma(1 / psy[[1]][1, 1], shape = shapePara, rate = ratePara, log = T)

    ## end model 5
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == F) {
    ## model 6


    ## post psy

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- shapePara + 1 / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / m + 1)
      ratePara <- ratePara + 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / m * diag(rep(1, p)))
    }
    shapePara <- shapePara + 1

    invpsy <- 1 / diag(psy[[1]])
    for (j in 1:p) {
      psyEval[j] <- dgamma(invpsy[j], shape = shapePara, rate = ratePara[j], log = T)
    }

    ## end model 6
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == T) {
    ## model 7


    ## post psy

    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- p / 2 * (nVec[k] + qVec[k] + (2 * delta - 2) / p + 1) + 1
      ratePara <- 1 / 2 * sum(diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta / p * diag(rep(1, p))))

      psyEval[k] <- dgamma(1 / psy[[k]][1, 1], shape = shapePara, rate = ratePara, log = T)
    }
    ## end model 7
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == F) {
    ## model 8

    ## post psy
    psyEval <- matrix(NA, nrow = m, ncol = p)
    shapePara <- 0
    ratePara <- 0
    for (k in 1:m) {
      shapePara <- 1 / 2 * (nVec[k] + qVec[k] + 2 * delta - 1) + 1
      ratePara <- 1 / 2 * diag(Cxxk[[k]] - 2 * Cxtytk[[k]] %*% t(tildaLambda[[k]])
        + tildaLambda[[k]] %*% (Cytytk[[k]] + A[[k]]) %*% t(tildaLambda[[k]])
        + 2 * bbeta * diag(rep(1, p)))


      invpsy <- 1 / diag(psy[[k]])
      for (j in 1:p) {
        psyEval[k, j] <- dgamma(invpsy[j], shape = shapePara, rate = ratePara[j], log = T)
      }
    }
    ## end model 8
  }
  return(sum(psyEval))
}

#' Hyperparameter set for the Bayesian PGMM sampler.
#'
#' @import methods
#' @slot alpha1 First Dirichlet prior parameter for component weights.
#' @slot alpha2 Second Dirichlet prior parameter for component weights.
#' @slot delta Shape parameter used in prior updates.
#' @slot ggamma Prior rate parameter used in covariance updates.
#' @slot bbeta Prior scale parameter used in covariance updates.
#' @keywords internal
#'
setClass(
  "Hparam",
  slots = c(
    alpha1 = "numeric",
    alpha2 = "numeric",
    delta  = "numeric",
    ggamma = "numeric",
    bbeta  = "numeric"
  ),
  prototype = list(
    alpha1 = numeric(),
    alpha2 = numeric(),
    delta  = numeric(),
    ggamma = numeric(),
    bbeta  = numeric()
  )
)


setValidity("Hparam", function(object) {
  if (object@alpha1 < 0 |
    object@alpha2 < 0 |
    object@delta < 0 |
    object@ggamma < 0 |
    object@bbeta < 0) {
    "Hyperparameter should be non-negative!"
  }
})

#' Summarize RJMCMC Samples from a Bayesian PGMM Fit
#'
#' Summarizes posterior samples from [pgmm_rjmcmc()] into the modal allocation,
#' posterior counts for the number of clusters, posterior counts for the eight
#' PGMM covariance-constraint models, and optionally the adjusted Rand index
#' against a known reference partition.
#'
#' @param fit Result list from [pgmm_rjmcmc()].
#' @param true_cluster Optional true or reference cluster allocation.
#'
#' @return A list with `allocation`, `n_clusters`, `n_constraints`, and optionally
#'   `ari`.
#' @importFrom mclust adjustedRandIndex
#' @name summarize_pgmm_rjmcmc
#' @export
summarize_pgmm_rjmcmc <- function(fit, true_cluster = NULL) {
  if (!is.list(fit)) {
    stop("fit must be a result list from pgmm_rjmcmc()", call. = FALSE)
  }
  if (!is.list(fit$allocation_samples) || length(fit$allocation_samples) == 0L) {
    stop("fit$allocation_samples must contain at least one allocation sample", call. = FALSE)
  }
  if (!is.list(fit$constraint_samples) || length(fit$constraint_samples) != length(fit$allocation_samples)) {
    stop("fit$constraint_samples must match fit$allocation_samples", call. = FALSE)
  }

  allocation <- summarize_allocations(fit$allocation_samples)

  n_clusters <- table(sapply(fit$allocation_samples, function(x) {
    length(unique(x))
  }))

  n_constraints <- fit$constraint_samples
  n_constraints <- constraint_list_to_models(n_constraints)
  n_constraints <- table(n_constraints, dnn = "")



  summary <- list(allocation = allocation, n_clusters = n_clusters, n_constraints = n_constraints)

  if (!is.null(true_cluster)) {
    ari <- adjustedRandIndex(true_cluster, allocation)
    summary$ari <- ari
  }

  summary
}

test_that("summarize_pgmm_rjmcmc summarizes posterior allocation and models", {
  pgmm_res <- list(
    allocation_samples = list(c(1, 1, 2), c(1, 2, 2), c(1, 1, 2)),
    constraint_samples = list(c(0, 0, 0), c(1, 0, 0), c(1, 0, 0))
  )

  summary <- summarize_pgmm_rjmcmc(pgmm_res, true_cluster = c(1, 1, 2))

  expect_equal(summary$allocation, c(1, 1, 2))
  expect_equal(as.integer(summary$n_clusters["2"]), 3L)
  expect_equal(as.integer(summary$n_constraints["CUU"]), 2L)
  expect_equal(as.integer(summary$n_constraints["UUU"]), 1L)
  expect_equal(summary$ari, 1)
})

test_that("internal allocation summarizer keeps compatibility alias", {
  z_samples <- list(c(1, 1, 2), c(1, 2, 2), c(1, 1, 2))

  expect_equal(bpgmm:::summarize_allocations(z_samples), c(1, 1, 2))
  expect_equal(bpgmm:::summarize_allocations_legacy(z_samples), bpgmm:::summarize_allocations(z_samples))
})

test_that("summarize_pgmm_rjmcmc validates result structure", {
  expect_error(summarize_pgmm_rjmcmc(list()), "allocation_samples")
  expect_error(
    summarize_pgmm_rjmcmc(list(allocation_samples = list(), constraint_samples = list())),
    "allocation_samples"
  )
  expect_error(
    summarize_pgmm_rjmcmc(list(allocation_samples = list(c(1, 2)), constraint_samples = list())),
    "constraint_samples"
  )
})

#' (internal)
#' @noRd
calculate_cxy <- function(m, n, hparam, theta_y_list, z, q_vec, x) {
  calculate_cxy_native(m, n, hparam, theta_y_list, z, q_vec, x)
}

#' (internal)
#' @noRd
calculate_post_lambda_psi <- function(m, p, hparam, cxy_list, theta_y_list, q_vec, constraint) {
  calculate_post_lambda_psi_native(m, p, hparam, cxy_list, theta_y_list, q_vec, constraint)
}

#' (internal)
#' @noRd
update_post_z_cpp <- function(x, m, n, theta_y_list) {
  update_post_z_native(x, m, n, theta_y_list)
}

#' (internal)
#' @noRd
update_latent_scores_cpp <- function(x, theta_y_list, z, clus_ind, q_vec) {
  update_latent_scores_native(x, theta_y_list, z, clus_ind, q_vec)
}

#' (internal)
#' @noRd
update_hyperparameter <- function(m, p, q, hparam, theta_y_list, d_vec, s_vec) {
  update_hyperparameter_native(m, p, q, hparam, theta_y_list, d_vec, s_vec)
}

#' (internal)
#' @noRd
get_z_mat_cpp <- function(z, m, n) {
  get_z_matrix_native(z, m, n)
}

#' (internal)
#' @noRd
calculate_ratio_cpp <- function(log_denominator, log_numerator) {
  calculate_ratio_native(log_denominator, log_numerator)
}

#' (internal)
#' @noRd
evaluate_prior_psi_cpp <- function(psy, p, m, delta, bbeta, constraint, clus_ind) {
  evaluate_prior_psi_native(psy, p, m, delta, bbeta, constraint, clus_ind)
}

#' (internal)
#' @noRd
evaluate_prior_lambda_cpp <- function(p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind) {
  evaluate_prior_lambda_native(p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind)
}

test_that("Rcpp get_z_mat_cpp matches the R implementation", {
  labels <- c(1, 2, 1, 3)

  expect_equal(
    bpgmm:::get_z_mat_cpp(labels, m = 3, n = length(labels)),
    bpgmm:::get_z_mat_r(labels, m = 3, n = length(labels))
  )
})

test_that("native Rcpp wrappers use snake_case names", {
  native_names <- ls(getNamespace("bpgmm"), pattern = "^[a-z].*_native$")

  expect_true(all(grepl("^[a-z][a-z0-9_]*$", native_names)))
  expect_false(any(grepl("[A-Z]", native_names)))
  expect_setequal(
    native_names,
    c(
      "calculate_cxy_native",
      "calculate_post_lambda_psi_native",
      "calculate_ratio_native",
      "evaluate_prior_lambda_native",
      "evaluate_prior_psi_native",
      "get_z_matrix_native",
      "multivariate_normal_density_native",
      "update_hyperparameter_native",
      "update_latent_scores_native",
      "update_post_z_native"
    )
  )
})

test_that("Rcpp get_z_mat_cpp rejects labels outside 1:m", {
  expect_error(
    bpgmm:::get_z_mat_cpp(c(1, 0, 2), m = 2, n = 3),
    "cluster labels must be integers in 1:m"
  )
  expect_error(
    bpgmm:::get_z_mat_cpp(c(1, 3, 2), m = 2, n = 3),
    "cluster labels must be integers in 1:m"
  )
})

test_that("Rcpp multivariate_normal_density_native matches mvtnorm::dmvnorm", {
  x <- rbind(c(0, 0), c(1, -1))
  mean <- c(0.25, -0.5)
  sigma <- matrix(c(2, 0.4, 0.4, 1.5), nrow = 2)

  expect_equal(
    as.numeric(bpgmm:::multivariate_normal_density_native(x, mean, sigma, TRUE)),
    as.numeric(mvtnorm::dmvnorm(x, mean = mean, sigma = sigma, log = TRUE)),
    tolerance = 1e-10
  )
})

test_that("Rcpp multivariate_normal_density_native validates dimensions and covariance", {
  x <- matrix(c(1, 2), nrow = 1)

  expect_error(
    bpgmm:::multivariate_normal_density_native(x, mean = 0, sigma = diag(2), logd = TRUE),
    "mean length"
  )
  expect_error(
    bpgmm:::multivariate_normal_density_native(x, mean = c(0, 0), sigma = diag(3), logd = TRUE),
    "sigma"
  )
  expect_error(
    bpgmm:::multivariate_normal_density_native(
      x,
      mean = c(0, 0),
      sigma = matrix(c(1, 2, 2, 1), nrow = 2),
      logd = TRUE
    ),
    "positive definite"
  )
})

test_that("Rcpp calculate_ratio_cpp is stable for large log probabilities", {
  expect_equal(
    bpgmm:::calculate_ratio_cpp(1000, c(1000, 1001, 1002)),
    exp(1000 - 1002) / sum(exp(c(1000, 1001, 1002) - 1002)),
    tolerance = 1e-12
  )
})

test_that("Rcpp calculate_ratio_cpp validates finite log inputs", {
  expect_error(
    bpgmm:::calculate_ratio_cpp(0, numeric()),
    "log_numerator"
  )
  expect_error(
    bpgmm:::calculate_ratio_cpp(Inf, c(0, 1)),
    "finite"
  )
  expect_error(
    bpgmm:::calculate_ratio_cpp(0, c(0, NA)),
    "finite"
  )
})

test_that("Rcpp prior evaluators match closed-form reference values", {
  p <- 2
  m <- 2
  alpha2 <- 3
  delta <- 4
  bbeta <- 2
  q_vec <- c(1, 1)
  clus_ind <- c(1, 1)
  psy <- list(diag(c(1.2, 1.4)), diag(c(1.1, 1.3)))
  lambda <- list(matrix(c(0.2, -0.1), nrow = 2), matrix(c(-0.15, 0.25), nrow = 2))

  psi_expected <- sum(dgamma(1 / diag(psy[[1]]), delta, rate = bbeta, log = TRUE)) +
    sum(dgamma(1 / diag(psy[[2]]), delta, rate = bbeta, log = TRUE))
  lambda_expected <- mvtnorm::dmvnorm(lambda[[1]][, 1], rep(0, p), 1 / alpha2 * psy[[1]], log = TRUE) +
    mvtnorm::dmvnorm(lambda[[2]][, 1], rep(0, p), 1 / alpha2 * psy[[2]], log = TRUE)

  expect_equal(
    bpgmm:::evaluate_prior_psi_cpp(psy, p, m, delta, bbeta, c(0, 0, 0), clus_ind),
    psi_expected,
    tolerance = 1e-10
  )
  expect_equal(
    bpgmm:::evaluate_prior_lambda_cpp(p, m, alpha2, q_vec, psy, lambda, c(0, 0, 0), clus_ind),
    lambda_expected,
    tolerance = 1e-10
  )
})

test_that("Rcpp update_post_z_cpp uses log mixture weights", {
  n <- 2000
  theta <- new(
    "ThetaYList",
    tao = c(0.99, 0.01),
    psy = list(matrix(1), matrix(1)),
    M = list(0, 0),
    lambda = list(matrix(0), matrix(0)),
    Y = list(matrix(0, nrow = 1, ncol = n), matrix(0, nrow = 1, ncol = n))
  )

  set.seed(22)
  labels <- bpgmm:::update_post_z_cpp(matrix(0, nrow = 1, ncol = n), m = 2, n = n, theta)

  expect_gt(mean(labels == 1), 0.95)
})

test_that("Rcpp update_post_z_cpp validates dimensions and mixture weights", {
  theta <- new(
    "ThetaYList",
    tao = c(1, 0),
    psy = list(matrix(1), matrix(1)),
    M = list(0, 0),
    lambda = list(matrix(0), matrix(0)),
    Y = list(matrix(0, nrow = 1, ncol = 1), matrix(0, nrow = 1, ncol = 1))
  )

  expect_error(
    bpgmm:::update_post_z_cpp(matrix(0, nrow = 1, ncol = 1), m = 2, n = 1, theta),
    "tao"
  )
  expect_error(
    bpgmm:::update_post_z_cpp(matrix(0, nrow = 1, ncol = 2), m = 2, n = 1, theta),
    "n"
  )
})

test_that("Rcpp update_latent_scores_cpp samples latent score matrices", {
  x <- matrix(c(-1, 0, 1, 2, 3, 4), nrow = 2)
  theta <- new(
    "ThetaYList",
    tao = c(0.5, 0.5),
    psy = list(diag(2), diag(c(1.2, 1.4))),
    M = list(c(0, 0), c(1, 1)),
    lambda = list(matrix(c(0.2, -0.1), nrow = 2), matrix(c(0.1, 0.3), nrow = 2)),
    Y = list(matrix(0, nrow = 1, ncol = 3), matrix(0, nrow = 1, ncol = 3))
  )

  set.seed(2026)
  scores <- bpgmm:::update_latent_scores_cpp(
    x = x,
    theta_y_list = theta,
    z = c(1, 2, 1),
    clus_ind = c(1, 1),
    q_vec = c(1, 1)
  )

  expect_type(scores, "list")
  expect_length(scores, 2)
  expect_true(all(vapply(scores, function(y) identical(dim(y), c(1L, 3L)), logical(1))))
  expect_true(all(is.finite(unlist(scores))))
})

test_that("Rcpp update_latent_scores_cpp validates native inputs", {
  theta <- new(
    "ThetaYList",
    tao = c(1),
    psy = list(diag(2)),
    M = list(c(0, 0)),
    lambda = list(matrix(c(0.1, 0.2), nrow = 2)),
    Y = list(matrix(0, nrow = 1, ncol = 2))
  )
  x <- matrix(c(1, 2, 3, 4), nrow = 2)

  expect_error(
    bpgmm:::update_latent_scores_cpp(x, theta, z = c(1, 2), clus_ind = c(1), q_vec = c(1)),
    "cluster labels"
  )
  expect_error(
    bpgmm:::update_latent_scores_cpp(x, theta, z = c(1, 1), clus_ind = c(1), q_vec = c(0)),
    "q_vec"
  )
})

test_that("Rcpp calculate_cxy validates native inputs", {
  theta <- new(
    "ThetaYList",
    tao = c(1),
    psy = list(diag(2)),
    M = list(c(0, 0)),
    lambda = list(matrix(c(0.1, 0.2), nrow = 2)),
    Y = list(matrix(c(0.3, 0.4), nrow = 1, ncol = 2))
  )
  hparam <- new("Hparam", alpha1 = 2, alpha2 = 3, bbeta = 2, delta = 3, ggamma = 1)
  x <- matrix(c(1, 2, 3, 4), nrow = 2)

  expect_error(
    bpgmm:::calculate_cxy(1, 2, hparam, theta, c(1, 1), c(0), x),
    "q_vec entries must be positive"
  )
  x_bad <- x
  x_bad[1, 1] <- Inf
  expect_error(
    bpgmm:::calculate_cxy(1, 2, hparam, theta, c(1, 1), c(1), x_bad),
    "X must contain only finite values"
  )
})

test_that("Rcpp posterior lambda/psi update validates constraints", {
  theta <- new(
    "ThetaYList",
    tao = c(1),
    psy = list(diag(2)),
    M = list(c(0, 0)),
    lambda = list(matrix(c(0.1, 0.2), nrow = 2)),
    Y = list(matrix(c(0.3, 0.4), nrow = 1, ncol = 2))
  )
  hparam <- new("Hparam", alpha1 = 2, alpha2 = 3, bbeta = 2, delta = 3, ggamma = 1)
  x <- matrix(c(1, 2, 3, 4), nrow = 2)
  cxy <- bpgmm:::calculate_cxy(1, 2, hparam, theta, c(1, 1), c(1), x)

  expect_error(
    bpgmm:::calculate_post_lambda_psi(1, 2, hparam, cxy, theta, c(1), c(1, 0)),
    "constraint must have length 3"
  )
  expect_error(
    bpgmm:::calculate_post_lambda_psi(1, 2, hparam, cxy, theta, c(1), c(1, 0, 2)),
    "constraint entries must be 0/1"
  )
})

test_that("Rcpp hyperparameter update validates native hyperparameters", {
  theta <- new(
    "ThetaYList",
    tao = c(1),
    psy = list(diag(2)),
    M = list(c(0, 0)),
    lambda = list(matrix(c(0.1, 0.2), nrow = 2)),
    Y = list(matrix(c(0.3, 0.4), nrow = 1, ncol = 2))
  )
  hparam <- new("Hparam", alpha1 = 2, alpha2 = 3, bbeta = 2, delta = 3, ggamma = 1)

  expect_error(
    bpgmm:::update_hyperparameter(1, 2, 1, hparam, theta, c(1, 1), c(1, 1, 1)),
    "d_vec must have length 3"
  )
  expect_error(
    bpgmm:::update_hyperparameter(1, 2, 1, hparam, theta, c(1, 1, 1), c(1, NA, 1)),
    "s_vec entries must be positive finite values"
  )
})

#' (internal)
#' @noRd
#' @noRd
vstep_rjmcmc_update <- function(X,
                              muBar,
                              p,
                              thetaYList,
                              ZOneDim,
                              hparam,
                              hparamInit,
                              qVec,
                              qnew,
                              ggamma,
                              dVec,
                              sVec,
                              constraint,
                              clusInd) {



  ## rjmcmc for V
  # print("vStep")
  n <- ncol(X)
  m <- sum(clusInd)
  # steps = c("stay", "lambda", "psi1", "psi2")
  steps <- c("lambda", "psi1", "psi2")
  currentStep <- sample(size = 1, x = steps)
  # currentStep = "psi2"
  # currentStep = "stay"


  ## trans to non empty obj
  NEobj <- to_non_empty_theta_y_list(thetaYList, ZOneDim, qVec, clusInd)
  thetaYList <- NEobj$thetaYList
  ZOneDim <- NEobj$ZOneDim
  qVec <- NEobj$qVec
  # if(currentStep == "stay"){
  #   print("stay")
  #   ZOneDim = update_post_z_cpp(X, m, n, thetaYList)
  #   # ZOneDim = update_post_z_r(m, n, thetaYList)
  #   thetaYList = update_post_theta_y(m,  n,hparam, thetaYList, ZOneDim,  qVec,constraint, X, ggamma)
  #   hparam = update_hyperparameter(m, p, qnew, hparam,thetaYList, dVec, sVec)
  #
  # }
  if (currentStep == "lambda") {

    # print("lambda step")
    proposeConstraint <- constraint
    proposeConstraint[1] <- (constraint[1] + 1) %% 2

    CxyList <- calculate_cxy(m, n, hparam, thetaYList, ZOneDim, qVec, X)

    newLambda <- calculate_proposal_lambda(hparam, thetaYList, CxyList, proposeConstraint, m, p, qVec)

    newthetaYList <- thetaYList
    newthetaYList@lambda <- newLambda

    newCxyList <- calculate_cxy(m, n, hparam, newthetaYList, ZOneDim, qVec, X)

    oldDensity <- likelihood(thetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, sort(clusInd, decreasing = T))
    oldLambdaEval <- evaluate_proposal_lambda(hparam, newthetaYList, newCxyList, constraint, thetaYList@lambda, m, qVec, p)
    newLambdaEval <- evaluate_proposal_lambda(hparam, thetaYList, CxyList, proposeConstraint, newthetaYList@lambda, m, qVec, p)




    ## Gibbs
    newhparam <- hparam
    for (i in 1:10) {
      newZOneDim <- update_post_z_cpp(X, m, n, newthetaYList)
      # newZOneDim = update_post_z_r(m, n, newthetaYList)

      newthetaYList <- update_post_theta_y(m = m, n = n, p, newhparam, newthetaYList, ZOneDim = newZOneDim, qVec = qVec, constraint = proposeConstraint, X, ggamma)
      newhparam <- update_hyperparameter(m, p, qnew, newhparam, newthetaYList, dVec, sVec)
    }

    newDensity <- likelihood(newthetaYList, newZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, newhparam, newthetaYList, newZOneDim, qVec, proposeConstraint, sort(clusInd, decreasing = T))

    numer <- newDensity + oldLambdaEval
    denom <- oldDensity + newLambdaEval
    # diffVec[h] = numer - denom
    # cat("accepting prob = exp ", numer - denom, " \n")

    probAlpha <- calculate_ratio_cpp(numer, denom)
    acceptP <- min(1, probAlpha)
    res <- rbinom(1, size = 1, prob = acceptP)

    if (res == 1) {
      # print("lambda success=====>")
      # cat("lambda prob = ", probAlpha, "====>\n")

      constraint <- proposeConstraint
      thetaYList <- newthetaYList
      ZOneDim <- newZOneDim
      hparam <- newhparam
    } else {
      # print("lambda fail=====>")
    }
  }
  else if (currentStep == "psi1" | currentStep == "psi2") {

    # print(currentStep)
    proposeConstraint <- constraint
    if (currentStep == "psi1") {
      proposeConstraint[2] <- (constraint[2] + 1) %% 2
    } else {
      proposeConstraint[3] <- (constraint[3] + 1) %% 2
    }

    CxyList <- calculate_cxy(m, n, hparam, thetaYList, ZOneDim, qVec, X)
    newPsy <- calculate_proposal_psi(hparam, thetaYList, CxyList, proposeConstraint, m, p, qVec)

    newthetaYList <- thetaYList
    newthetaYList@psy <- newPsy
    newCxyList <- calculate_cxy(m, n, hparam, newthetaYList, ZOneDim, qVec, X)


    # oldDensity = likelihood(thetaYList, ZOneDim,qVec,muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint,clusInd)
    # newDensity = likelihood(newthetaYList, ZOneDim,qVec,muBar, X) + evaluate_prior(m, p, muBar,hparam, newthetaYList, ZOneDim, qVec, proposeConstraint,clusInd)

    oldDensity <- likelihood(thetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, sort(clusInd, decreasing = T))
    newDensity <- likelihood(newthetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, newthetaYList, ZOneDim, qVec, proposeConstraint, sort(clusInd, decreasing = T))


    oldPsyEval <- evaluate_proposal_psi(hparam, newthetaYList, newCxyList, constraint, thetaYList@psy, m, p, qVec)
    newPsyEval <- evaluate_proposal_psi(hparam, thetaYList, CxyList, proposeConstraint, newthetaYList@psy, m, p, qVec)

    numer <- newDensity + oldPsyEval
    denom <- oldDensity + newPsyEval
    # print(numer - denom)
    # diffVec[h] = numer - denom

    probAlpha <- calculate_ratio_cpp(numer, denom)
    acceptP <- min(1, probAlpha)
    res <- rbinom(1, size = 1, prob = acceptP)

    if (res == 1) {
      # print("psi success=====>")
      # cat("psi prob = ", probAlpha, "====>\n")
      constraint <- proposeConstraint
      thetaYList <- newthetaYList
      # psy = newPsy
    } else {

      # print("psi fail=====>")
    }
  }

  ## trans to empty obj
  Eobj <- to_empty_theta_y_list(thetaYList, ZOneDim, qnew, clusInd)
  thetaYList <- Eobj$thetaYList
  ZOneDim <- Eobj$ZOneDim
  qVec <- Eobj$qVec



  return(list(thetaYList = thetaYList, ZOneDim = ZOneDim, hparam = hparam, constraint = constraint))
}

#' (internal)
#' @noRd
#' @noRd
get_z_mat_r <- function(ZOneDim, m, n) {
  Zmat <- matrix(NA, m, n)
  for (i in 1:m) {
    Zmat[i, ] <- as.numeric(ZOneDim == i)
  }
  Zmat
}

#' (internal)
#' @noRd
#' @noRd
calculate_ratio <- function(deno, nume) {
  ## deno nume both in log scale
  maxNume <- max(nume)
  transDeno <- deno - maxNume
  transNume <- nume - maxNume
  res <- exp(transDeno) / (sum(exp(transNume)))
  res
}


#' (internal)
#' @noRd
#' @noRd
constraint_list_to_models <- function(stringList) {
  vapply(stringList, constraint_to_model, character(1))
}



#' (internal)
#' @noRd
#' @noRd
likelihood <- function(thetaYList, ZOneDim, qqVec, muBar, X) {
  m <- length(qqVec)
  n <- ncol(X)
  xvec <- c()
  ## 2.1: X
  xVal <- 0
  for (i in 1:n) {
    ## for subject i
    k <- ZOneDim[i]

    # if(is.vector(X[, ZOneDim == k])){
    #   meanx = X[, ZOneDim == k]
    # }else{
    #   meanx = apply(X[, ZOneDim == k], 1, mean)
    # }

    meanx <- thetaYList@lambda[[k]] %*% thetaYList@Y[[k]][, i] + c(thetaYList@M[[k]])
    varx <- thetaYList@psy[[k]]
    xvec[i] <- (mvtnorm::dmvnorm(x = X[, i], mean = meanx, sigma = varx, log = T))
    xVal <- xVal + mvtnorm::dmvnorm(x = X[, i], mean = meanx, sigma = varx, log = T)
  }

  xVal
  # xvec
}

#' Summarize cluster allocations by posterior mode.
#'
#' @param Zlist List of sampled allocation vectors.
#' @param index Sample indices to include in the posterior mode.
#'
#' @return An integer vector of modal cluster allocations.
#' @noRd
#' @noRd
summarize_allocations <- function(Zlist, index = seq_along(Zlist)) {
  sampleSize <- length(Zlist[[1]])
  res <- integer(sampleSize)

  for (i in seq_len(sampleSize)) {
    temp <- integer(length(index))
    for (j in seq_along(index)) {
      temp[j] <- Zlist[[index[j]]][i]
    }
    res[i] <- get_mode(temp)
  }

  res
}

#' (internal)
#' @noRd
summarize_allocations_legacy <- summarize_allocations



#' (internal)
#' @noRd
#' @noRd
get_mode <- function(v) {
  v <- v[!is.na(v)]
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}


#' (internal)
#' @noRd
#' @noRd
calculate_covariance_list <- function(psyList, lambdaList) {
  m <- length(psyList)
  varList <- list()
  for (i in 1:m) {
    tempCov <- psyList[[i]] + lambdaList[[i]] %*% t(lambdaList[[i]])
    varList[[i]] <- tempCov
  }
  return(varList)
}

#' (internal)
#' @noRd
#' @noRd
update_latent_scores <- function(X, thetaYList, ZOneDim, clusInd, qVec) {
  update_latent_scores_cpp(X, thetaYList, ZOneDim, clusInd, qVec)
}


#' (internal)
#' @noRd
#' @noRd
clear_inactive_components <- function(thetaYList, clusInd, mMax) {
  resThetaYList <- thetaYList

  for (i in 1:mMax) {
    if (clusInd[i] == 1) {
      resThetaYList@tao[i] <- thetaYList@tao[i]
      resThetaYList@psy[[i]] <- thetaYList@psy[[i]]
      resThetaYList@M[[i]] <- thetaYList@M[[i]]
      resThetaYList@lambda[[i]] <- thetaYList@lambda[[i]]
      resThetaYList@Y[[i]] <- thetaYList@Y[[i]]
    } else {
      resThetaYList@tao[i] <- 0
      resThetaYList@psy[[i]] <- NA
      resThetaYList@M[[i]] <- NA
      resThetaYList@lambda[[i]] <- NA
      resThetaYList@Y[[i]] <- NA
    }
  }
  return(resThetaYList)
}


#' (internal)
#' @noRd
#' @noRd
combine_cluster_parameters <- function(oldList, newList, ind) {
  resList <- oldList
  resList@tao <- oldList@tao * (1 - newList@tao)
  resList@tao[ind] <- newList@tao

  resList@psy[ind] <- newList@psy
  resList@M[ind] <- newList@M
  resList@lambda[ind] <- newList@lambda
  resList@Y[ind] <- newList@Y
  return(resList)
}




#' (internal)
#' @noRd
#' @noRd
subset_theta_y <- function(thetaYList, Ind) {
  new("ThetaYList",
    tao = thetaYList@tao[Ind],
    psy = thetaYList@psy[Ind],
    M = thetaYList@M[Ind],
    lambda = thetaYList@lambda[Ind],
    Y = thetaYList@Y[Ind]
  )
}

#' (internal)
#' @noRd
#' @noRd
remove_theta_y_component <- function(thetaYList, Ind) {
  res <- thetaYList

  res@tao[Ind] <- NA
  res@tao <- res@tao / sum(res@tao, na.rm = T)
  res@psy[[Ind]] <- NA
  res@M[[Ind]] <- NA
  res@lambda[[Ind]] <- NA
  res@Y[[Ind]] <- NA

  res
}

#' (internal)
#' @noRd
#' @noRd
change_constraint_format <- function(strNum) {
  digits <- regmatches(strNum, gregexpr("[01]", strNum))[[1]]
  constraint_to_model(as.integer(digits))
}

test_that("calculate_ratio is stable for log-scale probabilities", {
  expect_equal(
    bpgmm:::calculate_ratio(log(2), log(c(2, 3, 5))),
    0.2,
    tolerance = 1e-12
  )
})

test_that("get_z_mat_r converts one-dimensional labels to indicator rows", {
  expect_equal(
    bpgmm:::get_z_mat_r(c(1, 2, 1, 3), m = 3, n = 4),
    rbind(c(1, 0, 1, 0), c(0, 1, 0, 0), c(0, 0, 0, 1))
  )
})

#' (internal)
#' @noRd
#' @noRd
mstep_rjmcmc_update <- function(X,
                              muBar,
                              p,
                              thetaYList,
                              ZOneDim,
                              hparam,
                              hparamInit,
                              qVec,
                              qnew,
                              dVec,
                              sVec,
                              constraint,
                              clusInd,
                              mVec,
                              Mind) {


  # print("mStep")
  ## rjmcmc for m

  m <- sum(clusInd)
  n <- ncol(X)
  # empty cluster
  emptyZ <- setdiff(which(clusInd == 1), ZOneDim)

  currentStep <- c()

  ## birth and death
  if (Mind == "BD") {
    if (length(emptyZ) > 1) {
      currentStep <- c("death")
    }
    else if (m == mVec[1] | length(unique(ZOneDim)) == 1) {
      currentStep <- c("birth")
    }
    else if (m == mVec[2]) {
      if (length(emptyZ) > 0) {
        currentStep <- c("death")
      } else {
        currentStep <- c("NA")
      }
    } else {
      if (length(emptyZ) > 0) {
        currentStep <- c("birth", "death")
      } else {
        currentStep <- c("birth")
      }
    }
  }

  if (Mind == "SC") {
    if (m == mVec[1] | length(unique(ZOneDim)) == 1) {
      currentStep <- c("split")
    } else if (m == mVec[2]) {
      currentStep <- c("combine")
    } else {
      currentStep <- c("combine", "split")
    }
  }


  indChar <- sample(currentStep, 1)
  # cat("======",indChar, "======>\n")

  if (indChar == "birth") {
    birthInd <- which(clusInd == 0)[1]

    newClusterThetaY <- generate_new_cluster_parameters(m, n, p, hparam, muBar, qnew, thetaYList, constraint, clusInd)

    ## adjust tao in the combine
    combinedThetaList <- combine_cluster_parameters(thetaYList, newClusterThetaY, birthInd)
    jacobianAdj <- m * log(1 - newClusterThetaY@tao)

    ## adjust q
    qVecNew <- qVec
    qVecNew[birthInd] <- qnew


    ## adjust clusInd
    clusIndNew <- clusInd
    clusIndNew[birthInd] <- 1


    oldDensity <- likelihood(thetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, clusInd)
    newDensity <- likelihood(combinedThetaList, ZOneDim, qVecNew, muBar, X) + evaluate_prior(m + 1, p, muBar, hparam, combinedThetaList, ZOneDim, qVecNew, constraint, clusIndNew)

    newClusterThetaYEval <- evaluate_new_cluster_parameters(m, p, hparam, muBar, qnew, thetaYList, newClusterThetaY, constraint)

    emptyZ <- setdiff(which(clusIndNew == 1), ZOneDim)

    numeratorAlpha <- newDensity + jacobianAdj
    denominatorAlpha <- oldDensity + newClusterThetaYEval + log(length(emptyZ))

    probAlpha <- calculate_ratio_cpp(numeratorAlpha, denominatorAlpha)
    acceptP <- min(1, probAlpha)

    res <- rbinom(1, size = 1, prob = acceptP)
    # cat("birth prob = ", probAlpha, "====>\n")


    if (res == 1) {
      ## birth
      # print("birth succeed=====>")
      # cat("birth prob = ", probAlpha, "====>\n")

      thetaYList <- combinedThetaList
      qVec <- qVecNew
      clusInd <- clusIndNew
      thetaYList <- clear_inactive_components(thetaYList, clusInd, mVec[2])
    } else {
      ## stay
      # print("birth reject =====>")
    }
  }
  else if (indChar == "death") {
    ## no update

    emptyZ <- setdiff(which(clusInd == 1), ZOneDim)
    chooseZprob <- 0
    if (length(emptyZ) == 1) {
      deathInd <- emptyZ
    } else {
      deathInd <- sample(x = emptyZ, size = 1)
      chooseZprob <- log(1 / length(emptyZ))
    }

    ## adj qVec and clusInd
    qVecNew <- qVec
    clusIndNew <- clusInd
    qVecNew[deathInd] <- 0
    clusIndNew[deathInd] <- 0


    dropCluster <- subset_theta_y(thetaYList, deathInd)

    ## adjust tao

    leftThetaY <- remove_theta_y_component(thetaYList, deathInd)
    dropClusterDval <- evaluate_new_cluster_parameters(m - 1, p, hparam, muBar, qnew, leftThetaY, dropCluster, constraint)

    jacobianAdj <- (m - 1) * log(1 - dropCluster@tao)


    oldDensity <- evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, clusInd)
    newDensity <- evaluate_prior(m - 1, p, muBar, hparam, leftThetaY, ZOneDim, qVecNew, constraint, clusIndNew)


    numeratorAlpha <- newDensity + dropClusterDval + chooseZprob
    denominatorAlpha <- oldDensity + jacobianAdj

    # cat("diff  = ", numeratorAlpha - denominatorAlpha, "=====> \n" )

    probAlpha <- calculate_ratio_cpp(numeratorAlpha, denominatorAlpha)
    acceptP <- min(1, probAlpha)

    res <- rbinom(1, size = 1, prob = acceptP)

    if (res == 1) {
      ## contract
      # print("death succeed=====>")
      # cat("death prob = ", probAlpha, "====>\n")

      # stop("=====>")

      thetaYList <- leftThetaY
      qVec <- qVecNew
      clusInd <- clusIndNew
      thetaYList <- clear_inactive_components(thetaYList, clusInd, mVec[2])
    } else {
      # print("death stay =====>")
    }
  }

  else if (indChar == "combine") {
    ## choose two  positions to combine
    combineClusInd <- sample(x = sort(unique(ZOneDim)), size = 2, prob = 1 / table(ZOneDim))

    logProbCombineZ <- log(1 / choose(length(unique(ZOneDim)), 2))

    clusIndNew <- clusInd
    qVecNew <- qVec
    clusIndNew[combineClusInd] <- 0
    qVecNew[combineClusInd] <- 0

    ## choose one position to fill combined clus
    combinedClusInd <- which(clusIndNew == 0)[1]
    clusIndNew[combinedClusInd] <- 1
    qVecNew[combinedClusInd] <- qnew

    ## propose values for new clusters
    combinedThetaYList <- propose_combined_clusters(X, thetaYList, hparam, combineClusInd, combinedClusInd, qVecNew, constraint)
    combinedThetaYList <- clear_inactive_components(combinedThetaYList, clusIndNew, mVec[2])
    combineEval <- evaluate_combined_clusters(X, combinedThetaYList, hparam, combinedClusInd, qVecNew, constraint)
    ## evaluate values for old cluster
    splitEval <- evaluate_split_clusters(X, combinedThetaYList, thetaYList, hparam, combinedClusInd, combineClusInd, qVec, constraint)


    ## calculate Jacobian

    JacEval <- calculate_jacobian(p, combinedThetaYList, thetaYList, combinedClusInd, combineClusInd)


    ## combine Z

    ZOneDimNew <- combine_allocations(ZOneDim, combineClusInd, combinedClusInd)

    ## eval split

    evalZProb <- evaluate_split_allocations(X, ZOneDimNew, ZOneDim, thetaYList, combinedClusInd, combineClusInd)

    # update Y
    combinedThetaYList@Y <- update_latent_scores(X, combinedThetaYList, ZOneDimNew, clusIndNew, qVecNew)



    combinedThetaYListTEST <- combinedThetaYList
    ZOneDimNewTEST <- ZOneDimNew
    for (i in 1:5) {
      MCMCobj <- stay_mcmc_update(X, combinedThetaYListTEST, ZOneDimNewTEST, hparam, qVecNew, qnew, dVec, sVec, constraint, clusIndNew)
      ZOneDimNewTEST <- MCMCobj$ZOneDim
      combinedThetaYListTEST <- MCMCobj$thetaYList
      # hparam = MCMCobj$hparam
    }


    oldDensity <- likelihood(thetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, clusInd)
    newDensity <- likelihood(combinedThetaYListTEST, ZOneDimNewTEST, qVecNew, muBar, X) + evaluate_prior(m - 1, p, muBar, hparam, combinedThetaYListTEST, ZOneDimNewTEST, qVecNew, constraint, clusIndNew)

    numer <- newDensity + splitEval + JacEval
    denom <- oldDensity + combineEval + logProbCombineZ

    # cat("combine diff = ", newDensity - oldDensity, "====>\n")
    # cat("combine diff = ", numer - denom, "====>\n")

    probAlpha <- calculate_ratio_cpp(numer, denom)
    acceptP <- min(1, probAlpha)
    res <- rbinom(1, size = 1, prob = acceptP)

    if (res == 1) {
      # print("combine success=====>")

      thetaYList <- combinedThetaYListTEST
      ZOneDim <- ZOneDimNewTEST
      clusInd <- clusIndNew
      qVec <- qVecNew
      thetaYList <- clear_inactive_components(thetaYList, clusInd, mVec[2])
    } else {
      # print("combine fail=====>")
    }
  }

  else if (indChar == "split") {
    ## choose one existing position to split
    if (length(unique(ZOneDim)) == 1) {
      splitClusInd <- unique(ZOneDim)
    } else {
      splitClusInd <- sample(sort(unique(ZOneDim)), size = 1, prob = table(ZOneDim))
    }
    logProbSplitZ <- log(1 / length(unique(ZOneDim)))

    clusIndNew <- clusInd
    qVecNew <- qVec
    clusIndNew[splitClusInd] <- 0
    qVecNew[splitClusInd] <- 0
    ## choose two positions to fill the two clusters
    splitedClusInd <- which(clusIndNew == 0)[1:2]
    clusIndNew[splitedClusInd] <- 1
    qVecNew[splitedClusInd] <- qnew
    ## propose values for new clusters

    splitedThetaYList <- propose_split_clusters(X, thetaYList, hparam, splitClusInd, splitedClusInd, qVecNew, constraint)
    splitEval <- evaluate_split_clusters(X, thetaYList, splitedThetaYList, hparam, splitClusInd, splitedClusInd, qVecNew, constraint)

    ## calculate Jacobian

    JacEval <- calculate_jacobian(p, thetaYList, splitedThetaYList, splitClusInd, splitedClusInd)

    ## evaluate values for old cluster
    combineEval <- evaluate_combined_clusters(X, thetaYList, hparam, splitClusInd, qVec, constraint)

    ## split Z

    splitZObj <- split_allocations(X, ZOneDim, splitedThetaYList, splitClusInd, splitedClusInd)

    ZOneDimNew <- splitZObj$ZOneDim
    evalZProb <- splitZObj$evalProb

    # update Y
    splitedThetaYList@Y <- update_latent_scores(X, splitedThetaYList, ZOneDimNew, clusIndNew, qVecNew)

    splitedThetaYListTEST <- splitedThetaYList
    ZOneDimNewTEST <- ZOneDimNew
    for (i in 1:5) {
      MCMCobj <- stay_mcmc_update(X, splitedThetaYListTEST, ZOneDimNewTEST, hparam, qVecNew, qnew, dVec, sVec, constraint, clusIndNew)
      ZOneDimNewTEST <- MCMCobj$ZOneDim
      splitedThetaYListTEST <- MCMCobj$thetaYList
    }

    ## calculate acceptancy propability

    oldDensity <- likelihood(thetaYList, ZOneDim, qVec, muBar, X) + evaluate_prior(m, p, muBar, hparam, thetaYList, ZOneDim, qVec, constraint, clusInd)
    newDensity <- likelihood(splitedThetaYListTEST, ZOneDimNewTEST, qVecNew, muBar, X) + evaluate_prior(m + 1, p, muBar, hparam, splitedThetaYListTEST, ZOneDimNewTEST, qVecNew, constraint, clusIndNew)

    numer <- newDensity + combineEval
    denom <- oldDensity + splitEval + logProbSplitZ + JacEval

    # cat("diff  = ", numer - denom, "====>\n")
    probAlpha <- calculate_ratio_cpp(numer, denom)
    acceptP <- min(1, probAlpha)
    res <- rbinom(1, size = 1, prob = acceptP)

    if (res == 1) {
      # print("split success=====>")
      # cat("split prob = ", probAlpha, "====>\n")


      thetaYList <- splitedThetaYListTEST
      ZOneDim <- ZOneDimNewTEST
      clusInd <- clusIndNew
      qVec <- qVecNew
      thetaYList <- clear_inactive_components(thetaYList, clusInd, mVec[2])
    } else {
      # print("split fail=====>")
    }
  }

  return(list(thetaYList = thetaYList, ZOneDim = ZOneDim, hparam = hparam, qVec = qVec, clusInd = clusInd))
}

utils::globalVariables(c(
  "mVec", "generate_new_cluster_parameters", "evaluate_new_cluster_parameters",
  "propose_combined_clusters", "evaluate_combined_clusters",
  "evaluate_split_clusters", "calculate_jacobian",
  "combine_allocations", "evaluate_split_allocations", "update_latent_scores",
  "propose_split_clusters", "split_allocations", "res"
))

#' Run Multiple Independent Bayesian PGMM Chains
#'
#' Runs independent `pgmm_rjmcmc()` chains, optionally in parallel. This is the
#' safest way to use multiple CPU cores because each MCMC iteration depends on
#' the previous state, while independent chains can be evaluated concurrently.
#'
#' @inheritParams pgmm_rjmcmc
#' @param chains positive integer giving the number of independent chains.
#' @param cores positive integer giving the number of worker processes to use.
#'   Values greater than `chains` are reduced to `chains`.
#' @param seed optional integer seed used to generate deterministic per-chain
#'   seeds.
#'
#' @return A list with one fitted `pgmm_rjmcmc()` result per chain. The result
#'   has class `bpgmm_rjmcmc_chains` and stores the per-chain seeds in the
#'   `chain_seeds` attribute.
#' @export
pgmm_rjmcmc_chains <- function(X,
                               m_init,
                               m_range,
                               q_new,
                               delta = 2,
                               ggamma = 2,
                               burn = 20,
                               niter = 1000,
                               constraint = c(0, 0, 0),
                               d_vec = c(1, 1, 1),
                               s_vec = c(1, 1, 1),
                               m_step = 0,
                               v_step = 0,
                               split_combine = 0,
                               verbose = FALSE,
                               chains = 2,
                               cores = min(chains, available_cores()),
                               seed = NULL) {
  chains <- validate_positive_integer(chains, "chains")
  cores <- validate_positive_integer(cores, "cores")
  cores <- min(cores, chains)

  if (!is.null(seed)) {
    if (!isTRUE(length(seed) == 1L) || !is.finite(seed)) {
      stop("seed must be NULL or a finite scalar", call. = FALSE)
    }
    old_seed <- if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    } else {
      NULL
    }
    on.exit(restore_random_seed(old_seed), add = TRUE)
    set.seed(as.integer(seed))
  }

  chain_seeds <- sample.int(.Machine$integer.max, chains)
  run_chain <- function(chain_id) {
    set.seed(chain_seeds[[chain_id]])
    pgmm_rjmcmc(
      X = X,
      m_init = m_init,
      m_range = m_range,
      q_new = q_new,
      delta = delta,
      ggamma = ggamma,
      burn = burn,
      niter = niter,
      constraint = constraint,
      d_vec = d_vec,
      s_vec = s_vec,
      m_step = m_step,
      v_step = v_step,
      split_combine = split_combine,
      verbose = verbose
    )
  }

  fits <- if (cores == 1L) {
    lapply(seq_len(chains), run_chain)
  } else if (.Platform$OS.type == "windows") {
    cluster <- parallel::makeCluster(cores)
    on.exit(parallel::stopCluster(cluster), add = TRUE)
    parallel::clusterEvalQ(cluster, library(bpgmm))
    parallel::clusterExport(
      cluster,
      varlist = c(
        "X", "m_init", "m_range", "q_new", "delta", "ggamma", "burn",
        "niter", "constraint", "d_vec", "s_vec", "m_step", "v_step",
        "split_combine", "verbose", "chain_seeds"
      ),
      envir = environment()
    )
    parallel::parLapply(cluster, seq_len(chains), function(chain_id) {
      set.seed(chain_seeds[[chain_id]])
      bpgmm::pgmm_rjmcmc(
        X = X,
        m_init = m_init,
        m_range = m_range,
        q_new = q_new,
        delta = delta,
        ggamma = ggamma,
        burn = burn,
        niter = niter,
        constraint = constraint,
        d_vec = d_vec,
        s_vec = s_vec,
        m_step = m_step,
        v_step = v_step,
        split_combine = split_combine,
        verbose = verbose
      )
    })
  } else {
    parallel::mclapply(seq_len(chains), run_chain, mc.cores = cores)
  }

  names(fits) <- paste0("chain_", seq_len(chains))
  attr(fits, "chain_seeds") <- chain_seeds
  class(fits) <- c("bpgmm_rjmcmc_chains", class(fits))
  fits
}

test_that("pgmm_rjmcmc_chains validates parallel arguments", {
  x <- matrix(rnorm(12), nrow = 2)

  expect_error(
    pgmm_rjmcmc_chains(x, m_init = 1, m_range = c(1, 2), q_new = 1, chains = 0),
    "chains"
  )
  expect_error(
    pgmm_rjmcmc_chains(x, m_init = 1, m_range = c(1, 2), q_new = 1, cores = 0),
    "cores"
  )
  expect_error(
    pgmm_rjmcmc_chains(x, m_init = 1, m_range = c(1, 2), q_new = 1, seed = NA),
    "seed"
  )
})

test_that("pgmm_rjmcmc_chains runs deterministic independent chains", {
  set.seed(2026)
  x <- cbind(
    matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
    matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
  )

  fit_a <- pgmm_rjmcmc_chains(
    x,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 0,
    niter = 1,
    chains = 2,
    cores = 1,
    seed = 42,
    verbose = FALSE
  )
  fit_b <- pgmm_rjmcmc_chains(
    x,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 0,
    niter = 1,
    chains = 2,
    cores = 1,
    seed = 42,
    verbose = FALSE
  )

  expect_s3_class(fit_a, "bpgmm_rjmcmc_chains")
  expect_length(fit_a, 2)
  expect_named(fit_a, c("chain_1", "chain_2"))
  expect_length(attr(fit_a, "chain_seeds"), 2)
  expect_equal(attr(fit_a, "chain_seeds"), attr(fit_b, "chain_seeds"))
  expect_equal(fit_a[[1]]$allocation_samples, fit_b[[1]]$allocation_samples)
  expect_equal(fit_a[[2]]$allocation_samples, fit_b[[2]]$allocation_samples)
})

#' Convert PGMM Constraint Codes to Paper Model Names
#'
#' The paper represents the eight PGMM covariance structures with three-letter
#' model names. Each letter is either `C` for constrained or `U` for
#' unconstrained. The letters indicate whether the loading matrix is shared
#' across clusters, whether the noise covariance is shared across clusters, and
#' whether the noise covariance is isotropic within clusters.
#'
#' @param constraint Integer or numeric vector of length three with entries
#'   `0` or `1`. `1` maps to `C`; `0` maps to `U`.
#'
#' @return A character scalar, one of `CCC`, `CCU`, `CUC`, `CUU`, `UCC`,
#'   `UCU`, `UUC`, or `UUU`.
#' @export
constraint_to_model <- function(constraint) {
  if (!is.numeric(constraint) && !is.integer(constraint)) {
    stop("constraint must be a numeric or integer vector", call. = FALSE)
  }
  if (length(constraint) != 3) {
    stop("constraint must have length 3", call. = FALSE)
  }
  if (any(is.na(constraint)) || any(!(constraint %in% c(0, 1)))) {
    stop("constraint entries must be 0/1 values", call. = FALSE)
  }

  paste0(ifelse(as.integer(constraint) == 1L, "C", "U"), collapse = "")
}

#' Convert PGMM Paper Model Names to Constraint Codes
#'
#' @param model Character scalar naming one of the eight PGMM covariance
#'   structures: `CCC`, `CCU`, `CUC`, `CUU`, `UCC`, `UCU`, `UUC`, or `UUU`.
#'
#' @return Integer vector of length three. `1` means constrained and `0` means
#'   unconstrained.
#' @export
model_to_constraint <- function(model) {
  valid_models <- c("CCC", "CCU", "CUC", "CUU", "UCC", "UCU", "UUC", "UUU")

  if (!is.character(model) || length(model) != 1L || is.na(model)) {
    stop("model must be a character scalar", call. = FALSE)
  }

  model <- toupper(model)
  if (!(model %in% valid_models)) {
    stop(
      "model must be one of: ",
      paste(valid_models, collapse = ", "),
      call. = FALSE
    )
  }

  as.integer(strsplit(chartr("CU", "10", model), "", fixed = TRUE)[[1]])
}

test_that("constraint helpers map between paper model names and binary codes", {
  expect_equal(model_to_constraint("CCC"), c(1, 1, 1))
  expect_equal(model_to_constraint("CCU"), c(1, 1, 0))
  expect_equal(model_to_constraint("CUC"), c(1, 0, 1))
  expect_equal(model_to_constraint("CUU"), c(1, 0, 0))
  expect_equal(model_to_constraint("UCC"), c(0, 1, 1))
  expect_equal(model_to_constraint("UCU"), c(0, 1, 0))
  expect_equal(model_to_constraint("UUC"), c(0, 0, 1))
  expect_equal(model_to_constraint("UUU"), c(0, 0, 0))

  expect_equal(constraint_to_model(c(1, 1, 1)), "CCC")
  expect_equal(constraint_to_model(c(0, 0, 0)), "UUU")
  expect_equal(bpgmm:::change_constraint_format("(1, 0, 0)"), "CUU")
})

test_that("constraint helpers reject invalid encodings", {
  expect_error(model_to_constraint("ABC"), "one of")
  expect_error(constraint_to_model(c(1, 0)), "length 3")
  expect_error(constraint_to_model(c(1, 0, 2)), "0/1")
})

test_that("pgmm_rjmcmc default constraint is a numeric vector", {
  x <- matrix(rnorm(12), nrow = 2)

  expect_error(
    pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, burn = 0, niter = 1, verbose = FALSE),
    NA
  )
})

test_that("pgmm_rjmcmc accepts paper model labels for constraints", {
  x <- matrix(rnorm(12), nrow = 2)

  fit <- pgmm_rjmcmc(
    x,
    m_init = 1,
    m_range = c(1, 2),
    q_new = 1,
    burn = 0,
    niter = 1,
    constraint = "UUU",
    verbose = FALSE
  )

  expect_equal(fit$constraint_samples[[1]], model_to_constraint("UUU"))
})

test_that("pgmm_rjmcmc honors zero posterior iterations", {
  set.seed(2026)
  x <- cbind(
    matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
    matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
  )

  fit <- pgmm_rjmcmc(
    x,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 0,
    niter = 0,
    verbose = FALSE
  )

  expect_length(fit$allocation_samples, 0)
  expect_length(fit$constraint_samples, 0)
  expect_length(fit$alpha1_samples, 0)
})

test_that("pgmm_rjmcmc can run cluster-number and covariance RJMCMC steps", {
  set.seed(7)
  x <- cbind(
    matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
    matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
  )

  fit <- pgmm_rjmcmc(
    x,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 0,
    niter = 1,
    m_step = 1,
    v_step = 1,
    verbose = FALSE
  )

  expect_length(fit$allocation_samples, 1)
  expect_length(fit$active_cluster_samples, 1)
  expect_length(fit$constraint_samples, 1)
})

test_that("pgmm_rjmcmc can run split/combine RJMCMC moves", {
  set.seed(1)
  x <- cbind(
    matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
    matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
  )

  fit <- pgmm_rjmcmc(
    x,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 0,
    niter = 1,
    m_step = 1,
    split_combine = 1,
    verbose = FALSE
  )

  expect_length(fit$allocation_samples, 1)
  expect_length(fit$active_cluster_samples, 1)
})

test_that("pgmm_rjmcmc validates user-facing inputs", {
  x <- matrix(rnorm(12), nrow = 2)

  expect_error(pgmm_rjmcmc(x, m_init = 0, m_range = c(1, 2), q_new = 1), "m_init")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(2, 1), q_new = 1), "m_range")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 0), "q_new")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, burn = -1), "burn")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, niter = -1), "niter")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, constraint = c(0, 0)), "constraint")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, m_step = 2), "m_step")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, v_step = NA), "v_step")
  expect_error(pgmm_rjmcmc(x, m_init = 1, m_range = c(1, 2), q_new = 1, split_combine = -1), "split_combine")
  expect_error(pgmm_rjmcmc(as.data.frame(x), m_init = 1, m_range = c(1, 2), q_new = 1), "X")
})

#' (internal)
#' @noRd
#' @noRd
update_post_theta_y <- function(m, n, p, hparam, thetaYList, ZOneDim, qVec, constraint, X, ggamma) {
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2
  bbeta <- hparam@bbeta
  lambda <- thetaYList@lambda
  Y <- thetaYList@Y
  M <- thetaYList@M
  psy <- thetaYList@psy

  ## post for Theta = {tao, M, Lambda, psy}
  CxyList <- calculate_cxy(m, n, hparam, thetaYList, ZOneDim, qVec, X)
  Cxxk <- CxyList$Cxxk
  Cxyk <- CxyList$Cxyk
  Cyyk <- CxyList$Cyyk
  Cytytk <- CxyList$Cytytk
  Cxtytk <- CxyList$Cxtytk
  CxL1k <- CxyList$CxL1k
  Cxmyk <- CxyList$Cxmyk
  sumCxmyk <- CxyList$sumCxmyk
  sumCyyk <- CxyList$sumCyyk
  A <- CxyList$A
  nVec <- CxyList$nVec

  Zmat <- get_z_mat_r(ZOneDim, m, n)


  # post tao
  tao <- rdirichlet(1, nVec + ggamma)

  #  post mu
  M <- list()
  for (k in 1:m) {
    M[[k]] <- rmvnorm(1, mean = CxL1k[[k]] * (sum(Zmat[k, ]) + alpha1)^(-1), sigma = (sum(Zmat[k, ]) + alpha1)^(-1) * psy[[k]])
  }

  ## lambda; psy
  lambdaPsy <- calculate_post_lambda_psi(m, p, hparam, CxyList, thetaYList, qVec, constraint)
  # lambdaPsy = CalculatePostLambdaPsy(alpha1, alpha2, bbeta, CxyList, M, psy, constraint)
  lambda <- lambdaPsy$lambda
  psy <- lambdaPsy$psy

  ## post Y
  theta_for_y <- new("ThetaYList", tao = tao, psy = psy, M = M, lambda = lambda, Y = Y)
  Y <- update_latent_scores_cpp(X, theta_for_y, ZOneDim, rep(1, m), qVec)

  new("ThetaYList", tao = tao, psy = psy, M = M, lambda = lambda, Y = Y)
}

#' Bayesian Model-Based Clustering with Parsimonious Gaussian Mixture Models
#'
#' Carries out model-based clustering using parsimonious Gaussian mixture
#' models. MCMC is used for parameter estimation and RJMCMC is used for model
#' selection.
#'
#' The `constraint` argument follows the three-letter PGMM model notation used
#' in Lu, Li, and Love (2021). The first entry indicates whether loading
#' matrices are shared across clusters, the second whether noise covariance
#' matrices are shared across clusters, and the third whether the noise
#' covariance is isotropic within each cluster. Use [model_to_constraint()] to
#' convert model names such as `CCC`, `CCU`, `CUC`, `CUU`, `UCC`, `UCU`, `UUC`,
#' and `UUU` into the numeric vector used internally.
#'
#' @import stats MASS mcmcse pgmm label.switching fabMix
#' @param X the observation matrix with variables in rows and observations in
#'   columns.
#' @param m_init the number of initial clusters.
#' @param m_range the allowed range for the number of clusters.
#' @param q_new the number of latent factors for a new cluster.
#' @param delta scalar hyperparameter for the noise covariance prior
#' @param ggamma scalar hyperparameter used in covariance-structure proposals
#' @param burn the number of burn-in iterations
#' @param niter the number of posterior sampling iterations
#' @param constraint initial PGMM covariance constraint. Use a three-letter
#'   model label such as `"CCC"` or `"UUU"`, or a numeric vector of length
#'   three with binary entries. For example, `c(1, 1, 1)` is `CCC`, the fully
#'   constrained model, and `c(0, 0, 0)` is `UUU`, the fully unconstrained
#'   model.
#' @param d_vec a vector of hyperparameters with length three, shape parameters
#'   for alpha1, alpha2 and bbeta respectively
#' @param s_vec a vector of hyperparameters with length three, rate parameters
#'   for alpha1, alpha2 and bbeta respectively
#' @param m_step indicator for RJMCMC model selection on the number of clusters.
#' @param v_step indicator for RJMCMC model selection on covariance structures.
#' @param split_combine indicator for using split/combine moves in the
#'   cluster-number RJMCMC step.
#' @param verbose logical; if `TRUE`, print iteration progress.
#'
#' @return A list of posterior samples with snake_case fields:
#'   `tau_samples`, `psi_samples`, `mean_samples`, `lambda_samples`,
#'   `factor_score_samples`, `allocation_samples`, `constraint_samples`,
#'   `alpha1_samples`, `alpha2_samples`, `beta_samples`, and
#'   `active_cluster_samples`.
#' @name pgmm_rjmcmc
#' @export

pgmm_rjmcmc <- function(X,
                       m_init,
                       m_range,
                       q_new,
                       delta = 2,
                       ggamma = 2,
                       burn = 20,
                       niter = 1000,
                       constraint = c(0, 0, 0),
                       d_vec = c(1, 1, 1),
                       s_vec = c(1, 1, 1),
                       m_step = 0,
                       v_step = 0,
                       split_combine = 0,
                       verbose = TRUE) {
  if (!is.matrix(X) || !is.numeric(X)) {
    stop("X must be a numeric matrix with variables in rows and observations in columns", call. = FALSE)
  }
  if (any(!is.finite(X))) {
    stop("X must contain only finite numeric values", call. = FALSE)
  }
  if (!isTRUE(length(m_range) == 2L) || any(!is.finite(m_range)) || any(m_range < 1) || m_range[1] > m_range[2]) {
    stop("m_range must be a length-two increasing positive numeric vector", call. = FALSE)
  }
  if (!isTRUE(length(m_init) == 1L) || !is.finite(m_init) || m_init < 1) {
    stop("m_init must be a positive scalar", call. = FALSE)
  }
  if (m_init < m_range[1] || m_init > m_range[2]) {
    stop("m_init must lie within m_range", call. = FALSE)
  }
  if (m_init > ncol(X)) {
    stop("m_init cannot exceed the number of observations in X", call. = FALSE)
  }
  if (!isTRUE(length(q_new) == 1L) || !is.finite(q_new) || q_new < 1) {
    stop("q_new must be a positive scalar", call. = FALSE)
  }
  if (!isTRUE(length(burn) == 1L) || !is.finite(burn) || burn < 0) {
    stop("burn must be a non-negative scalar", call. = FALSE)
  }
  if (!isTRUE(length(niter) == 1L) || !is.finite(niter) || niter < 0) {
    stop("niter must be a non-negative scalar", call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("verbose must be TRUE or FALSE", call. = FALSE)
  }
  if (!isTRUE(length(m_step) == 1L) || is.na(m_step) || !(m_step %in% c(0, 1))) {
    stop("m_step must be 0 or 1", call. = FALSE)
  }
  if (!isTRUE(length(v_step) == 1L) || is.na(v_step) || !(v_step %in% c(0, 1))) {
    stop("v_step must be 0 or 1", call. = FALSE)
  }
  if (!isTRUE(length(split_combine) == 1L) || is.na(split_combine) || !(split_combine %in% c(0, 1))) {
    stop("split_combine must be 0 or 1", call. = FALSE)
  }

  m_init <- as.integer(m_init)
  m_range <- as.integer(m_range)
  q_new <- as.integer(q_new)
  burn <- as.integer(burn)
  niter <- as.integer(niter)
  m_step <- as.integer(m_step)
  v_step <- as.integer(v_step)
  split_combine <- as.integer(split_combine)
  if (is.character(constraint)) {
    constraint <- model_to_constraint(constraint)
  } else {
    constraint <- model_to_constraint(constraint_to_model(constraint))
  }

  n <- ncol(X)
  p <- nrow(X)

  alpha1 <- rgamma(1, d_vec[1], s_vec[1])
  alpha2 <- rgamma(1, d_vec[2], s_vec[2])
  bbeta <- rgamma(1, d_vec[3], s_vec[3])

  hparam <- new("Hparam", alpha1 = alpha1, alpha2 = alpha2, bbeta = bbeta, delta = delta, ggamma = ggamma)

  hparamInit <- hparam

  muBar <- X[, sample.int(n, 1)]

  ## cluster indicator
  clusInd <- rep(0, m_range[2])
  clusInd[1:m_init] <- 1

  ## qinit
  qVec <- rep(0, m_range[2])
  qVec[1:m_init] <- q_new


  ## priors
  ZOneDim <- kmeans(x = t(X), centers = m_init)$cluster
  thetaYList <- generate_prior_theta_y(m_init, n, p, muBar, hparam, qVec, ZOneDim, constraint)

  ## burn in
  for (i in seq_len(burn)) {
    MCMCobj <- stay_mcmc_update(X, thetaYList, ZOneDim, hparam, qVec, q_new, d_vec, s_vec, constraint, clusInd)
    ZOneDim <- MCMCobj$ZOneDim
    thetaYList <- MCMCobj$thetaYList
    hparam <- MCMCobj$hparam
    hparam@alpha2 <- max(0.01, hparam@alpha2)
  }

  thetaYList <- clear_inactive_components(thetaYList, clusInd, m_range[2])
  ##
  alpha1_samples <- c()
  alpha2_samples <- c()
  beta_samples <- c()
  tau_samples <- list()
  psi_samples <- list()
  mean_samples <- list()
  lambda_samples <- list()
  factor_score_samples <- list()
  allocation_samples <- list()
  constraint_samples <- list()
  active_cluster_samples <- list()
  ##

  for (h in seq_len(niter)) {
    if (verbose) {
      cat("iter = ", h, "======>\n")
    }

    ## choose m or choose v
    if (m_step == 1) {
      MCMCobj <- mstep_rjmcmc_update(X, muBar, p, thetaYList, ZOneDim, hparam, hparamInit, qVec, q_new, d_vec, s_vec, constraint, clusInd, m_range, "BD")
      ZOneDim <- MCMCobj$ZOneDim
      thetaYList <- MCMCobj$thetaYList
      hparam <- MCMCobj$hparam
      qVec <- MCMCobj$qVec
      clusInd <- MCMCobj$clusInd
      ##
      if (split_combine == 1) {
        MCMCobj <- mstep_rjmcmc_update(X, muBar, p, thetaYList, ZOneDim, hparam, hparamInit, qVec, q_new, d_vec, s_vec, constraint, clusInd, m_range, "SC")
        ZOneDim <- MCMCobj$ZOneDim
        thetaYList <- MCMCobj$thetaYList
        hparam <- MCMCobj$hparam
        qVec <- MCMCobj$qVec
        clusInd <- MCMCobj$clusInd
      }
    }

    if (v_step == 1) {
      MCMCobj <- vstep_rjmcmc_update(X, muBar, p, thetaYList, ZOneDim, hparam, hparamInit, qVec, q_new, ggamma, d_vec, s_vec, constraint, clusInd)
      ZOneDim <- MCMCobj$ZOneDim
      thetaYList <- MCMCobj$thetaYList
      hparam <- MCMCobj$hparam
      constraint <- MCMCobj$constraint
    }

    # stay step
    MCMCobj <- stay_mcmc_update(X, thetaYList, ZOneDim, hparam, qVec, q_new, d_vec, s_vec, constraint, clusInd)
    ZOneDim <- MCMCobj$ZOneDim
    thetaYList <- MCMCobj$thetaYList
    hparam <- MCMCobj$hparam
    hparam@alpha2 <- max(0.01, hparam@alpha2)

    ## save
    active_cluster_samples[[h]] <- clusInd
    alpha1_samples[h] <- hparam@alpha1
    alpha2_samples[h] <- hparam@alpha2
    beta_samples[h] <- hparam@bbeta
    tau_samples[[h]] <- thetaYList@tao
    psi_samples[[h]] <- thetaYList@psy
    mean_samples[[h]] <- thetaYList@M
    lambda_samples[[h]] <- thetaYList@lambda
    factor_score_samples[[h]] <- thetaYList@Y
    allocation_samples[[h]] <- ZOneDim
    constraint_samples[[h]] <- constraint
  }

  list(
    tau_samples = tau_samples,
    psi_samples = psi_samples,
    mean_samples = mean_samples,
    lambda_samples = lambda_samples,
    factor_score_samples = factor_score_samples,
    allocation_samples = allocation_samples,
    constraint_samples = constraint_samples,
    alpha1_samples = alpha1_samples,
    alpha2_samples = alpha2_samples,
    beta_samples = beta_samples,
    active_cluster_samples = active_cluster_samples
  )
}

#' @import methods
#' @name ThetaYList
#' @title ThetaYList-class
#' @description Parameter set for sampled PGMM component parameters.
#'
#' @slot tao Numeric vector of component mixing weights.
#' @slot psy List of diagonal noise covariance matrices.
#' @slot M List of component mean vectors.
#' @slot lambda List of component factor loading matrices.
#' @slot Y List of latent factor score matrices.
#' @keywords internal
setClass(
  "ThetaYList",
  slots = c(
    tao = "vector",
    psy = "list",
    M = "list",
    lambda = "list",
    Y = "list"
  ),
  prototype = list(
    tao = c(),
    psy = list(),
    M = list(),
    lambda = list(),
    Y = list()
  )
)

setValidity("ThetaYList", function(object) {
  if (any(object@tao > 1) | any(object@tao < 0)) {
    "@tao should be in range 0 to 1"
  }
})


#' #' Getter
#' #' @import methods
#' #' @name tao
#' #' @title tao-getter
#' #' @aliases tao
#' #' @description Definition of parameter sets
#' setGeneric("tao", function(x) standardGeneric("tao"))
#' setMethod("tao", "ThetaYList", function(x) x@tao)
#' setGeneric("psy", function(x) standardGeneric("psy"))
#' setMethod("psy", "ThetaYList", function(x) x@psy)
#' setGeneric("M", function(x) standardGeneric("M"))
#' setMethod("M", "ThetaYList", function(x) x@M)
#' setGeneric("lambda", function(x) standardGeneric("lambda"))
#' setMethod("lambda", "ThetaYList", function(x) x@lambda)
#' setGeneric("Y", function(x) standardGeneric("Y"))
#' setMethod("Y", "ThetaYList", function(x) x@Y)
#'
#' #' Setter with validation
#' setGeneric("tao<-", function(x, value) standardGeneric("tao<-"))
#' setMethod("tao<-", "ThetaYList", function(x, value) {
#'   x@tao <- value
#'   validObject(x)
#'   x
#' })
#' setGeneric("psy<-", function(x, value) standardGeneric("psy<-"))
#' setMethod("psy<-", "ThetaYList", function(x, value) {
#'   x@psy <- value
#'   validObject(x)
#'   x
#' })
#' setGeneric("M<-", function(x, value) standardGeneric("M<-"))
#' setMethod("M<-", "ThetaYList", function(x, value) {
#'   x@M <- value
#'   validObject(x)
#'   x
#' })
#' setGeneric("lambda<-", function(x, value) standardGeneric("lambda<-"))
#' setMethod("lambda<-", "ThetaYList", function(x, value) {
#'   x@lambda <- value
#'   validObject(x)
#'   x
#' })
#' setGeneric("Y<-", function(x, value) standardGeneric("Y<-"))
#' setMethod("Y<-", "ThetaYList", function(x, value) {
#'   x@Y <- value
#'   validObject(x)
#'   x
#' })

#' propose_split_clusters
#'
#' @param X X
#' @param thetaYList thetaYList
#' @param hparam hparam
#' @param splitClusInd splitClusInd
#' @param splitedClusInd splitedClusInd
#' @param qVec qVec
#' @param constraint constraint
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' splitClusInd <- 1
#' splitedClusInd <- c(1, 2)
#' propose_split_clusters(X, thetaYList, hparam, splitClusInd, splitedClusInd, qVec, constraint)
#' @noRd
propose_split_clusters <- function(X, thetaYList, hparam, splitClusInd, splitedClusInd, qVec, constraint) {
  p <- nrow(X)
  n <- ncol(X)

  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2


  resThetaYList <- thetaYList

  ## beta(2,2) is symatric between (0,1)
  a1 <- rbeta(1, 2, 2)
  # a2 = rbeta(p, 2, 2)
  a2 <- rgamma(p, 1, 2)


  # print(a1)
  # print("===========propose>")
  # print(a2)
  # print("===========propose>")
  ## propose weight
  w1 <- thetaYList@tao[splitClusInd] * a1
  w2 <- thetaYList@tao[splitClusInd] * (1 - a1)

  resThetaYList@tao[splitedClusInd] <- c(w1, w2)

  ## propose mean
  splitMu <- thetaYList@M[[splitClusInd]]
  ## calculate split clus Variance
  splitClusVar <- thetaYList@psy[[splitClusInd]] + thetaYList@lambda[[splitClusInd]] %*% t(thetaYList@lambda[[splitClusInd]])
  splitClusSDvec <- sqrt(diag(splitClusVar))
  # print("=======")
  # print(splitClusSDvec)
  mu1 <- c()
  mu2 <- c()

  for (i in 1:p) {
    signInd <- sample(c(-1, 1), 1)
    mu1[i] <- splitMu[i] + signInd * a2[i] * splitClusSDvec[i] * sqrt(w2 / w1)
    mu2[i] <- splitMu[i] + -1 * signInd * a2[i] * splitClusSDvec[i] * sqrt(w1 / w2)
  }

  # print(mu1)
  # print(mu2)
  # print(splitMu)
  # print("=======>")

  resThetaYList@M[[splitedClusInd[1]]] <- t(mu1)
  resThetaYList@M[[splitedClusInd[2]]] <- t(mu2)

  ## propose lambda and psy
  if (constraint[2] == 1) {
    psy1 <- thetaYList@psy[[splitClusInd]]
    psy2 <- thetaYList@psy[[splitClusInd]]
    psy <- list(psy1, psy2)
  } else {
    psy <- generate_prior_psi(p, 2, delta, bbeta, constraint)
    psy1 <- psy[[1]]
    psy2 <- psy[[2]]
  }

  resThetaYList@psy[[splitedClusInd[1]]] <- psy1
  resThetaYList@psy[[splitedClusInd[2]]] <- psy2


  if (constraint[1] == 1) {
    lambda1 <- thetaYList@lambda[[splitClusInd]]
    lambda2 <- thetaYList@lambda[[splitClusInd]]
  } else {
    lambda <- generate_prior_lambda(p, 2, alpha2, qVec[splitedClusInd], psy, constraint)
    lambda1 <- lambda[[1]]
    lambda2 <- lambda[[2]]
  }

  resThetaYList@lambda[[splitedClusInd[1]]] <- lambda1
  resThetaYList@lambda[[splitedClusInd[2]]] <- lambda2

  ## propose Y
  for (k in splitedClusInd) {
    resThetaYList@Y[[k]] <- matrix(NA, qVec[k], n)
    for (i in 1:n) {
      resThetaYList@Y[[k]][, i] <- mvtnorm::rmvnorm(1, mean = rep(0, qVec[k]), sigma = diag(qVec[k]))
    }
  }
  return(resThetaYList)
}



#' evaluate_split_clusters
#'
#' @param X X
#' @param thetaYList thetaYList
#' @param splitedThetaYList splitedThetaYList
#' @param hparam hparam
#' @param splitClusInd splitClusInd
#' @param splitedClusInd splitedClusInd
#' @param qVec qVec
#' @param constraint constraint
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 1
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' splitedThetaYList <- thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' qnew <- 1
#' dVec <- c(1, 1, 1)
#' sVec <- c(1, 1, 1)
#' splitClusInd <- 1
#' splitedClusInd <- c(1, 1)
#' evaluate_split_clusters(X, thetaYList, splitedThetaYList, hparam, splitClusInd, splitedClusInd, qVec, constraint)
#' @noRd
evaluate_split_clusters <- function(X, thetaYList, splitedThetaYList, hparam, splitClusInd, splitedClusInd, qVec, constraint) {
  p <- nrow(X)
  n <- ncol(X)

  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  ## evaluate a1
  w1 <- splitedThetaYList@tao[splitedClusInd[1]]
  w2 <- splitedThetaYList@tao[splitedClusInd[2]]
  w <- thetaYList@tao[splitClusInd]


  EVALa1 <- dbeta(w1 / w, 2, 2, log = T)

  ## evaluate a2

  splitMu <- thetaYList@M[[splitClusInd]]
  ## calculate split clus Variance

  splitClusVar <- thetaYList@psy[[splitClusInd]] + thetaYList@lambda[[splitClusInd]] %*% t(thetaYList@lambda[[splitClusInd]])
  splitClusSDvec <- sqrt(diag(splitClusVar))
  # print("=======")
  # print(splitClusSDvec)

  mu1 <- splitedThetaYList@M[[splitedClusInd[1]]]
  mu2 <- splitedThetaYList@M[[splitedClusInd[2]]]

  # print(mu1)
  # print(mu2)
  # print(splitMu)
  # print("=======>")
  a2 <- c()
  EVALa2 <- c()
  for (i in 1:p) {
    a2[i] <- abs((mu1[i] - splitMu[i]) / (splitClusSDvec[i] * sqrt(w2 / w1)))
    # a2[i] = abs( (mu2[i] - splitMu[i]) /(splitClusSDvec[i] * sqrt(w1/w2)) )

    # EVALa2[i] = dbeta(a2[i],2,2, log = T)
    # EVALa2[i] = dbeta(a2[i],1,1, log = T)

    EVALa2[i] <- dgamma(a2[i], 1, 2, log = T)
  }
  # print(a2)
  # print(EVALa2)
  # x = rbeta(10000, 2,2)
  # y = x/(1-x)* rgamma(10000,1,2)
  #
  # hist(sqrt(y))
  #
  # hist(rgamma(10000, 2,4))
  # hist(rbeta(10000, 2,2))

  # print(w1/w)
  # print("===========evaluate>")
  # print(a2)
  # print("===========evaluate>")
  ## eval  psy
  psy1 <- splitedThetaYList@psy[[splitedClusInd[1]]]
  psy2 <- splitedThetaYList@psy[[splitedClusInd[2]]]
  psy <- list(psy1, psy2)
  evalpsy <- 0
  if (constraint[2] == 0) {
    evalpsy <- evaluate_prior_psi(psy, p, 2, delta, bbeta, constraint, c(1, 1))
  }

  ## eval lambda
  lambda1 <- splitedThetaYList@lambda[[splitedClusInd[1]]]
  lambda2 <- splitedThetaYList@lambda[[splitedClusInd[2]]]
  lambda <- list(lambda1, lambda2)
  evallambda <- 0
  if (constraint[1] == 0) {
    evallambda <- evaluate_prior_lambda(p, 2, alpha2, qVec[splitedClusInd], psy, lambda, constraint, c(1, 1))
  }

  # signEval = log(0.5 ^ p)
  # print(c(EVALa1, EVALa2, evalpsy, evallambda))
  return(sum(EVALa1, EVALa2, evalpsy, evallambda))
}


#' split_allocations
#'
#' @param ZOneDim ZOneDim
#' @param splitedThetaYList splitedThetaYList
#' @param splitClusInd splitClusInd
#' @param splitedClusInd splitedClusInd
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 2
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' splitedThetaYList <-
#'   new("ThetaYList",
#'     tao = c(0.90162050961987, 0.0983794903801295),
#'     psy = list(
#'       structure(c(3.68472841602225, 0, 0, 8.34691978354054), .Dim = c(2L, 2L)),
#'       structure(c(0.785011896130842, 0, 0, 1.19022383323437), .Dim = c(2L, 2L))
#'     ),
#'     M = list(structure(c(
#'       2.96424305287004,
#'       1.08454861414306
#'     ), .Dim = 1:2), structure(c(
#'       -0.232625450433964,
#'       0.984505960868685
#'     ), .Dim = 1:2)), lambda = list(structure(c(
#'       -0.964026624054337,
#'       0.89378616732449
#'     ), .Dim = 2:1), structure(c(
#'       0.533334148228635,
#'       -1.80033696090263
#'     ), .Dim = 2:1)), Y = list(structure(c(
#'       -0.15346475266988,
#'       1.6584112693271, 0.409294936277862, -1.46628591247549, -0.532753243163142,
#'       -0.332143130316749, 0.307558110800446, -0.525374243612587, 0.527667526535661,
#'       0.748193650431916
#'     ), .Dim = c(1L, 10L)), structure(c(
#'       0.571325118638535,
#'       0.542462985882966, 0.559971315637159, -1.73905343105432, -0.583549598471542,
#'       1.71264245945391, -0.327119395945831, 1.02464651767821, -1.11462280255215,
#'       0.81095592501554
#'     ), .Dim = c(1L, 10L)))
#'   )
#' splitClusInd <- 1
#' splitedClusInd <- c(1, 2)
#' split_allocations(ZOneDim, splitedThetaYList, splitClusInd, splitedClusInd)
#' @noRd
split_allocations <- function(X, ZOneDim, splitedThetaYList, splitClusInd, splitedClusInd) {
  resZOneDim <- ZOneDim
  evalProb <- c()

  splitObsInd <- which(ZOneDim == splitClusInd)
  for (j in 1:length(splitObsInd)) {
    # print(j)
    i <- splitObsInd[j]

    p1 <- log(splitedThetaYList@tao[splitedClusInd[1]]) + mvtnorm::dmvnorm(X[, i],
      mean = splitedThetaYList@M[[splitedClusInd[1]]],
      sigma = splitedThetaYList@psy[[splitedClusInd[1]]] + splitedThetaYList@lambda[[splitedClusInd[1]]] %*% t(splitedThetaYList@lambda[[splitedClusInd[1]]]),
      log = T
    )
    # p1 =   mvtnorm::dmvnorm(X[,i], mean = splitedThetaYList@M[[splitedClusInd[1]]],
    #                                                               sigma = splitedThetaYList@psy[[splitedClusInd[1]]] + splitedThetaYList@lambda[[splitedClusInd[1]]]%*%t(splitedThetaYList@lambda[[splitedClusInd[1]]]),
    #                                                               log = T)

    p2 <- log(splitedThetaYList@tao[splitedClusInd[2]]) + mvtnorm::dmvnorm(X[, i],
      mean = splitedThetaYList@M[[splitedClusInd[2]]],
      sigma = splitedThetaYList@psy[[splitedClusInd[2]]] + splitedThetaYList@lambda[[splitedClusInd[2]]] %*% t(splitedThetaYList@lambda[[splitedClusInd[2]]]),
      log = T
    )
    # p2 =  mvtnorm::dmvnorm(X[,i], mean = splitedThetaYList@M[[splitedClusInd[2]]],
    #                                                               sigma = splitedThetaYList@psy[[splitedClusInd[2]]] + splitedThetaYList@lambda[[splitedClusInd[2]]]%*%t(splitedThetaYList@lambda[[splitedClusInd[2]]]),
    #                                                               log = T)
    # cat(" for sub", i,"===\n")
    # cat("p1 = ", p1, "===\n")
    # cat("p2 = ", p2, "===\n")
    # print("=======>")
    if (is.infinite(p1)) {
      print("===infi")
      p1 <- -1e-10
    }
    if (is.infinite(p2)) {
      print("===infi")
      p2 <- -1e-10
    }
    # print(c(p1,p2))
    prob <- calculate_ratio_cpp(p1, c(p1, p2))
    # cat("prob = ", prob, "====>\n")
    probVec <- c(prob, 1 - prob)

    Zind <- sample(x = 1:2, size = 1, prob = probVec)
    evalProb[j] <- probVec[Zind]
    resZOneDim[i] <- splitedClusInd[Zind]
  }
  return(list(ZOneDim = resZOneDim, evalProb = sum(log(evalProb))))
}



#' evaluate_split_allocations
#'
#' @param ZOneDimSplit ZOneDimSplit
#' @param ZOneDimSplited ZOneDimSplited
#' @param splitedThetaYList splitedThetaYList
#' @param splitClusInd splitClusInd
#' @param splitedClusInd splitedClusInd
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 2
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDimSplit <- sample(seq_len(m), n, replace = TRUE)
#' ZOneDimSplit[ZOneDimSplit == 2] <- 1
#' constraint <- c(0, 0, 0)
#' splitedThetaYList <- new("ThetaYList", tao = c(0.90162050961987, 0.0983794903801295), psy = list(structure(c(3.68472841602225, 0, 0, 8.34691978354054), .Dim = c(2L, 2L)), structure(c(0.785011896130842, 0, 0, 1.19022383323437), .Dim = c(2L, 2L))), M = list(structure(c(
#'   2.96424305287004,
#'   1.08454861414306
#' ), .Dim = 1:2), structure(c(
#'   -0.232625450433964,
#'   0.984505960868685
#' ), .Dim = 1:2)), lambda = list(structure(c(
#'   -0.964026624054337,
#'   0.89378616732449
#' ), .Dim = 2:1), structure(c(
#'   0.533334148228635,
#'   -1.80033696090263
#' ), .Dim = 2:1)), Y = list(structure(c(
#'   -0.15346475266988,
#'   1.6584112693271, 0.409294936277862, -1.46628591247549, -0.532753243163142,
#'   -0.332143130316749, 0.307558110800446, -0.525374243612587, 0.527667526535661,
#'   0.748193650431916
#' ), .Dim = c(1L, 10L)), structure(c(
#'   0.571325118638535,
#'   0.542462985882966, 0.559971315637159, -1.73905343105432, -0.583549598471542,
#'   1.71264245945391, -0.327119395945831, 1.02464651767821, -1.11462280255215,
#'   0.81095592501554
#' ), .Dim = c(1L, 10L))))
#' splitClusInd <- 1
#' splitedClusInd <- c(1, 2)
#' ZOneDimSplited <- sample(seq_len(m), n, replace = TRUE)
#' evaluate_split_allocations(ZOneDimSplit, ZOneDimSplited, splitedThetaYList, splitClusInd, splitedClusInd)
#' @noRd
evaluate_split_allocations <- function(X, ZOneDimSplit, ZOneDimSplited, splitedThetaYList, splitClusInd, splitedClusInd) {
  evalProb <- c()

  splitObsInd <- which(ZOneDimSplit == splitClusInd)
  for (j in 1:length(splitObsInd)) {
    i <- splitObsInd[j]

    p1 <- log(splitedThetaYList@tao[splitedClusInd[1]]) + mvtnorm::dmvnorm(X[, i],
      mean = splitedThetaYList@M[[splitedClusInd[1]]],
      sigma = splitedThetaYList@psy[[splitedClusInd[1]]] + splitedThetaYList@lambda[[splitedClusInd[1]]] %*% t(splitedThetaYList@lambda[[splitedClusInd[1]]]),
      log = T
    )


    p2 <- log(splitedThetaYList@tao[splitedClusInd[2]]) + mvtnorm::dmvnorm(X[, i],
      mean = splitedThetaYList@M[[splitedClusInd[2]]],
      sigma = splitedThetaYList@psy[[splitedClusInd[2]]] + splitedThetaYList@lambda[[splitedClusInd[2]]] %*% t(splitedThetaYList@lambda[[splitedClusInd[2]]]),
      log = T
    )

    prob <- calculate_ratio_cpp(p1, c(p1, p2))
    # cat("prob = ", prob, "====>\n")
    probVec <- c(prob, 1 - prob)
    # print(prob)
    Zind <- which(splitedClusInd == ZOneDimSplited[i])
    # Zind = sample(x = 1:2,size = 1, prob = probVec)
    evalProb[j] <- probVec[Zind]
  }
  return(sum(log(evalProb)))
}


#' calculate_jacobian
#'
#' @param p p
#' @param thetaYList thetaYList
#' @param splitedThetaYList splitedThetaYList
#' @param splitClusInd splitClusInd
#' @param splitedClusInd splitedClusInd
#'
#' @noRd
#'
#' @examples
#' set.seed(100)
#' n <- 10
#' p <- 2
#' q <- 1
#' K <- 2
#' m <- 2
#' muBar <- c(0, 0)
#' qVec <- c(1, 1)
#' constraint <- c(0, 0, 0)
#' X <- t(
#'   fabMix::simData(
#'     sameLambda = TRUE,
#'     sameSigma = TRUE,
#'     K.true = K,
#'     n = n,
#'     q = q,
#'     p = p,
#'     sINV_values = 1 / ((1:p))
#'   )$data
#' )
#' hparam <- new(
#'   "Hparam",
#'   alpha1 = 0.567755037123148,
#'   alpha2 = 1.1870201935945,
#'   delta = 2,
#'   ggamma = 2,
#'   bbeta = 3.39466184520673
#' )
#' ZOneDim <- sample(seq_len(m), n, replace = TRUE)
#' thetaYList <-
#'   new(
#'     "ThetaYList",
#'     tao = 0.366618687752634,
#'     psy = list(structure(
#'       c(
#'         4.18375613018654,
#'         0, 0, 5.46215996830771
#'       ),
#'       .Dim = c(2L, 2L)
#'     )),
#'     M = list(structure(
#'       c(
#'         3.27412045866392,
#'         -2.40544145363349
#'       ),
#'       .Dim = 1:2
#'     )),
#'     lambda = list(structure(
#'       c(
#'         2.51015961514781,
#'         -0.0741189919182549
#'       ),
#'       .Dim = 2:1
#'     )),
#'     Y = list(structure(
#'       c(
#'         -0.244239011725104,
#'         -0.26876172736886,
#'         0.193431511203083,
#'         0.41624466812811,
#'         -0.54581548068437,
#'         -0.0479517628308146,
#'         -0.633383997203325,
#'         0.856855296613208,
#'         0.792850576988512,
#'         0.268208848994559
#'       ),
#'       .Dim = c(1L, 10L)
#'     ))
#'   )
#' splitedThetaYList <- new("ThetaYList", tao = c(0.90162050961987, 0.0983794903801295), psy = list(structure(c(3.68472841602225, 0, 0, 8.34691978354054), .Dim = c(2L, 2L)), structure(c(0.785011896130842, 0, 0, 1.19022383323437), .Dim = c(2L, 2L))), M = list(structure(c(
#'   2.96424305287004,
#'   1.08454861414306
#' ), .Dim = 1:2), structure(c(
#'   -0.232625450433964,
#'   0.984505960868685
#' ), .Dim = 1:2)), lambda = list(structure(c(
#'   -0.964026624054337,
#'   0.89378616732449
#' ), .Dim = 2:1), structure(c(
#'   0.533334148228635,
#'   -1.80033696090263
#' ), .Dim = 2:1)), Y = list(structure(c(
#'   -0.15346475266988,
#'   1.6584112693271, 0.409294936277862, -1.46628591247549, -0.532753243163142,
#'   -0.332143130316749, 0.307558110800446, -0.525374243612587, 0.527667526535661,
#'   0.748193650431916
#' ), .Dim = c(1L, 10L)), structure(c(
#'   0.571325118638535,
#'   0.542462985882966, 0.559971315637159, -1.73905343105432, -0.583549598471542,
#'   1.71264245945391, -0.327119395945831, 1.02464651767821, -1.11462280255215,
#'   0.81095592501554
#' ), .Dim = c(1L, 10L))))
#' splitClusInd <- 1
#' splitedClusInd <- c(1, 2)
#' calculate_jacobian(p, thetaYList, splitedThetaYList, splitClusInd, splitedClusInd)
#' @noRd
calculate_jacobian <- function(p, thetaYList, splitedThetaYList, splitClusInd, splitedClusInd) {
  w1 <- splitedThetaYList@tao[splitedClusInd[1]]
  w2 <- splitedThetaYList@tao[splitedClusInd[2]]
  w <- thetaYList@tao[splitClusInd]

  splitClusVar <- thetaYList@psy[[splitClusInd]] + thetaYList@lambda[[splitClusInd]] %*% t(thetaYList@lambda[[splitClusInd]])
  splitClusSDvec <- sqrt(diag(splitClusVar))

  mu1 <- splitedThetaYList@M[[splitedClusInd[1]]]
  mu2 <- splitedThetaYList@M[[splitedClusInd[2]]]
  splitMu <- thetaYList@M[[splitClusInd]]

  a2 <- c()
  for (i in 1:p) {
    a2[i] <- abs((mu1[i] - splitMu[i]) / (splitClusSDvec[i] * sqrt(w2 / w1)))
  }

  res <- log(w) + sum(log(abs((mu1 - mu2) / a2)))
  return(res)
}

#' PriorThetaY list
#' @description generate prior value for parameter Theta and Y.
#' @importFrom gtools rdirichlet
#' @importFrom mvtnorm rmvnorm
#' @import stats
#' @param m the number of cluster
#' @param n sample size
#' @param p number of covariates
#' @param muBar parameter
#' @param hparam hyperparameters
#' @param qVec the vector of the number of factors in each clusters
#' @param ZOneDim ZOneDim
#' @param constraint constraint
#' @noRd
generate_prior_theta_y <- function(m,
                                n,
                                p,
                                muBar,
                                hparam,
                                qVec,
                                ZOneDim,
                                constraint) {
  ggamma <- hparam@ggamma
  delta <- hparam@delta
  bbeta <- hparam@bbeta
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  # prior tao
  if (m == 1) {
    tao <- rbeta(n = 1, shape1 = 1, shape2 = m)
  } else {
    tao <- gtools::rdirichlet(n = 1, alpha = rep(ggamma, m))
  }

  # prior psy
  psy <- generate_prior_psi(p, m, delta, bbeta, constraint)

  # prior M
  M <- list()
  for (i in 1:m) {
    M[[i]] <- (mvtnorm::rmvnorm(n = 1, mean = muBar, sigma = 1 / alpha1 * psy[[i]]))
  }

  # prior lambda
  lambda <- generate_prior_lambda(p, m, alpha2, qVec, psy, constraint)


  # Zmat = get_z_mat_r(ZOneDim, m, n)
  Zmat <- get_z_mat_cpp(ZOneDim, m, n)

  # post Y
  Y <- list()
  for (k in 1:m) {
    Y[[k]] <- matrix(NA, qVec[k], n)
    for (i in 1:n) {
      Y[[k]][, i] <- mvtnorm::rmvnorm(1, mean = rep(0, qVec[k]), sigma = diag(qVec[k]))
    }
  }

  new("ThetaYList",
    tao = tao,
    psy = psy,
    M = M,
    lambda = lambda,
    Y = Y
  )
}

#' (internal)
#' @noRd
#' @noRd
evaluate_prior <- function(m,
                          p,
                          muBar,
                          hparam,
                          thetaYList,
                          ZOneDim,
                          qVec,
                          constraint,
                          clusInd) {
  ggamma <- hparam@ggamma
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2
  bbeta <- hparam@bbeta
  delta <- hparam@delta

  loopm <- which(clusInd == 1)

  ## 2.2: Y
  # Yval = 0
  # for(k in loopm){
  #   Yval = Yval + sum(dnorm(thetaYList@Y[[k]], log = T))
  # }


  ## 2.3: Z
  # adjustTao = thetaYList@tao/sum(thetaYList@tao, na.rm = T)

  Zval <- sum(log(thetaYList@tao[ZOneDim]), na.rm = TRUE)
  # Zval = log(sum(adjustTao[ZOneDim], na.rm = T))
  ## 2.6: tao
  # taoVal = log(gtools::ddirichlet(x = adjustTao[!is.na(adjustTao)], alpha = rep(ggamma, m)))
  # print(thetaYList@tao[clusInd == 1])
  # print(rep(ggamma, m))

  taoVal <- log(gtools::ddirichlet(x = thetaYList@tao[clusInd == 1], alpha = rep(ggamma, m)))

  ## test
  # Zval = 0;taoVal = 0
  ##


  ## 2.7: M
  Mval <- 0
  for (k in loopm) {
    # cat("k =", k , "===> \n")
    Mval <- Mval + mvtnorm::dmvnorm(c(thetaYList@M[[k]]), mean = c(muBar), sigma = 1 / alpha1 * thetaYList@psy[[k]], log = T)
  }

  ## 2.8: lambda
  lambdaVal <- evaluate_prior_lambda(
    p, m, alpha2, qVec, thetaYList@psy, thetaYList@lambda, constraint,
    clusInd
  )

  ## 2.9: psy
  psyVal <- evaluate_prior_psi(thetaYList@psy, p, m, delta, bbeta, constraint, clusInd)


  # print(Zval)
  # print(taoVal)
  # print(Mval)
  # print(lambdaVal)
  # print(psyVal)
  # totalVal = sum(Yval + Zval + taoVal + Mval + lambdaVal + psyVal)


  # print(c( Zval,taoVal,Mval,lambdaVal,psyVal))
  totalVal <- sum(Zval + taoVal + Mval + lambdaVal + psyVal)
  return(totalVal)
}
#' generate_prior_psi
#'
#' @description generate prior value for parameter Psi
#' @import stats
#' @param p the number of features
#' @param m the number of clusters
#' @param delta hyperparameters
#' @param bbeta hyperparameters
#' @param constraint the pgmm constraint, a vector of length three with binary entry. For example, c(1,1,1) means the fully constraint model
#'
#' @noRd
generate_prior_psi <- function(p,
                             m,
                             delta,
                             bbeta,
                             constraint) {
  psy <- list()

  if (constraint[2] == T & constraint[3] == T) {
    for (i in 1:m) {
      if (i == 1) {
        psyValue <- 1 / rgamma(1, shape = delta, rate = bbeta)
        psy[[i]] <- diag(rep(psyValue, p))
      } else {
        psy[[i]] <- psy[[1]]
      }
    }
  } else if (constraint[2] == T & constraint[3] == F) {
    for (i in 1:m) {
      if (i == 1) {
        psy[[i]] <- diag(1 / rgamma(p, shape = delta, rate = bbeta))
      } else {
        psy[[i]] <- psy[[1]]
      }
    }
  } else if (constraint[2] == F & constraint[3] == T) {
    for (i in 1:m) {
      psyValue <- 1 / rgamma(1, shape = delta, rate = bbeta)
      psy[[i]] <- diag(rep(psyValue, p))
    }
  } else {
    for (i in 1:m) {
      psy[[i]] <- diag(1 / rgamma(p, shape = delta, rate = bbeta))
    }
  }
  return(psy)
}

#' (internal)
#' @noRd
#' @noRd
evaluate_prior_psi <- function(psy,
                             p,
                             m,
                             delta,
                             bbeta,
                             constraint,
                             clusInd) {
  evaluate_prior_psi_cpp(psy, p, m, delta, bbeta, constraint, clusInd)
}


#' generate_prior_lambda
#'
#' @description evaluate prior value for parameter Lambda
#' @importFrom mvtnorm rmvnorm
#' @param p the number of features
#' @param m the number of clusters
#' @param alpha2 hyper parameter
#' @param qVec parameter
#' @param psy parameter
#' @param constraint parameter
#'
#' @noRd
generate_prior_lambda <- function(p,
                                m,
                                alpha2,
                                qVec,
                                psy,
                                constraint) {
  lambda <- list()
  if (constraint[1] == T & constraint[2] == T) {
    for (k in 1:m) {
      if (k == 1) {
        qk <- qVec[k]
        lambda[[k]] <- matrix(0, p, qk)
        for (j in 1:qk) {
          lambda[[k]][, j] <- mvtnorm::rmvnorm(1, rep(0, p), 1 / alpha2 * psy[[k]])
        }
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }
  } else if (constraint[1] == T & constraint[2] == F) {
    psyAve <- matrix(0, nrow = p, ncol = p)
    for (k in 1:m) {
      psyAve <- psyAve + solve(psy[[k]])
    }
    psyAve <- solve(1 / m * psyAve)

    for (k in 1:m) {
      if (k == 1) {
        qk <- qVec[k]
        lambda[[k]] <- matrix(0, p, qk)
        for (j in 1:qk) {
          lambda[[k]][, j] <- mvtnorm::rmvnorm(1, rep(0, p), 1 / alpha2 * psyAve)
        }
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }
  } else {
    for (k in 1:m) {
      qk <- qVec[k]
      lambda[[k]] <- matrix(0, p, qk)
      for (j in 1:qk) {
        lambda[[k]][, j] <- mvtnorm::rmvnorm(1, rep(0, p), 1 / alpha2 * psy[[k]])
      }
    }
  }
  return(lambda)
}

#' (internal)
#' @noRd
#' @noRd
evaluate_prior_lambda <- function(p,
                                m,
                                alpha2,
                                qVec,
                                psy,
                                lambda,
                                constraint,
                                clusInd) {
  evaluate_prior_lambda_cpp(p, m, alpha2, qVec, psy, lambda, constraint, clusInd)
}

test_theta <- function(n = 4) {
  new(
    "ThetaYList",
    tao = c(0.55, 0.45),
    psy = list(diag(c(1.2, 1.4)), diag(c(1.1, 1.3))),
    M = list(matrix(c(-0.5, 0.25), nrow = 1), matrix(c(0.75, -0.2), nrow = 1)),
    lambda = list(matrix(c(0.2, -0.1), nrow = 2), matrix(c(-0.15, 0.25), nrow = 2)),
    Y = list(
      matrix(c(-0.4, 0.1, 0.3, -0.2), nrow = 1, ncol = n),
      matrix(c(0.2, -0.3, 0.4, 0.1), nrow = 1, ncol = n)
    )
  )
}

test_hparam <- function() {
  new("Hparam", alpha1 = 2, alpha2 = 3, bbeta = 2, delta = 3, ggamma = 1)
}

test_constraints <- list(
  c(TRUE, TRUE, TRUE),
  c(TRUE, TRUE, FALSE),
  c(TRUE, FALSE, TRUE),
  c(TRUE, FALSE, FALSE),
  c(FALSE, TRUE, TRUE),
  c(FALSE, TRUE, FALSE),
  c(FALSE, FALSE, TRUE),
  c(FALSE, FALSE, FALSE)
)

test_that("prior generators produce valid constrained parameters", {
  p <- 2
  m <- 2
  q_vec <- c(1, 1)

  for (constraint in test_constraints) {
    set.seed(100)
    psy <- bpgmm:::generate_prior_psi(p, m, delta = 3, bbeta = 2, constraint)
    expect_length(psy, m)
    expect_true(all(vapply(psy, function(x) all(dim(x) == c(p, p)) && all(diag(x) > 0), logical(1))))

    lambda <- bpgmm:::generate_prior_lambda(p, m, alpha2 = 3, q_vec, psy, constraint)
    expect_length(lambda, m)
    expect_true(all(vapply(lambda, function(x) all(dim(x) == c(p, 1)), logical(1))))
  }
})

test_that("prior evaluators return finite log densities across constraints", {
  p <- 2
  m <- 2
  theta <- test_theta()
  q_vec <- c(1, 1)
  clus_ind <- c(1, 1)

  for (constraint in test_constraints) {
    expect_true(is.finite(bpgmm:::evaluate_prior_psi(theta@psy, p, m, 3, 2, constraint, clus_ind)))
    expect_true(is.finite(bpgmm:::evaluate_prior_lambda(
      p, m, alpha2 = 3, qVec = q_vec, psy = theta@psy, lambda = theta@lambda,
      constraint = constraint, clusInd = clus_ind
    )))
  }
})

test_that("joint prior uses product of allocation probabilities", {
  p <- 2
  m <- 2
  theta <- test_theta()
  theta@tao <- c(0.8, 0.2)
  hparam <- test_hparam()
  q_vec <- c(1, 1)
  clus_ind <- c(1, 1)
  constraint <- c(FALSE, FALSE, FALSE)

  z_same <- c(1, 1, 1, 1)
  z_mixed <- c(1, 1, 2, 2)

  same_prior <- bpgmm:::evaluate_prior(m, p, c(0, 0), hparam, theta, z_same, q_vec, constraint, clus_ind)
  mixed_prior <- bpgmm:::evaluate_prior(m, p, c(0, 0), hparam, theta, z_mixed, q_vec, constraint, clus_ind)

  expect_equal(
    same_prior - mixed_prior,
    sum(log(theta@tao[z_same])) - sum(log(theta@tao[z_mixed])),
    tolerance = 1e-10
  )
})

test_that("proposal generators and evaluators cover all PGMM covariance constraints", {
  p <- 2
  m <- 2
  n <- 4
  q_vec <- c(1, 1)
  x <- matrix(c(-0.7, 0.1, -0.2, 0.4, 0.8, -0.3, 1.1, 0.2), nrow = p)
  z <- c(1, 1, 2, 2)
  theta <- test_theta(n)
  hparam <- test_hparam()
  cxy <- bpgmm:::calculate_cxy(m, n, hparam, theta, z, q_vec, x)

  for (constraint in test_constraints) {
    set.seed(100)
    post <- bpgmm:::calculate_post_lambda_psi(m, p, hparam, cxy, theta, q_vec, constraint)
    expect_named(post, c("lambda", "psy"))
    expect_length(post$lambda, m)
    expect_length(post$psy, m)
    expect_true(all(vapply(post$lambda, function(x) all(dim(x) == c(p, 1)), logical(1))))
    expect_true(all(vapply(post$psy, function(x) all(dim(x) == c(p, p)) && all(diag(x) > 0), logical(1))))

    set.seed(101)
    lambda <- bpgmm:::calculate_proposal_lambda(hparam, theta, cxy, constraint, m, p, q_vec)
    expect_length(lambda, m)
    expect_true(all(vapply(lambda, function(x) all(dim(x) == c(p, 1)), logical(1))))
    expect_true(is.finite(bpgmm:::evaluate_proposal_lambda(hparam, theta, cxy, constraint, lambda, m, q_vec, p)))

    theta_lambda <- theta
    theta_lambda@lambda <- lambda
    cxy_lambda <- bpgmm:::calculate_cxy(m, n, hparam, theta_lambda, z, q_vec, x)

    set.seed(102)
    psy <- bpgmm:::calculate_proposal_psi(hparam, theta_lambda, cxy_lambda, constraint, m, p, q_vec)
    expect_length(psy, m)
    expect_true(all(vapply(psy, function(x) all(dim(x) == c(p, p)) && all(diag(x) > 0), logical(1))))
    expect_true(is.finite(bpgmm:::evaluate_proposal_psi(hparam, theta_lambda, cxy_lambda, constraint, psy, m, p, q_vec)))
  }
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calculate_cxy_native <- function(m, n, hparam, theta_y_list, z, q_vec, X) {
    .Call(`_bpgmm_calculate_cxy_native`, m, n, hparam, theta_y_list, z, q_vec, X)
}

calculate_post_lambda_psi_native <- function(m, p, hparam, cxy_list, theta_y_list, q_vec, constraint) {
    .Call(`_bpgmm_calculate_post_lambda_psi_native`, m, p, hparam, cxy_list, theta_y_list, q_vec, constraint)
}

evaluate_prior_psi_native <- function(psy, p, m, delta, bbeta, constraint, clus_ind) {
    .Call(`_bpgmm_evaluate_prior_psi_native`, psy, p, m, delta, bbeta, constraint, clus_ind)
}

evaluate_prior_lambda_native <- function(p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind) {
    .Call(`_bpgmm_evaluate_prior_lambda_native`, p, m, alpha2, q_vec, psy, lambda, constraint, clus_ind)
}

update_latent_scores_native <- function(X, theta_y_list, z, clus_ind, q_vec) {
    .Call(`_bpgmm_update_latent_scores_native`, X, theta_y_list, z, clus_ind, q_vec)
}

update_post_z_native <- function(X, m, n, theta_y_list) {
    .Call(`_bpgmm_update_post_z_native`, X, m, n, theta_y_list)
}

update_hyperparameter_native <- function(m, p, q, hparam, theta_y_list, d_vec, s_vec) {
    .Call(`_bpgmm_update_hyperparameter_native`, m, p, q, hparam, theta_y_list, d_vec, s_vec)
}

get_z_matrix_native <- function(z, m, n) {
    .Call(`_bpgmm_get_z_matrix_native`, z, m, n)
}

multivariate_normal_density_native <- function(x, mean, sigma, logd) {
    .Call(`_bpgmm_multivariate_normal_density_native`, x, mean, sigma, logd)
}

calculate_ratio_native <- function(log_denominator, log_numerator) {
    .Call(`_bpgmm_calculate_ratio_native`, log_denominator, log_numerator)
}

#' calculate_proposal_lambda
#'
#' @param hparam hparam
#' @param thetaYList thetaYList
#' @param CxyList CxyList
#' @param constraint constraint
#' @param m the number of clusters
#' @param qVec the vector of the number of factors in each clusters
#' @param p the number of features
#'
#' @noRd
calculate_proposal_lambda <- function(hparam, thetaYList, CxyList, constraint, m, p, qVec) {
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  M <- thetaYList@M
  psy <- thetaYList@psy
  ##
  Cxxk <- CxyList$Cxxk
  Cxyk <- CxyList$Cxyk
  Cyyk <- CxyList$Cyyk
  Cytytk <- CxyList$Cytytk
  Cxtytk <- CxyList$Cxtytk
  CxL1k <- CxyList$CxL1k
  Cxmyk <- CxyList$Cxmyk
  sumCxmyk <- CxyList$sumCxmyk
  sumCyyk <- CxyList$sumCyyk
  A <- CxyList$A
  nVec <- CxyList$nVec

  ##
  lambda <- list()
  if (constraint[1] == T & constraint[2] == T & constraint[3] == T) {
    ## model 1

    sumCxmyk <- 0
    sumCyyk <- 0
    for (k in 1:m) {
      sumCxmyk <- sumCxmyk + Cxmyk[[k]]
      sumCyyk <- sumCyyk + Cyyk[[k]] + alpha2 / m * diag(qVec[k])
    }

    for (k in 1:m) {
      if (k == 1) {
        lambda[[k]] <- mvtnorm::rmvnorm(1,
          mean = c(sumCxmyk %*% solve(sumCyyk)),
          sigma = kronecker(solve(sumCyyk), psy[[k]])
        )
        lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }
    ## model 1 end
  } else if (constraint[1] == T & constraint[2] == T & constraint[3] == F) {
    ## model 2

    sumCxmyk <- 0
    sumCyyk <- 0
    for (k in 1:m) {
      sumCxmyk <- sumCxmyk + Cxmyk[[k]]
      sumCyyk <- sumCyyk + Cyyk[[k]] + alpha2 / m * diag(qVec[k])
    }
    for (k in 1:m) {
      if (k == 1) {
        lambda[[k]] <- mvtnorm::rmvnorm(1,
          mean = c(sumCxmyk %*% solve(sumCyyk)),
          sigma = kronecker(solve(sumCyyk), psy[[k]])
        )
        lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }

    ## end model 2
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == T) {
    ## model 3


    sumPhiCxy <- 0
    sumPhiCyy <- 0

    for (k in 1:m) {
      sumPhiCxy <- sumPhiCxy + 1 / psy[[k]][1, 1] * Cxmyk[[k]]
      sumPhiCyy <- sumPhiCyy + 1 / psy[[k]][1, 1] * (Cyyk[[k]] + alpha2 / m * diag(qVec[k]))
    }

    for (k in 1:m) {
      if (k == 1) {
        lambda[[k]] <- mvtnorm::rmvnorm(1,
          mean = c(sumPhiCxy %*% solve(sumPhiCyy)),
          sigma = kronecker(solve(sumPhiCyy), diag(p))
        )

        lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }

    ## end model 3
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == F) {
    ## model 4
    sumVar <- 0
    B <- 0
    for (k in 1:m) {
      sumVar <- sumVar + kronecker(
        Cyyk[[k]] + alpha2 / m * diag(qVec[k]),
        solve(psy[[k]])
      )
      B <- B + solve(psy[[k]]) %*% Cxmyk[[k]]
    }
    lambdaVar <- solve(sumVar)
    lambdaMean <- t(c(B)) %*% lambdaVar
    for (k in 1:m) {
      if (k == 1) {
        lambda[[k]] <- mvtnorm::rmvnorm(1,
          mean = lambdaMean,
          sigma = lambdaVar
        )

        lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
      } else {
        lambda[[k]] <- lambda[[1]]
      }
    }
    ## end model 4
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == T) {
    ## model 5
    for (k in 1:m) {
      lambda[[k]] <- mvtnorm::rmvnorm(1,
        mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(Cyyk[[k]] + alpha2 * diag(qVec[k])), psy[[k]])
      )
      lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
    }

    ## end model 5
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == F) {
    ## model 6
    for (k in 1:m) {
      lambda[[k]] <- mvtnorm::rmvnorm(1,
        mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]])
      )
      lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
    }

    ## end model 6
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == T) {
    ## model 7
    for (k in 1:m) {
      lambda[[k]] <- mvtnorm::rmvnorm(1,
        mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]])
      )
      lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
    }
    ## end model 7
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == F) {
    ## model 8
    for (k in 1:m) {
      lambda[[k]] <- mvtnorm::rmvnorm(1,
        mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]])
      )
      lambda[[k]] <- matrix(lambda[[k]], p, qVec[k])
    }
    ## end model 8
  }
  return(lambda)
}

#' evaluate_proposal_lambda
#'
#' @param hparam hparam
#' @param thetaYList thetaYList
#' @param CxyList CxyList
#' @param constraint constraint
#' @param newlambda newlambda
#' @param m the number of clusters
#' @param qVec the vector of the number of factors in each clusters
#' @param p the number of features
#'
#' @noRd
evaluate_proposal_lambda <- function(hparam, thetaYList, CxyList, constraint, newlambda, m, qVec, p) {
  alpha1 <- hparam@alpha1
  alpha2 <- hparam@alpha2

  M <- thetaYList@M
  psy <- thetaYList@psy
  lambda <- newlambda
  ##
  Cxxk <- CxyList$Cxxk
  Cxyk <- CxyList$Cxyk
  Cyyk <- CxyList$Cyyk
  Cytytk <- CxyList$Cytytk
  Cxtytk <- CxyList$Cxtytk
  CxL1k <- CxyList$CxL1k
  Cxmyk <- CxyList$Cxmyk
  sumCxmyk <- CxyList$sumCxmyk
  sumCyyk <- CxyList$sumCyyk
  A <- CxyList$A
  nVec <- CxyList$nVec

  ##
  lambdaEval <- c()
  if (constraint[1] == T & constraint[2] == T & constraint[3] == T) {
    ## model 1

    sumCxmyk <- 0
    sumCyyk <- 0
    for (k in 1:m) {
      sumCxmyk <- sumCxmyk + Cxmyk[[k]]
      sumCyyk <- sumCyyk + Cyyk[[k]] + alpha2 / m * diag(qVec[k])
    }

    for (k in 1:m) {
      if (k == 1) {
        lambdaEval[k] <- mvtnorm::dmvnorm(
          x = c(lambda[[k]]), mean = c(sumCxmyk %*% solve(sumCyyk)),
          sigma = kronecker(solve(sumCyyk), psy[[k]]), log = T
        )
      } else {
        lambdaEval[k] <- 0
      }
    }
    ## model 1 end
  } else if (constraint[1] == T & constraint[2] == T & constraint[3] == F) {
    ## model 2

    sumCxmyk <- 0
    sumCyyk <- 0
    for (k in 1:m) {
      sumCxmyk <- sumCxmyk + Cxmyk[[k]]
      sumCyyk <- sumCyyk + Cyyk[[k]] + alpha2 / m * diag(qVec[k])
    }

    for (k in 1:m) {
      if (k == 1) {
        lambdaEval[k] <- mvtnorm::dmvnorm(
          x = c(lambda[[k]]), mean = c(sumCxmyk %*% solve(sumCyyk)),
          sigma = kronecker(solve(sumCyyk), psy[[k]]),
          log = T
        )
      } else {
        lambdaEval[k] <- 0
      }
    }

    ## end model 2
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == T) {
    ## model 3

    sumPhiCxy <- 0
    sumPhiCyy <- 0

    for (k in 1:m) {
      sumPhiCxy <- sumPhiCxy + 1 / psy[[k]][1, 1] * Cxmyk[[k]]
      sumPhiCyy <- sumPhiCyy + 1 / psy[[k]][1, 1] * (Cyyk[[k]] + alpha2 / m * diag(qVec[k]))
    }

    for (k in 1:m) {
      if (k == 1) {
        lambdaEval[k] <- mvtnorm::dmvnorm(
          x = c(lambda[[k]]), mean = c(sumPhiCxy %*% solve(sumPhiCyy)),
          sigma = kronecker(solve(sumPhiCyy), diag(p)), log = T
        )
      } else {
        lambdaEval[k] <- 0
      }
    }
    ## end model 3
  } else if (constraint[1] == T & constraint[2] == F & constraint[3] == F) {
    ## model 4
    sumVar <- 0
    B <- 0
    for (k in 1:m) {
      sumVar <- sumVar + kronecker(
        Cyyk[[k]] + alpha2 / m * diag(qVec[k]),
        solve(psy[[k]])
      )
      B <- B + solve(psy[[k]]) %*% Cxmyk[[k]]
    }
    lambdaVar <- solve(sumVar)
    lambdaMean <- t(c(B)) %*% lambdaVar
    for (k in 1:m) {
      if (k == 1) {
        lambdaEval[k] <- mvtnorm::dmvnorm(
          x = c(lambda[[k]]), mean = lambdaMean,
          sigma = lambdaVar, log = T
        )
      } else {
        lambdaEval[k] <- 0
      }
    }
    ## end model 4
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == T) {
    ## model 5
    for (k in 1:m) {
      lambdaEval[k] <- mvtnorm::dmvnorm(
        x = c(lambda[[k]]), mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]]), log = T
      )
    }

    ## end model 5
  } else if (constraint[1] == F & constraint[2] == T & constraint[3] == F) {
    ## model 6
    for (k in 1:m) {
      lambdaEval[k] <- mvtnorm::dmvnorm(
        x = c(lambda[[k]]), mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]]), log = T
      )
    }

    ## end model 6
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == T) {
    ## model 7
    for (k in 1:m) {
      lambdaEval[k] <- mvtnorm::dmvnorm(
        x = c(lambda[[k]]), mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]]), log = T
      )
    }
    ## end model 7
  } else if (constraint[1] == F & constraint[2] == F & constraint[3] == F) {
    ## model 8
    for (k in 1:m) {
      lambdaEval[k] <- mvtnorm::dmvnorm(c(lambda[[k]]),
        mean = c(Cxmyk[[k]] %*% solve(Cyyk[[k]] + alpha2 * diag(qVec[k]))),
        sigma = kronecker(solve(sumCyyk), psy[[k]]), log = T
      )
    }
    ## end model 8
  }
  return(sum(lambdaEval))
}

#' (internal)
#' @noRd
#' @noRd
update_post_z_r <- function(X, m, n, thetaYList) {
  tao <- thetaYList@tao
  psy <- thetaYList@psy
  M <- thetaYList@M
  lambda <- thetaYList@lambda


  pMat <- matrix(NA, m, n)
  ## evaluate density
  dMat <- matrix(NA, m, n)

  for (k in 1:m) {
    for (i in 1:n) {
      dMat[k, i] <- mvtnorm::dmvnorm(X[, i],
        mean = M[[k]], sigma = psy[[k]] + lambda[[k]] %*% t(lambda[[k]]),
        log = T
      )
    }
  }

  for (k in 1:m) {
    dMat[k, ] <- dMat[k, ] + log(tao[k])
  }

  for (i in 1:n) {
    for (k in 1:m) {
      pMat[k, i] <- calculate_ratio_cpp(dMat[k, i], dMat[, i])
    }
  }

  ZOneDim <- c()
  for (i in 1:n) {
    tempProb <- as.numeric(pMat[, i])
    ZOneDim[i] <- sample(x = 1:m, size = 1, prob = tempProb)
  }
  ZOneDim
}

test_that("package citation includes the methodology paper DOI", {
  cites <- suppressWarnings(citation("bpgmm"))
  dois <- vapply(cites, function(x) {
    doi <- x$doi
    if (is.null(doi)) {
      NA_character_
    } else {
      doi
    }
  }, character(1))

  expect_true("10.1007/s00357-021-09391-8" %in% dois)
})

test_that("documented public API remains exported", {
  exports <- getNamespaceExports("bpgmm")

  expect_true(all(c(
    "pgmm_rjmcmc",
    "summarize_pgmm_rjmcmc",
    "constraint_to_model",
    "model_to_constraint"
  ) %in% exports))

  expect_false(any(c(
    "pgmmRJMCMC",
    "summarizePgmmRJMCMC",
    "summerizePgmmRJMCMC"
  ) %in% exports))
})

---
title: "Worked examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Worked examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This example gives a complete, runnable `bpgmm` analysis. It is small enough to
build quickly on CRAN and pkgdown, but it uses the same functions as a larger
analysis. The focus is the structure of one fitted object; model-selection
interpretation is handled separately.

## Simulate a small clustering problem

`pgmm_rjmcmc()` expects a numeric matrix with variables in rows and observations
in columns. The code below creates two well-separated clusters in two
dimensions.

```{r}
library(bpgmm)

set.seed(2026)

X <- cbind(
  matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
  matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
)
known_labels <- rep(1:2, each = 4)

dim(X)
known_labels
```

```{r, fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of simulated observations colored by known cluster."}
cluster_cols <- c("#0072B2", "#D55E00", "#009E73")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Simulated data",
  asp = 1
)
legend(
  "topleft",
  legend = paste("Known", sort(unique(known_labels))),
  col = cluster_cols[sort(unique(known_labels))],
  pch = 19,
  bty = "n"
)
```

## Fit a fixed-model chain

This fit keeps \(m\) and the covariance model fixed, so the output structure is
explicit. The fitted likelihood contribution is

\[
p(x_i \mid z_i = k, \Theta) =
N_p(x_i \mid \mu_k, \Lambda_k\Lambda_k^\top + \Psi_k).
\]

For model selection across \(m\) and covariance labels, use the separate
model-selection vignette.

```{r}
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 1,
    niter = 3,
    constraint = "UUU",
    m_step = 0,
    v_step = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)
```

The returned object stores posterior samples for allocations, covariance
constraints, component parameters, and hyperparameters.

```{r}
names(fit)
length(fit$allocation_samples)
```

## Summarize posterior samples

`summarize_pgmm_rjmcmc()` returns the modal allocation, posterior counts for the
number of clusters, posterior counts for covariance models, and optionally an
adjusted Rand index.

```{r}
fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)

fit_summary$allocation
fit_summary$n_clusters
fit_summary$n_constraints
fit_summary$ari
```

The modal allocation is computed coordinate-wise from the saved posterior
allocations:

\[
\hat{z}_i = \operatorname{mode}\{z_i^{(1)},\ldots,z_i^{(S)}\},
\]

where \(S\) is the number of saved posterior samples.

```{r, fig.width = 7, fig.height = 4, fig.alt = "Two-panel plot comparing known labels and posterior allocation."}
old_par <- par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Known labels",
  asp = 1
)
plot(
  X[1, ], X[2, ],
  col = cluster_cols[fit_summary$allocation],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Posterior allocation",
  asp = 1
)
par(old_par)
```

## Inspect component parameters

The sampler stores draws of \(\tau_k\), \(\mu_k\), \(\Lambda_k\), and
\(\Psi_k\). For this fixed-model example, the final saved draw is enough to
show the object shape.

```{r}
last <- length(fit$tau_samples)

fit$tau_samples[[last]]
fit$mean_samples[[last]]
lapply(fit$lambda_samples[[last]], dim)
lapply(fit$psi_samples[[last]], dim)
```

## Scale the example to real data

For an applied analysis, keep the same sequence of steps but use larger sampler
settings. The exact values depend on data size and convergence diagnostics.

```{r, eval = FALSE}
fit <- pgmm_rjmcmc(
  X = your_data_matrix,
  m_init = 2,
  m_range = c(1, 10),
  q_new = 4,
  burn = 5000,
  niter = 15000,
  constraint = model_to_constraint("UUU"),
  m_step = 1,
  v_step = 1,
  split_combine = 1,
  verbose = FALSE
)

summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)
```

## What to report

For a concise analysis report, include:

- the data orientation and preprocessing choices;
- `m_range`, `q_new`, starting covariance model, and whether `m_step`,
  `v_step`, and `split_combine` were enabled;
- the number of burn-in and posterior iterations;
- the posterior modal allocation or a co-clustering matrix;
- ARI only when a reference partition is scientifically meaningful.

The diagnostics vignette gives examples of trace plots and co-clustering
summaries for multiple chains.

---
title: "Preparing data and choosing sampler settings"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Preparing data and choosing sampler settings}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Before calling `pgmm_rjmcmc()`, the analyst must specify the data orientation,
scaling, missing-value handling, latent-factor dimension, cluster range, and
starting covariance model.

## Matrix orientation

`bpgmm` expects a numeric matrix with variables in rows and observations in
columns. Many R data sets use the opposite convention, so transpose after
selecting numeric variables.

The package convention is

\[
X =
\begin{bmatrix}
x_{11} & \cdots & x_{1n} \\
\vdots &        & \vdots \\
x_{p1} & \cdots & x_{pn}
\end{bmatrix},
\]

where row \(j\) is variable \(j\) and column \(i\) is observation \(x_i\).

```{r}
library(bpgmm)

iris_numeric <- as.matrix(iris[, 1:4])
iris_labels <- as.integer(iris$Species)

dim(iris_numeric)

X <- t(iris_numeric)
dim(X)
```

Rows now correspond to variables and columns correspond to observations.

```{r}
rownames(X)
X[, 1:3]
```

## Check finite numeric inputs

The sampler requires finite numeric values. Handle missing values before
fitting. Common choices include complete-case filtering, domain-specific
imputation, or fitting the model to a subset of variables with reliable
measurements.

```{r}
all(is.finite(X))
storage.mode(X)
```

## Scale variables

Mixture models are sensitive to measurement scale. If variables are measured in
different units, standardizing each variable is usually a sensible default.
For each variable \(j\), the usual transformation is

\[
x_{ji}^{\mathrm{scaled}} =
\frac{x_{ji} - \bar{x}_{j\cdot}}{s_j},
\qquad
s_j^2 = \frac{1}{n - 1}\sum_{i=1}^n
(x_{ji} - \bar{x}_{j\cdot})^2 .
\]

```{r}
X_scaled <- t(scale(t(X)))

round(rowMeans(X_scaled), 6)
round(apply(X_scaled, 1, sd), 6)
```

The package does not scale internally because some scientific applications need
the original measurement scale. Scaling should be an explicit analysis choice.

## Choose `q_new`

`q_new` is the latent-factor dimension assigned to newly proposed clusters. It
controls the dimension of the factor-analyzer part of the covariance model.
In the paper's notation,

\[
\Lambda_k \in \mathbb{R}^{p \times q_k}, \qquad
y_{ki} \in \mathbb{R}^{q_k}.
\]

The package uses `q_new` as the \(q_k\) value for a newly created component.

Useful starting points:

- `q_new = 1` for very small examples or when covariance structure should be
  simple.
- `q_new = 2` or `3` for moderate-dimensional data.
- Larger values only when the number of variables and observations support the
  extra covariance flexibility.

The value should be smaller than the number of observed variables.

```{r}
p <- nrow(X_scaled)
q_new <- min(2, p - 1)
q_new
```

## Choose `m_range`

`m_range` is the allowed cluster-number range. A wide range gives RJMCMC more
freedom, but also increases the model space. Start with a scientifically
reasonable range, then assess sensitivity.

```{r}
m_init <- 3
m_range <- c(1, 5)
```

For data with a known reference label, such as `iris`, the reference partition
gives a simple check on the range. In unsupervised applications, use domain
knowledge and exploratory plots.

```{r, fig.width = 5.5, fig.height = 4.2, fig.alt = "Scatter plot of scaled iris variables colored by species."}
species_cols <- c("#0072B2", "#D55E00", "#009E73")
plot(
  X_scaled[1, ], X_scaled[2, ],
  col = species_cols[iris_labels],
  pch = 19,
  xlab = rownames(X_scaled)[1],
  ylab = rownames(X_scaled)[2],
  main = "Scaled iris data",
  asp = 1
)
legend(
  "topleft",
  legend = levels(iris$Species),
  col = species_cols,
  pch = 19,
  bty = "n"
)
```

## Choose the starting covariance model

The three-letter model labels describe whether loading matrices and noise
covariances are shared across clusters. `UUU` is flexible; `CCC` is more
constrained. A flexible starting model is often reasonable when using
`v_step = 1`, because the sampler can move across covariance structures.

The fitted covariance is always

\[
\Sigma_k = \Lambda_k\Lambda_k^\top + \Psi_k,
\]

but the label controls whether \(\Lambda_k\) and \(\Psi_k\) are shared and
whether \(\Psi_k\) is isotropic or diagonal.

```{r}
model_to_constraint("UUU")
constraint_to_model(c(1, 1, 1))
```

## A prepared call

The call below is not evaluated in the vignette because applied analyses should
use longer chains and repeated runs. It records how the prepared objects enter
the package interface.

```{r, eval = FALSE}
fit <- pgmm_rjmcmc(
  X = X_scaled,
  m_init = m_init,
  m_range = m_range,
  q_new = q_new,
  burn = 1000,
  niter = 5000,
  constraint = "UUU",
  m_step = 1,
  v_step = 1,
  split_combine = 1,
  verbose = FALSE
)

summarize_pgmm_rjmcmc(fit, true_cluster = iris_labels)
```

## Practical checklist

Before fitting:

- verify that `X` is numeric with variables in rows;
- remove or impute missing and non-finite values;
- decide whether variables should be scaled;
- choose a scientifically plausible `m_range`;
- choose `q_new` smaller than the number of variables;
- use multiple chains for applied analyses.

---
title: "Exploratory variable prioritization after bpgmm clustering"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Exploratory variable prioritization after bpgmm clustering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

`bpgmm` performs Bayesian model selection for the number of clusters and PGMM
covariance structure. It does not currently implement a formal spike-and-slab
or inclusion-indicator variable-selection prior. Still, users often want to
understand which observed variables help explain the fitted clustering.

The calculations below:

- fit `bpgmm`;
- summarize the posterior modal allocation;
- rank variables by between-cluster separation;
- compare that ranking with posterior loading magnitudes.

These summaries are diagnostic and exploratory. Treat them as variable
prioritization, not as formal posterior variable inclusion probabilities.

## Simulate data with informative and weak variables

The simulation differs from the model-selection example. The aim is not to
recover \(m\), but to examine how posterior output can be post-processed to
rank variables. The first three variables have different cluster means,
variables four and five have mostly covariance differences, and variable six is
weak noise.

```{r}
library(bpgmm)

simulate_screening_data <- function(n_per_cluster = 18, p = 6) {
  means <- rbind(
    c(-2.5, -1.5,  0.0, 0, 0, 0),
    c( 2.0, -0.5,  1.8, 0, 0, 0),
    c( 0.0,  2.2, -1.8, 0, 0, 0)
  )

  covariances <- list(
    diag(c(0.35, 0.35, 0.35, 1.40, 0.35, 1.20)),
    diag(c(0.35, 0.35, 0.35, 0.35, 1.40, 1.20)),
    matrix(c(
      0.35, 0,    0,    0,    0,    0,
      0,    0.35, 0,    0,    0,    0,
      0,    0,    0.35, 0,    0,    0,
      0,    0,    0,    1.00, 0.45, 0,
      0,    0,    0,    0.45, 1.00, 0,
      0,    0,    0,    0,    0,    1.20
    ), nrow = p)
  )

  n <- n_per_cluster * 3
  X <- matrix(NA_real_, nrow = p, ncol = n)
  true_cluster <- rep(seq_len(3), each = n_per_cluster)

  column <- 1
  for (k in seq_len(3)) {
    for (i in seq_len(n_per_cluster)) {
      X[, column] <- MASS::mvrnorm(1, mu = means[k, ], Sigma = covariances[[k]])
      column <- column + 1
    }
  }

  rownames(X) <- paste0("variable_", seq_len(p))
  list(X = X, true_cluster = true_cluster)
}

set.seed(2028)
sim <- simulate_screening_data()
X <- sim$X
true_cluster <- sim$true_cluster
```

The fitted PGMM still uses

\[
\Sigma_k = \Lambda_k\Lambda_k^\top + \Psi_k,
\]

but this simulation deliberately separates mean-driven variables from
covariance-driven variables so the two rankings below answer different
questions.

## Fit a short clustering chain

```{r}
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 3,
    m_range = c(1, 5),
    q_new = 2,
    burn = 2,
    niter = 6,
    constraint = "UUU",
    m_step = 1,
    v_step = 1,
    verbose = FALSE
  )
})
tail(fit_log, 1)

fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = true_cluster)
fit_summary$ari
```

## Rank variables by posterior allocation separation

One simple score is the fraction of total variation explained by the posterior
modal clusters:

\[
R_j^2 =
\frac{\sum_k n_k(\bar{x}_{jk} - \bar{x}_{j\cdot})^2}
     {\sum_i (x_{ji} - \bar{x}_{j\cdot})^2}.
\]

This quantity is not a posterior inclusion probability; it summarizes
separation under the fitted partition.

```{r}
cluster_separation <- function(X, allocation) {
  vapply(seq_len(nrow(X)), function(j) {
    x <- X[j, ]
    overall <- mean(x)
    total <- sum((x - overall)^2)
    between <- sum(vapply(split(x, allocation), function(group) {
      length(group) * (mean(group) - overall)^2
    }, numeric(1)))
    if (total == 0) 0 else between / total
  }, numeric(1))
}

separation <- cluster_separation(X, fit_summary$allocation)
separation_table <- data.frame(
  variable = rownames(X),
  separation = round(separation, 3)
)
separation_table[order(separation_table$separation, decreasing = TRUE), ]
```

```{r, fig.width = 6.5, fig.height = 4, fig.alt = "Bar plot of exploratory variable separation scores."}
ordered <- order(separation, decreasing = TRUE)
barplot(
  separation[ordered],
  names.arg = rownames(X)[ordered],
  las = 2,
  col = "#56B4E9",
  border = NA,
  ylab = "Between-cluster variation / total variation",
  main = "Exploratory variable separation"
)
```

## Compare with loading magnitudes

The loading matrices describe how observed variables relate to latent factors
inside each component. A simple posterior diagnostic is the average absolute
loading magnitude across saved samples and active components.

\[
L_j =
\frac{1}{|\mathcal{S}|}
\sum_{s \in \mathcal{S}}
\frac{1}{|A_s|}
\sum_{k \in A_s}
\frac{1}{q_k}\sum_{\ell = 1}^{q_k}
|\lambda_{jk\ell}^{(s)}|,
\]

where \(A_s\) is the set of active clusters in posterior sample \(s\). Large
\(L_j\) means variable \(j\) contributes strongly to the latent-factor
covariance structure; it is not a posterior inclusion probability.

```{r}
loading_importance <- function(fit) {
  totals <- NULL
  count <- 0

  for (sample_id in seq_along(fit$lambda_samples)) {
    lambdas <- fit$lambda_samples[[sample_id]]
    active <- which(fit$active_cluster_samples[[sample_id]] == 1)

    for (k in active) {
      lambda <- lambdas[[k]]
      if (!is.matrix(lambda)) {
        next
      }
      score <- rowMeans(abs(lambda))
      if (is.null(totals)) {
        totals <- numeric(length(score))
      }
      totals <- totals + score
      count <- count + 1
    }
  }

  if (count == 0) {
    return(rep(NA_real_, nrow(X)))
  }
  totals / count
}

loading_score <- loading_importance(fit)
loading_table <- data.frame(
  variable = rownames(X),
  mean_abs_loading = round(loading_score, 3)
)
loading_table[order(loading_table$mean_abs_loading, decreasing = TRUE), ]
```

```{r, fig.width = 6.5, fig.height = 4, fig.alt = "Bar plot of average posterior absolute loading magnitudes by variable."}
ordered_loading <- order(loading_score, decreasing = TRUE)
barplot(
  loading_score[ordered_loading],
  names.arg = rownames(X)[ordered_loading],
  las = 2,
  col = "#E69F00",
  border = NA,
  ylab = "Mean absolute loading",
  main = "Posterior loading magnitude"
)
```

## Interpretation

Use both summaries together:

- high separation means a variable distinguishes the posterior clusters;
- high loading magnitude means a variable contributes to latent-factor
  covariance structure inside components;
- disagreement between the two summaries indicates that the mean structure and
  covariance structure carry different information.

For formal variable selection, the model would need an explicit variable
inclusion prior and posterior inclusion summaries. The current package is best
described as supporting model-based clustering, cluster-number selection, and
PGMM covariance-structure selection, with exploratory variable prioritization
available from posterior outputs.

## Suggested reporting language

When using these summaries in a manuscript or analysis report, use wording such
as:

> We ranked variables by their separation across posterior modal clusters and
> by posterior loading magnitudes. These rankings are exploratory diagnostics
> derived from the fitted clustering model, not posterior inclusion
> probabilities.

Avoid wording such as "selected variables" unless a formal variable-selection
model has been fitted. This distinction keeps the statistical target clear and
avoids overstating the model.

---
title: "Posterior diagnostics and multiple chains"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Posterior diagnostics and multiple chains}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

After fitting `bpgmm`, posterior samples can be inspected through chain-level
summaries, trace plots, and co-clustering probabilities. These diagnostics
complement the model-selection example.

The chains below are deliberately short so that the vignette builds quickly.
Applied analyses should use longer chains and examine stability across
independent runs.

## Simulate a diagnostic example

```{r}
library(bpgmm)

set.seed(2029)
X <- cbind(
  matrix(rnorm(10, mean = -2.0, sd = 0.25), nrow = 2),
  matrix(rnorm(10, mean =  0.0, sd = 0.25), nrow = 2),
  matrix(rnorm(10, mean =  2.0, sd = 0.25), nrow = 2)
)
known_labels <- rep(1:3, each = 5)
```

```{r, fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of a compact three-cluster diagnostic data set."}
cluster_cols <- c("#0072B2", "#D55E00", "#009E73", "#CC79A7")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Diagnostic example",
  asp = 1
)
```

## Run independent chains

`pgmm_rjmcmc_chains()` runs independent chains. The vignette uses `cores = 1`
for CRAN portability, but users can increase `cores` locally.

```{r}
fits <- pgmm_rjmcmc_chains(
  X = X,
  m_init = 3,
  m_range = c(1, 4),
  q_new = 1,
  burn = 1,
  niter = 4,
  constraint = "UUU",
  m_step = 1,
  v_step = 1,
  chains = 2,
  cores = 1,
  seed = 2029,
  verbose = FALSE
)

length(fits)
attr(fits, "chain_seeds")
```

## Summarize each chain

For chain \(c\), simple empirical summaries are

\[
\widehat{p}_c(m = r \mid X)
  = \frac{1}{S_c}\sum_{s=1}^{S_c} I(m_c^{(s)} = r),
\qquad
\widehat{p}_c(v = a \mid X)
  = \frac{1}{S_c}\sum_{s=1}^{S_c} I(v_c^{(s)} = a).
\]

Large disagreements across chains are a warning sign, especially when the
chains use different random seeds.

```{r}
chain_summaries <- lapply(fits, summarize_pgmm_rjmcmc, true_cluster = known_labels)

data.frame(
  chain = names(chain_summaries),
  ari = vapply(chain_summaries, function(x) x$ari, numeric(1)),
  modal_clusters = vapply(chain_summaries, function(x) {
    as.integer(names(which.max(x$n_clusters)))
  }, integer(1))
)
```

## Trace cluster counts and covariance models

The posterior samples store the active cluster indicators and covariance
constraint at each saved iteration. These traces are first diagnostics.

```{r}
cluster_count_trace <- function(fit) {
  vapply(fit$active_cluster_samples, sum, numeric(1))
}

constraint_trace <- function(fit) {
  vapply(fit$constraint_samples, constraint_to_model, character(1))
}

cluster_traces <- lapply(fits, cluster_count_trace)
constraint_traces <- lapply(fits, constraint_trace)

cluster_traces
constraint_traces
```

```{r, fig.width = 7, fig.height = 4, fig.alt = "Trace plot of sampled cluster counts across two short chains."}
old_par <- par(mar = c(4, 4, 3, 1))
plot(
  cluster_traces[[1]],
  type = "b",
  pch = 19,
  ylim = range(unlist(cluster_traces)),
  col = "#0072B2",
  xlab = "Saved iteration",
  ylab = "Active clusters",
  main = "Cluster-count trace"
)
lines(cluster_traces[[2]], type = "b", pch = 19, col = "#D55E00")
legend("topright", legend = names(fits), col = c("#0072B2", "#D55E00"), lty = 1, pch = 19, bty = "n")
par(old_par)
```

## Posterior co-clustering matrix

A co-clustering matrix estimates how often two observations are assigned to the
same cluster across posterior samples. It reports pairwise uncertainty rather
than only a single modal allocation.

\[
C_{ij} =
\Pr(z_i = z_j \mid X)
\approx
\frac{1}{S}\sum_{s=1}^{S}
I\{z_i^{(s)} = z_j^{(s)}\}.
\]

```{r}
co_clustering_matrix <- function(fit) {
  n <- length(fit$allocation_samples[[1]])
  out <- matrix(0, n, n)

  for (allocation in fit$allocation_samples) {
    out <- out + outer(allocation, allocation, "==")
  }

  out / length(fit$allocation_samples)
}

co_mat <- co_clustering_matrix(fits[[1]])
round(co_mat[1:6, 1:6], 2)
```

```{r, fig.width = 5.5, fig.height = 5, fig.alt = "Heatmap of posterior co-clustering probabilities."}
image(
  seq_len(nrow(co_mat)),
  seq_len(ncol(co_mat)),
  co_mat[nrow(co_mat):1, ],
  col = hcl.colors(20, "YlGnBu", rev = TRUE),
  xlab = "Observation",
  ylab = "Observation",
  main = "Posterior co-clustering"
)
```

## What to look for

Warning signs include:

- cluster-count traces that stay at the boundary of `m_range`;
- very different posterior summaries across independent chains;
- covariance-model traces that never move when `v_step = 1`;
- co-clustering matrices with no clear block structure;
- sensitivity to scaling, `q_new`, or `m_range`.

These diagnostics do not replace a long MCMC analysis, but they give a concise
description of the posterior output.

---
title: "Model and sampler details"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model and sampler details}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This section connects the notation in Lu, Li, and Love (2021) to the `bpgmm`
interface. It describes the fitted model, the covariance labels, and the
RJMCMC switches used by the sampler. Full fitted examples are given in the
getting-started and worked-example vignettes.

## Observation model

Let \(X = (x_1, \ldots, x_n)\) be a matrix of continuous observations with
\(p\) variables and \(n\) observations. `bpgmm` expects this same orientation:
variables in rows and observations in columns.

For cluster \(k\), the mixture of factor analyzers model writes

\[
x_i = \mu_k + \Lambda_k y_{ki} + \epsilon_{ki},
\]

where

\[
y_{ki} \sim N(0, I_{q_k}), \qquad
\epsilon_{ki} \sim N(0, \Psi_k).
\]

After integrating out the latent factor \(y_{ki}\), the cluster-specific
covariance is

\[
\Sigma_k = \Lambda_k \Lambda_k^\top + \Psi_k.
\]

The plot below shows the geometric role of this covariance. The loading matrix
\(\Lambda_k\) controls the dominant low-dimensional direction, while
\(\Psi_k\) adds cluster-specific noise around that subspace.

```{r, fig.width = 5.5, fig.height = 4.5, fig.alt = "Two covariance ellipses illustrating cluster-specific covariance matrices."}
ellipse_points <- function(center, sigma, level = 0.90, n = 120) {
  eig <- eigen(sigma, symmetric = TRUE)
  angles <- seq(0, 2 * pi, length.out = n)
  circle <- rbind(cos(angles), sin(angles))
  radius <- sqrt(qchisq(level, df = 2))
  points <- t(center + radius * eig$vectors %*% diag(sqrt(eig$values), 2) %*% circle)
  colnames(points) <- c("x", "y")
  points
}

lambda_1 <- matrix(c(1.1, 0.4), ncol = 1)
lambda_2 <- matrix(c(0.2, 1.0), ncol = 1)
psi_1 <- diag(c(0.20, 0.08))
psi_2 <- diag(c(0.10, 0.25))
sigma_1 <- lambda_1 %*% t(lambda_1) + psi_1
sigma_2 <- lambda_2 %*% t(lambda_2) + psi_2

ell_1 <- ellipse_points(c(-1.2, -0.5), sigma_1)
ell_2 <- ellipse_points(c(1.1, 0.6), sigma_2)

plot(
  ell_1,
  type = "l",
  lwd = 2,
  col = "#0072B2",
  xlim = c(-3.5, 3.5),
  ylim = c(-2.5, 3),
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Mixture-of-factor-analyzers covariance"
)
lines(ell_2, lwd = 2, col = "#D55E00")
points(rbind(c(-1.2, -0.5), c(1.1, 0.6)), pch = 19, col = c("#0072B2", "#D55E00"))
arrows(-1.2, -0.5, -1.2 + lambda_1[1], -0.5 + lambda_1[2], col = "#0072B2", lwd = 2, length = 0.08)
arrows(1.1, 0.6, 1.1 + lambda_2[1], 0.6 + lambda_2[2], col = "#D55E00", lwd = 2, length = 0.08)
legend(
  "topleft",
  legend = c("Cluster 1 covariance", "Cluster 2 covariance", "Loading direction"),
  col = c("#0072B2", "#D55E00", "gray30"),
  lwd = c(2, 2, 2),
  bty = "n"
)
```

The mixture density is

\[
f(x_i) = \sum_{k = 1}^m \tau_k
  N(x_i \mid \mu_k, \Lambda_k \Lambda_k^\top + \Psi_k),
\]

where \(\tau_k\) is the mixture weight and \(m\) is the number of clusters.
Equivalently, with allocation indicator \(z_i \in \{1,\ldots,m\}\),

\[
x_i \mid z_i = k, \Theta
  \sim N_p(\mu_k, \Sigma_k),
\qquad
\Pr(z_i = k \mid \tau) = \tau_k .
\]

In `pgmm_rjmcmc()`:

- `X` is the \(p \times n\) data matrix.
- `m_init` is the starting value of \(m\).
- `m_range` is the allowed range of \(m\).
- `q_new` is the factor dimension assigned to a newly proposed cluster.
- `constraint` is the starting covariance model.

## PGMM covariance constraints

The paper uses three letters to describe the covariance structure. Each letter
is either `C` for constrained or `U` for unconstrained.

| Letter | Meaning when `C` | Meaning when `U` |
|---|---|---|
| 1 | all clusters share one loading matrix \(\Lambda\) | each cluster has \(\Lambda_k\) |
| 2 | all clusters share one noise covariance \(\Psi\) | each cluster has \(\Psi_k\) |
| 3 | noise covariance is isotropic, \(\Psi_k = \psi_k I_p\) | noise covariance is diagonal |

The eight PGMM models are:

```{r}
library(bpgmm)

models <- c("CCC", "CCU", "CUC", "CUU", "UCC", "UCU", "UUC", "UUU")
data.frame(
  model = models,
  constraint = vapply(
    models,
    function(x) paste(model_to_constraint(x), collapse = ","),
    character(1)
  )
)
```

The package uses the numeric encoding internally: `1` means constrained and
`0` means unconstrained.

```{r}
model_to_constraint("UUU")
constraint_to_model(c(1, 1, 0))
```

## Priors and posterior updates

The supplement gives the natural conjugate priors used by the MCMC updates.
The main priors are:

\[
\tau \sim \mathrm{Dirichlet}(\gamma, \ldots, \gamma),
\]

\[
y_{ki} \sim N(0, I_{q_k}),
\]

\[
\mu_k \sim N(\bar{x}, \alpha_1^{-1} \Psi_k),
\]

\[
\lambda_{kj} \sim N(0, \alpha_2^{-1} \Psi_k),
\]

and

\[
\psi_{kj} \sim \mathrm{IG}(\delta, \beta).
\]

The complete parameter state can be read as

\[
\Theta =
\{\tau_k, \mu_k, \Lambda_k, \Psi_k, y_{ki}, z_i:
  k = 1,\ldots,m;\ i = 1,\ldots,n\},
\]

with hyperparameters

\[
H = (\alpha_1, \alpha_2, \beta, \delta, \gamma).
\]

The package samples the hyperparameters \(\alpha_1\), \(\alpha_2\), and
\(\beta\), with gamma hyperpriors controlled by:

- `d_vec`: shape parameters.
- `s_vec`: rate parameters.
- `delta`: inverse-gamma shape for the noise covariance.
- `ggamma`: Dirichlet concentration for mixture weights.

The allocation update uses the posterior probability

\[
p_{ki} =
\frac{
  \tau_k N(x_i \mid \mu_k, \Psi_k + \Lambda_k\Lambda_k^\top)
}{
  \sum_{\ell = 1}^{m}
  \tau_\ell N(x_i \mid \mu_\ell, \Psi_\ell + \Lambda_\ell\Lambda_\ell^\top)
}.
\]

Then

\[
(z_{i1}, \ldots, z_{im}) \mid X, \Theta, H
  \sim \mathrm{Multinomial}(1, p_{1i}, \ldots, p_{mi}).
\]

The allocation update uses this probability, and the joint posterior includes
the product of allocated mixture weights:

\[
p(Z \mid \tau) = \prod_{i = 1}^{n} \tau_{z_i}.
\]

## RJMCMC moves

RJMCMC is used because the number of parameters changes when the number of
clusters or the covariance constraint changes.

The cluster-number moves are:

| Move | Purpose |
|---|---|
| stay | update parameters without changing \(m\) |
| birth | add a new empty cluster |
| death | remove an empty cluster |
| split | split one occupied cluster into two clusters |
| combine | merge two occupied clusters |

The covariance-structure moves toggle one constraint at a time:

- toggle whether \(\Lambda_k\) is shared across clusters;
- toggle whether \(\Psi_k\) is shared across clusters;
- toggle whether \(\Psi_k\) is isotropic or diagonal.

In `pgmm_rjmcmc()`:

- `m_step = 1` enables birth/death moves for \(m\).
- `split_combine = 1` adds split/combine moves when `m_step = 1`.
- `v_step = 1` enables covariance-constraint moves.

For a proposed move from model \(M\) to \(M'\), the RJMCMC acceptance
probability has the standard form

\[
a =
\min\left\{
1,
\frac{
  p(X, Z, \Theta', H' \mid M')\,p(M')\,q(M \mid M')
}{
  p(X, Z, \Theta, H \mid M)\,p(M)\,q(M' \mid M)
}
\left|J\right|
\right\},
\]

where \(q(\cdot)\) is the proposal density and \(|J|\) is the Jacobian term
for dimension-changing moves such as split/combine. Birth/death and
split/combine change \(m\); covariance moves change the constraint label
\(v \in \{\mathrm{CCC}, \ldots, \mathrm{UUU}\}\).

## Interface mapping

The formula terms map to package output fields as follows:

| Paper notation | Package argument or output |
|---|---|
| \(X = (x_1,\ldots,x_n)\) | `X`, a \(p \times n\) numeric matrix |
| \(m\) | `m_init`, `m_range`, `active_cluster_samples` |
| \(q_k\) | `q_new` for newly proposed clusters |
| \(z_i\) | `allocation_samples`, `summary$allocation` |
| \(\tau_k\) | `tau_samples` |
| \(\mu_k\) | `mean_samples` |
| \(\Lambda_k\) | `lambda_samples` |
| \(\Psi_k\) | `psi_samples` |
| covariance model \(v\) | `constraint`, `constraint_samples` |

An applied model-selection call usually has the following structure:

```{r, eval = FALSE}
fit <- pgmm_rjmcmc(
  X = your_data_matrix,
  m_init = 5,
  m_range = c(1, 10),
  q_new = 4,
  burn = 5000,
  niter = 15000,
  constraint = model_to_constraint("UUU"),
  m_step = 1,
  v_step = 1,
  split_combine = 1,
  verbose = FALSE
)
```

## Citation

If you use these methods, cite:

Lu, X., Li, Y., & Love, T. (2021). On Bayesian Analysis of Parsimonious
Gaussian Mixture Models. *Journal of Classification*, 38, 576-593.
https://doi.org/10.1007/s00357-021-09391-8

In R:

```{r}
citation("bpgmm")
```

---
title: "Getting started with bpgmm"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with bpgmm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

`bpgmm` fits Bayesian parsimonious Gaussian mixture models for model-based
clustering. It targets three posterior inference goals described by Lu, Li, and
Love (2021): the partition of observations, the number of clusters, and the
cluster covariance structure.

The vignettes have separate roles:

| Article | Use it for |
|---|---|
| `data-preparation` | matrix orientation, scaling, and choosing `m_range` and `q_new` |
| `model-and-sampler` | formulas from the paper and their package arguments |
| `examples` | inspecting one fitted object |
| `model-selection` | RJMCMC summaries for \(m\) and covariance model \(v\) |
| `variable-prioritization` | exploratory variable rankings from posterior output |
| `posterior-diagnostics` | independent chains, traces, and co-clustering |

## Fit a model

Input data should be a numeric matrix with variables in rows and observations
in columns. The small example below has two variables and eight observations.
The example is short so the vignette runs quickly; applied analyses should use
larger `burn` and `niter` values.

```{r}
library(bpgmm)

set.seed(2026)

X <- cbind(
  matrix(rnorm(8, mean = -2, sd = 0.2), nrow = 2),
  matrix(rnorm(8, mean = 2, sd = 0.2), nrow = 2)
)
known_labels <- rep(1:2, each = 4)
```

The scatter plot gives a direct check of the simulated partition. Each point is
one observation, and the colors show the reference labels used later for the
ARI calculation.

```{r, fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot of the small two-cluster example."}
plot(
  X[1, ], X[2, ],
  col = c("#0072B2", "#D55E00")[known_labels],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Small two-cluster example",
  asp = 1
)
legend(
  "topleft",
  legend = paste("Reference", sort(unique(known_labels))),
  col = c("#0072B2", "#D55E00"),
  pch = 19,
  bty = "n"
)
```

```{r}
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 2,
    m_range = c(1, 3),
    q_new = 1,
    burn = 1,
    niter = 3,
    constraint = model_to_constraint("UUU"),
    m_step = 0,
    v_step = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)
```

The call fixes \(m = 2\) and the covariance model so the example is fast.
The important arguments are:

- `m_init` is the initial number of clusters.
- `m_range` is the allowed range for the number of clusters.
- `q_new` is the number of latent factors for a new cluster.
- `constraint` selects the starting covariance model.
- `m_step = 1` allows RJMCMC updates for the number of clusters.
- `v_step = 1` allows RJMCMC updates across covariance-constraint models.

## Summarize posterior samples

```{r}
summary <- summarize_pgmm_rjmcmc(fit, true_cluster = known_labels)

summary$allocation
summary$n_clusters
summary$n_constraints
summary$ari
```

If a reference partition is available, pass it as `true_cluster` to calculate
the adjusted Rand index.

The summary allocation can be plotted back on the original coordinates. This
plot is a first check before longer chains and convergence diagnostics.

```{r, fig.width = 5.5, fig.height = 4, fig.alt = "Scatter plot colored by posterior modal allocation."}
plot(
  X[1, ], X[2, ],
  col = c("#009E73", "#CC79A7", "#E69F00")[summary$allocation],
  pch = 19,
  xlab = "Variable 1",
  ylab = "Variable 2",
  main = "Posterior modal allocation",
  asp = 1
)
text(X[1, ], X[2, ], labels = seq_along(summary$allocation), pos = 3, cex = 0.75)
legend(
  "topleft",
  legend = paste("Cluster", sort(unique(summary$allocation))),
  col = c("#009E73", "#CC79A7", "#E69F00")[sort(unique(summary$allocation))],
  pch = 19,
  bty = "n"
)
```

## Common early mistakes

Common early problems are:

- `X` is accidentally supplied as observations by variables instead of
  variables by observations;
- variables with very different measurement scales are not standardized;
- `m_init` is outside `m_range`;
- `q_new` is too large for a small number of variables;
- `m_step` and `v_step` are left at zero when model selection is desired.

## Naming convention

Starting with version 1.2.0, the public API uses snake_case names throughout.
Use `pgmm_rjmcmc()`, `summarize_pgmm_rjmcmc()`, and snake_case argument names
such as `m_init`, `m_range`, and `q_new`.

## Citation

If you use `bpgmm` in published work, please cite the package and the
methodology paper:

```{r}
citation("bpgmm")
```

Lu, X., Li, Y., & Love, T. (2021). On Bayesian Analysis of Parsimonious
Gaussian Mixture Models. *Journal of Classification*, 38, 576-593.
https://doi.org/10.1007/s00357-021-09391-8

---
title: "Model selection on a larger simulated MFA data set"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model selection on a larger simulated MFA data set}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The following simulation uses a moderately larger mixture-of-factor-analyzers
data set to illustrate model selection in `bpgmm`. The chain is short so that
the vignette builds quickly on CRAN and pkgdown. Applied analyses should use
longer burn-in, more posterior samples, multiple chains, and convergence
diagnostics.

## Simulate clustered factor-analytic data

The simulator below creates three clusters, six observed variables, and two
latent factors. The first three variables carry most of the cluster separation,
while the last three variables are intentionally less informative.

The generating model matches the paper's mixture-of-factor-analyzers form:

\[
x_i = \mu_{z_i} + \Lambda_{z_i}y_i + \epsilon_i,
\qquad
y_i \sim N_2(0, I_2),
\qquad
\epsilon_i \sim N_6(0, \Psi_{z_i}).
\]

```{r}
library(bpgmm)

simulate_mfa_data <- function(n_per_cluster = 20, p = 6, q = 2) {
  means <- rbind(
    c(-3.0, -2.0,  0.0, 0, 0, 0),
    c( 2.5, -1.0,  2.0, 0, 0, 0),
    c( 0.0,  2.5, -2.0, 0, 0, 0)
  )

  lambdas <- list(
    matrix(c(1.2,  0.5, 0.2, 0, 0, 0,
             0.0,  0.2, 0.8, 0.3, 0, 0), nrow = p),
    matrix(c(0.2,  1.1, 0.6, 0, 0, 0,
             0.8,  0.1, 0.2, 0.6, 0, 0), nrow = p),
    matrix(c(0.7, -0.4, 1.0, 0, 0, 0,
            -0.2,  0.8, 0.4, 0.3, 0, 0), nrow = p)
  )

  psi <- list(
    diag(c(0.25, 0.35, 0.30, 0.80, 0.90, 1.00)),
    diag(c(0.30, 0.25, 0.40, 0.80, 0.90, 1.00)),
    diag(c(0.35, 0.30, 0.25, 0.80, 0.90, 1.00))
  )

  n <- n_per_cluster * 3
  X <- matrix(NA_real_, nrow = p, ncol = n)
  true_cluster <- rep(seq_len(3), each = n_per_cluster)

  column <- 1
  for (k in seq_len(3)) {
    for (i in seq_len(n_per_cluster)) {
      latent_score <- rnorm(q)
      noise <- MASS::mvrnorm(1, mu = rep(0, p), Sigma = psi[[k]])
      X[, column] <- means[k, ] + lambdas[[k]] %*% latent_score + noise
      column <- column + 1
    }
  }

  rownames(X) <- paste0("variable_", seq_len(p))
  list(X = X, true_cluster = true_cluster)
}

set.seed(2027)
sim <- simulate_mfa_data()
X <- sim$X
true_cluster <- sim$true_cluster

dim(X)
table(true_cluster)
```

The first two variables already show substantial separation, but the model is
fit to all six variables.

```{r, fig.width = 5.5, fig.height = 4.2, fig.alt = "Scatter plot of the first two variables colored by true cluster."}
cluster_cols <- c("#0072B2", "#D55E00", "#009E73", "#CC79A7", "#E69F00")
plot(
  X[1, ], X[2, ],
  col = cluster_cols[true_cluster],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "Simulated MFA data",
  asp = 1
)
legend(
  "topleft",
  legend = paste("True cluster", sort(unique(true_cluster))),
  col = cluster_cols[sort(unique(true_cluster))],
  pch = 19,
  bty = "n"
)
```

## Run RJMCMC model selection

Here `m_step = 1` allows the sampler to move across different numbers of
clusters, and `v_step = 1` allows moves across the PGMM covariance structures.
The example uses only a few iterations so that the vignette is fast.

The two discrete targets are

\[
p(m \mid X)
\quad \text{and} \quad
p(v \mid X),
\]

where \(v\) is one of the eight covariance labels
\(\{\mathrm{CCC}, \ldots, \mathrm{UUU}\}\). The saved samples estimate these
posterior probabilities by empirical frequencies:

\[
\widehat{p}(m = r \mid X)
  = \frac{1}{S}\sum_{s=1}^S I(m^{(s)} = r),
\qquad
\widehat{p}(v = a \mid X)
  = \frac{1}{S}\sum_{s=1}^S I(v^{(s)} = a).
\]

```{r}
fit_log <- capture.output({
  fit <- pgmm_rjmcmc(
    X = X,
    m_init = 3,
    m_range = c(1, 5),
    q_new = 2,
    burn = 2,
    niter = 6,
    constraint = "UUU",
    m_step = 1,
    v_step = 1,
    split_combine = 0,
    verbose = FALSE
  )
})
tail(fit_log, 1)
```

```{r}
fit_summary <- summarize_pgmm_rjmcmc(fit, true_cluster = true_cluster)

fit_summary$n_clusters
fit_summary$n_constraints
fit_summary$ari
```

The model-selection summaries are posterior sample counts. In an applied
analysis, these bars should be based on a much longer chain.

```{r, fig.width = 7, fig.height = 4, fig.alt = "Bar plots of posterior counts for cluster number and covariance model."}
old_par <- par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))
barplot(
  fit_summary$n_clusters,
  col = "#56B4E9",
  border = NA,
  xlab = "Number of clusters",
  ylab = "Posterior sample count",
  main = "Cluster-number selection"
)
barplot(
  fit_summary$n_constraints,
  col = "#E69F00",
  border = NA,
  xlab = "PGMM model",
  ylab = "Posterior sample count",
  main = "Covariance-model selection"
)
par(old_par)
```

## Compare true and posterior allocations

The posterior modal allocation can be plotted against the known simulation
labels.

```{r, fig.width = 7, fig.height = 4, fig.alt = "Two-panel scatter plot comparing true cluster labels and posterior modal allocation."}
old_par <- par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
plot(
  X[1, ], X[2, ],
  col = cluster_cols[true_cluster],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "True clusters",
  asp = 1
)
plot(
  X[1, ], X[2, ],
  col = cluster_cols[fit_summary$allocation],
  pch = 19,
  xlab = rownames(X)[1],
  ylab = rownames(X)[2],
  main = "Posterior modal allocation",
  asp = 1
)
par(old_par)
```

## Interpreting short versus long runs

The short chain in this vignette demonstrates the mechanics of model selection.
It should not be interpreted as a final posterior analysis. In applied work:

- increase `burn` and `niter` until summaries are stable;
- compare independent chains with `pgmm_rjmcmc_chains()`;
- check whether sampled cluster counts are stuck at either edge of `m_range`;
- rerun sensitivity checks for plausible values of `q_new`;
- inspect whether covariance-model posterior counts change when `v_step = 1`.

If the posterior mass stays at the upper end of `m_range`, expand the range or
revisit preprocessing. If it stays at the lower end, the data may support fewer
clusters than expected, or the covariance model may be too flexible for the
sample size.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize-result.R
\name{summarize_pgmm_rjmcmc}
\alias{summarize_pgmm_rjmcmc}
\title{Summarize RJMCMC Samples from a Bayesian PGMM Fit}
\usage{
summarize_pgmm_rjmcmc(fit, true_cluster = NULL)
}
\arguments{
\item{fit}{Result list from [pgmm_rjmcmc()].}

\item{true_cluster}{Optional true or reference cluster allocation.}
}
\value{
A list with `allocation`, `n_clusters`, `n_constraints`, and optionally
  `ari`.
}
\description{
Summarizes posterior samples from [pgmm_rjmcmc()] into the modal allocation,
posterior counts for the number of clusters, posterior counts for the eight
PGMM covariance-constraint models, and optionally the adjusted Rand index
against a known reference partition.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-hparam.R
\docType{class}
\name{Hparam-class}
\alias{Hparam-class}
\title{Hyperparameter set for the Bayesian PGMM sampler.}
\description{
Hyperparameter set for the Bayesian PGMM sampler.
}
\section{Slots}{

\describe{
\item{\code{alpha1}}{First Dirichlet prior parameter for component weights.}

\item{\code{alpha2}}{Second Dirichlet prior parameter for component weights.}

\item{\code{delta}}{Shape parameter used in prior updates.}

\item{\code{ggamma}}{Prior rate parameter used in covariance updates.}

\item{\code{bbeta}}{Prior scale parameter used in covariance updates.}
}}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgmm-main.R
\name{pgmm_rjmcmc}
\alias{pgmm_rjmcmc}
\title{Bayesian Model-Based Clustering with Parsimonious Gaussian Mixture Models}
\usage{
pgmm_rjmcmc(
  X,
  m_init,
  m_range,
  q_new,
  delta = 2,
  ggamma = 2,
  burn = 20,
  niter = 1000,
  constraint = c(0, 0, 0),
  d_vec = c(1, 1, 1),
  s_vec = c(1, 1, 1),
  m_step = 0,
  v_step = 0,
  split_combine = 0,
  verbose = TRUE
)
}
\arguments{
\item{X}{the observation matrix with variables in rows and observations in
columns.}

\item{m_init}{the number of initial clusters.}

\item{m_range}{the allowed range for the number of clusters.}

\item{q_new}{the number of latent factors for a new cluster.}

\item{delta}{scalar hyperparameter for the noise covariance prior}

\item{ggamma}{scalar hyperparameter used in covariance-structure proposals}

\item{burn}{the number of burn-in iterations}

\item{niter}{the number of posterior sampling iterations}

\item{constraint}{initial PGMM covariance constraint. Use a three-letter
model label such as `"CCC"` or `"UUU"`, or a numeric vector of length
three with binary entries. For example, `c(1, 1, 1)` is `CCC`, the fully
constrained model, and `c(0, 0, 0)` is `UUU`, the fully unconstrained
model.}

\item{d_vec}{a vector of hyperparameters with length three, shape parameters
for alpha1, alpha2 and bbeta respectively}

\item{s_vec}{a vector of hyperparameters with length three, rate parameters
for alpha1, alpha2 and bbeta respectively}

\item{m_step}{indicator for RJMCMC model selection on the number of clusters.}

\item{v_step}{indicator for RJMCMC model selection on covariance structures.}

\item{split_combine}{indicator for using split/combine moves in the
cluster-number RJMCMC step.}

\item{verbose}{logical; if `TRUE`, print iteration progress.}
}
\value{
A list of posterior samples with snake_case fields:
  `tau_samples`, `psi_samples`, `mean_samples`, `lambda_samples`,
  `factor_score_samples`, `allocation_samples`, `constraint_samples`,
  `alpha1_samples`, `alpha2_samples`, `beta_samples`, and
  `active_cluster_samples`.
}
\description{
Carries out model-based clustering using parsimonious Gaussian mixture
models. MCMC is used for parameter estimation and RJMCMC is used for model
selection.
}
\details{
The `constraint` argument follows the three-letter PGMM model notation used
in Lu, Li, and Love (2021). The first entry indicates whether loading
matrices are shared across clusters, the second whether noise covariance
matrices are shared across clusters, and the third whether the noise
covariance is isotropic within each cluster. Use [model_to_constraint()] to
convert model names such as `CCC`, `CCU`, `CUC`, `CUU`, `UCC`, `UCU`, `UUC`,
and `UUU` into the numeric vector used internally.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-theta-y.R
\docType{class}
\name{ThetaYList}
\alias{ThetaYList}
\title{ThetaYList-class}
\description{
Parameter set for sampled PGMM component parameters.
}
\section{Slots}{

\describe{
\item{\code{tao}}{Numeric vector of component mixing weights.}

\item{\code{psy}}{List of diagonal noise covariance matrices.}

\item{\code{M}}{List of component mean vectors.}

\item{\code{lambda}}{List of component factor loading matrices.}

\item{\code{Y}}{List of latent factor score matrices.}
}}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgmm-chains.R
\name{pgmm_rjmcmc_chains}
\alias{pgmm_rjmcmc_chains}
\title{Run Multiple Independent Bayesian PGMM Chains}
\usage{
pgmm_rjmcmc_chains(
  X,
  m_init,
  m_range,
  q_new,
  delta = 2,
  ggamma = 2,
  burn = 20,
  niter = 1000,
  constraint = c(0, 0, 0),
  d_vec = c(1, 1, 1),
  s_vec = c(1, 1, 1),
  m_step = 0,
  v_step = 0,
  split_combine = 0,
  verbose = FALSE,
  chains = 2,
  cores = min(chains, available_cores()),
  seed = NULL
)
}
\arguments{
\item{X}{the observation matrix with variables in rows and observations in
columns.}

\item{m_init}{the number of initial clusters.}

\item{m_range}{the allowed range for the number of clusters.}

\item{q_new}{the number of latent factors for a new cluster.}

\item{delta}{scalar hyperparameter for the noise covariance prior}

\item{ggamma}{scalar hyperparameter used in covariance-structure proposals}

\item{burn}{the number of burn-in iterations}

\item{niter}{the number of posterior sampling iterations}

\item{constraint}{initial PGMM covariance constraint. Use a three-letter
model label such as `"CCC"` or `"UUU"`, or a numeric vector of length
three with binary entries. For example, `c(1, 1, 1)` is `CCC`, the fully
constrained model, and `c(0, 0, 0)` is `UUU`, the fully unconstrained
model.}

\item{d_vec}{a vector of hyperparameters with length three, shape parameters
for alpha1, alpha2 and bbeta respectively}

\item{s_vec}{a vector of hyperparameters with length three, rate parameters
for alpha1, alpha2 and bbeta respectively}

\item{m_step}{indicator for RJMCMC model selection on the number of clusters.}

\item{v_step}{indicator for RJMCMC model selection on covariance structures.}

\item{split_combine}{indicator for using split/combine moves in the
cluster-number RJMCMC step.}

\item{verbose}{logical; if `TRUE`, print iteration progress.}

\item{chains}{positive integer giving the number of independent chains.}

\item{cores}{positive integer giving the number of worker processes to use.
Values greater than `chains` are reduced to `chains`.}

\item{seed}{optional integer seed used to generate deterministic per-chain
seeds.}
}
\value{
A list with one fitted `pgmm_rjmcmc()` result per chain. The result
  has class `bpgmm_rjmcmc_chains` and stores the per-chain seeds in the
  `chain_seeds` attribute.
}
\description{
Runs independent `pgmm_rjmcmc()` chains, optionally in parallel. This is the
safest way to use multiple CPU cores because each MCMC iteration depends on
the previous state, while independent chains can be evaluated concurrently.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constraints.R
\name{model_to_constraint}
\alias{model_to_constraint}
\title{Convert PGMM Paper Model Names to Constraint Codes}
\usage{
model_to_constraint(model)
}
\arguments{
\item{model}{Character scalar naming one of the eight PGMM covariance
structures: `CCC`, `CCU`, `CUC`, `CUU`, `UCC`, `UCU`, `UUC`, or `UUU`.}
}
\value{
Integer vector of length three. `1` means constrained and `0` means
  unconstrained.
}
\description{
Convert PGMM Paper Model Names to Constraint Codes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constraints.R
\name{constraint_to_model}
\alias{constraint_to_model}
\title{Convert PGMM Constraint Codes to Paper Model Names}
\usage{
constraint_to_model(constraint)
}
\arguments{
\item{constraint}{Integer or numeric vector of length three with entries
`0` or `1`. `1` maps to `C`; `0` maps to `U`.}
}
\value{
A character scalar, one of `CCC`, `CCU`, `CUC`, `CUU`, `UCC`,
  `UCU`, `UUC`, or `UUU`.
}
\description{
The paper represents the eight PGMM covariance structures with three-letter
model names. Each letter is either `C` for constrained or `U` for
unconstrained. The letters indicate whether the loading matrix is shared
across clusters, whether the noise covariance is shared across clusters, and
whether the noise covariance is isotropic within clusters.
}
